/*
 * @(#)JessDAML.java
 *
 * Ontology Graph API,  Copyright GRC International, Inc 2000, 2001
 *
 */
 package com.att.gs.codip.ograph.examples;

import java.util.Iterator;

import jess.Rete;
import com.att.gs.codip.ograph.DAML;
import com.att.gs.codip.ograph.OGOntology;
import com.att.gs.codip.ograph.OGUtility;
import com.att.gs.codip.ograph.OntologyManager;
import com.att.gs.codip.ograph.imp.OntologyManagerImp;

/**
 * Simple example illustrating the use of the DAML_API in Jess.
 * <p>
 * Usage: JessDAML [Ontologies]*
 * <p>
 * The Ontologies are spefcified by URI, supporting both
 * local files and http URLs. Any number of ontologies may
 * be specified.
 * <p>
 * Each ontology is is read, processed into a DAML Ontology instance
 * and then asserted into Jess.
 * <p>
 * After all ontologies have been processed,
 * a Jess command line processor is started. Jess
 * commands may now be entered. For example (facts) to show all
 * of the facts asserted based on the ontologies, and (exit) to
 * leave Jess.
 *
 **/
public class JessOG {

static OGOntology theDAMLCore;
static OntologyManager  theOntoManager;

static void bailOut() {
	System.out.println("Usage: java JessOG " +
			   "-[{iI} | -{fF} clipFile | -x | -X TransformURL ] [Ontologies]* ");
	System.out.println("Options");
	System.out.println(" -i or I - Use Instances Model");
	System.out.println(" -f or F - Load supplied clip.");
	System.out.println(" -x      - Apply the default  XSLT.");
	System.out.println(" -X      - Apply the supplied XSLT.");
	System.exit(1);
    }

public static void main(String[] args) throws Exception {
    // RDFFactory f;
    // Model      m;
    String loadClipFile ="OGJess.clp";

/*
 * Setup onto man.
 */
   theOntoManager = new OntologyManagerImp("JessOG");

   // Get the core daml ontology.
   theDAMLCore = theOntoManager.getDAML();
   // Add some aliases for it....while not semantically correct, it at least
   // will attempt to do something.
   theOntoManager.addAlias(DAML.DAML_URI_2000_10, theDAMLCore);
   theOntoManager.addAlias(DAML.DAML_URI_2000_12, theDAMLCore);

/*
 * Process the Arguments.
 */
    int idx = 0;
	int iFirstOnto = 0;
    boolean doInstances = false;

    while ((idx < args.length) &&
           (args[idx].startsWith("-"))) {
      if ("-i".equalsIgnoreCase(args[idx])) {
   	    // processing instances.
   	    doInstances = true;
   	    idx++;
   	    System.out.println("Using Instnaces.");
   	    }
	  else if ("-f".equalsIgnoreCase(args[idx])) {
		  // Base clp file to load.
		  idx++;
		  loadClipFile = args[idx];
		  idx++;
		  System.out.println("Loading from " + loadClipFile);
		  }
 	  else if ("-X".equals(args[idx])) {
   	    // Establish the supplied transformation.
   	    idx++;
   	    theOntoManager.setXSLT(args[idx]);
   	    idx++;
		System.out.println("Transforming with " + theOntoManager.getXSLT());
   	    }
	  else if ("-x".equals(args[idx])) {
   	    // Establish the default transformation.
   	    theOntoManager.setXSLT("preProcessOG.xsl");
   	    idx++;
		System.out.println("Transforming with default" + theOntoManager.getXSLT());
   	    }
   	  // unknown option or confussed.
      else {
      	System.out.println("Error in options.");
      	bailOut();
      	}
      } // while "-"
    if (idx < args.length) {
    	iFirstOnto = idx;
    	}
      else {// No ontologies supplied.
      	System.out.println("No ontologies supplied.");
    	bailOut();
    	}



/**
 * Jess stuff - Create and initialize Jess instance
 **/
    JessMainLoop jml = new JessMainLoop();
    jml.initialize(new Rete());
    Rete jmlRete = jml.getRete();
    // Put in the stored info before the rules are loaded!
    jmlRete.store("OM", theOntoManager);
    // Load the rules.
    jmlRete.executeCommand("(batch \""+loadClipFile+"\")");


/*
 * The leading arguments are the imported ontologies.
 */
    OGOntology theSrcOnto = null;
    for (int index =iFirstOnto; index < args.length; index++) {
      System.out.println("***********************************");
      System.out.println(" Parsing  Ontology " + args[index]);

      // Build from the  RDF
      theSrcOnto = theOntoManager.createOntology(args[index]);

      // Optional - Clean up the mess.
      OGUtility.fullGC();

      System.out.println(" Adding to Jess Engine ");
      System.out.println("***********************************");

      // Add to jess engine.
      try {
        if (doInstances)
          OGUtility.addJessInstances(jmlRete, theSrcOnto);
        else
          OGUtility.addJessOntotology(jmlRete, theSrcOnto);

        for (Iterator iter = theSrcOnto.imports(); iter.hasNext();) {
          if (doInstances)
            OGUtility.addJessInstances(jmlRete, (OGOntology)iter.next());
          else
            OGUtility.addJessOntotology(jmlRete, (OGOntology)iter.next());
          }
        }
      catch (Exception ex) {
        System.err.println(ex);
        }
      } // for


    System.out.println("***********************************");
    System.out.println(" Starting Jess Cmd Line ");
    System.out.println("***********************************");
/**
 * Jess sutff - commnand loop until (exit)
 **/
    jml.execute(true);

    } // main


} // JessOG
