/*
 * @(#)PROG2DAML.java
 *
 * Ontology Graph API,  Copyright GRC International, Inc 2000, 2001
 *
 * Builds an ontology using Ontology Graph API calls.
 *
 */
 package com.att.gs.codip.ograph.examples;

import java.util.Iterator;

import com.att.gs.codip.ograph.DAML;
import com.att.gs.codip.ograph.OGClass;
import com.att.gs.codip.ograph.OGInstance;
import com.att.gs.codip.ograph.OGLiteral;
import com.att.gs.codip.ograph.OGOntology;
import com.att.gs.codip.ograph.OGPropInstance;
import com.att.gs.codip.ograph.OGProperty;
import com.att.gs.codip.ograph.OGResource;
import com.att.gs.codip.ograph.OGUtility;
import com.att.gs.codip.ograph.OWL;
import com.att.gs.codip.ograph.OntologyManager;
import com.att.gs.codip.ograph.RDFS;
import com.att.gs.codip.ograph.XSD;
import com.att.gs.codip.ograph.imp.DAMLOntologyFactoryImp;
import com.att.gs.codip.ograph.imp.OGOntologyImp;
import com.att.gs.codip.ograph.imp.OntologyManagerImp;
import edu.unika.aifb.rdf.api.model.Model;
import edu.unika.aifb.rdf.api.syntax.RDFSerializer;
import edu.unika.aifb.rdf.api.util.RDFManager;

/**
 * Simple example illustrating the use of the DAML_API.
 *
 **/
public class PROG2DAML {

    static void bailOut() {
	System.out.println("Usage: java classes.utility.ograph.examples.newDAML " +
			   "[Ontology URI]* ");
	System.exit(1);
    }

public static void main(String[] args) throws Exception {

	if (args.length > 1) bailOut();

	String myURI = "urn:mem:foo";
        if (args.length >=  1) myURI = args[0];
        String myNS = myURI+"#";

	OGOntology coreOntology;
	OntologyManager  theOntoManager;

 	theOntoManager = new OntologyManagerImp("PROG2DAML");

   // Get the core daml ontology.
//   coreOntology = theOntoManager.getDAML();
   coreOntology = theOntoManager.getOWL();
   // Add some aliases for it....while not semantically correct, it at least
   // will attempt to do something.
   //theOntoManager.addAlias(DAML.DAML_URI_2000_10, coreOntology);
   //theOntoManager.addAlias(DAML.DAML_URI_2000_12, coreOntology);

   System.out.println("***********************************");
   System.out.println(" Testing imports.... ");
   System.out.println("***********************************");
   System.out.println("owl:Class " + coreOntology.lookupResource(OWL.OWL_NS + "Class"));
   System.out.println("rdf:type " + coreOntology.lookupResource(RDFS.RDF_NS + "type"));
   System.out.println("rdfs:Class " + coreOntology.lookupResource(RDFS.RDFS_NS + "Class"));
   System.out.println("xsd:double " + coreOntology.lookupResource(XSD.XSD_NS + "double"));

    System.out.println("***********************************");
    System.out.println(" Programatic build of "+myNS);
    System.out.println("***********************************");



        //OGOntology anOntology = new OGOntologyImp(myURI);
        OGOntology anOntology =theOntoManager.createOntology(myURI, false);

        anOntology.addImport(coreOntology);
        anOntology.setVersionInfo("This is a test, only a test, of PROG2DAML");

        OGClass classTop = anOntology.createClass(myNS+"classTop");
        classTop.addSubClassOf(OWL.Class);

        OGClass class1 = anOntology.createClass(myNS+"class1");
        class1.addSubClassOf(classTop);

        OGClass class2 = anOntology.createClass(myNS+"class2");
        class2.addSubClassOf(classTop);

        // Seperate class tree.
        OGClass class3 = anOntology.createClass(myNS+"class3");
        OGClass class4 = anOntology.createClass(myNS+"class4");

        OGProperty propObj = anOntology.createProperty(OWL.ObjectProperty,myNS+"myObjProp");
        // Class 1 to Class 2
/***
        OGProperty instPO12 = anOntology.createProperty(propObj); // anonymous.
        instPO12.setSubject(class1);
        class1.addProperty(instPO12);
        instPO12.setObject(class2);
***/

        // Class 3 to Class 4
        OGPropInstance instPO34 = anOntology.createPropInstance(propObj);
        instPO34.setSubject(class3);
        class3.addPropInstance(instPO34);
        instPO34.setObject(class4);

        OGProperty propDT = anOntology.createProperty(OWL.DatatypeProperty, myNS+"myDTProp");
        OGPropInstance instDT = anOntology.createPropInstance(propDT);


        instDT.addSubPropertyOf(OWL.DatatypeProperty);
        OGLiteral lit = anOntology.createLiteralByValue("57");

        instDT.setSubject(class2);
        class2.addPropInstance(instDT);
        instDT.setObject(lit);

	OGClass theRestriction = anOntology.createClass(OWL.Restriction);
	OGPropInstance onProperty = anOntology.createPropInstance(OWL.onProperty);
	onProperty.setSubject(theRestriction);
	onProperty.setObject(propDT);

	OGPropInstance toClass = anOntology.createPropInstance(OWL.allValuesFrom);
	toClass.setSubject(theRestriction);
	toClass.setObject(anOntology.lookupDatatype(XSD.XSD_NS+"string",true));

	theRestriction.addPropInstance(onProperty);
	theRestriction.addPropInstance(toClass);
	class2.addSubClassOf(theRestriction);

	//*******************************
	// Show the properties of class2
	//*******************************
    System.out.println("Class " + class2.getID());
    for (Iterator thePropInstances = class2.propInstances();
                  thePropInstances.hasNext();) {
         OGInstance anInstance = (OGInstance)thePropInstances.next();
         System.out.print(" has property "+ anInstance.getSubject() + " " +
                                              anInstance.getProperty() + " ");
         if (anInstance.getObject() instanceof OGLiteral)
           System.out.println(" L:"+ ((OGLiteral)anInstance.getObject()).getValue());
         else
           System.out.println(" R:"+ anInstance.getObject());
        }
	for (Iterator myClassRestrictions = class2.subClassOf(OWL.Restriction);
                  myClassRestrictions .hasNext();) {
    OGResource aResource = (OGResource)myClassRestrictions .next();
    System.out.println(" Restiction " + aResource.getID());
    for (Iterator thePropInstances = aResource.propInstances();
                  thePropInstances.hasNext();) {
         OGInstance anInstance = (OGInstance)thePropInstances.next();
         System.out.println("   "+ anInstance.getSubject() + " " +
                                   anInstance.getProperty() + " " +
                                   anInstance.getObject());
        }
    }


    System.out.println("***********************************");
    System.out.println(" DAML");
    System.out.println("***********************************");
       OGUtility.printOntotology(System.out, anOntology);


    System.out.println("***********************************");
    System.out.println(" RDF");
    System.out.println("***********************************");

    DAMLOntologyFactoryImp theOntoFactory = new DAMLOntologyFactoryImp();

    /** STAN **/
    //RDFFactory f = new RDFFactoryImpl();
    //Model m = anOntology.toRDFModel();
    //RDFUtil.dumpModel(m, System.out, f.createSerializer());
	/** KAON **/
	RDFSerializer s = RDFManager.createSerializer();
	Model theRDFModel;

	theRDFModel = theOntoFactory.toRDFModel(anOntology);

	s.addNamespacePrefix("daml",DAML.DAML_NS);
	s.addNamespacePrefix("xsd",XSD.XSD_NS);
	s.addNamespacePrefix("this",anOntology.getURI());
	s.serialize(theRDFModel, System.out, RDFSerializer.DEFAULT_ENCODING);


    System.out.println("***********************************");
    System.out.println(" Build SubOntology One");
    System.out.println("***********************************");

    // Copy a sub ontology tree...
    //
    // Currently must use the Imp!!!
    //
    OGOntologyImp aSubOntology = new OGOntologyImp(myURI+"-sub1");
    theOntoManager.addOntology(aSubOntology);

    // Use the method from the instance of the source ontology,
    // not the instance of the copy. Unless, you want the copy to
    // be empty.
    ((OGOntologyImp)anOntology).copySubOntology(class3,aSubOntology);
    OGUtility.printOntotology(System.out, aSubOntology);
    //
    // Create as string.
    //
    System.out.println("***********************************");
    System.out.println(" RDF String ");
    System.out.println("***********************************");
    /** STAN **/
    //m = aSubOntology.toRDFModel();
	//System.out.println(RDFUtil.dumpModel(m, f.createSerializer()));
	/** KAON **/
	theRDFModel = theOntoFactory.toRDFModel(aSubOntology);
	s.addNamespacePrefix("this",aSubOntology.getURI());
	s.serialize(theRDFModel, System.out, RDFSerializer.DEFAULT_ENCODING);


    System.out.println("***********************************");
    System.out.println(" Build SubOntology Two");
    System.out.println("***********************************");

    // Copy a sub ontology tree...
    //
    // Currently must use the Imp!!!
    //
    OGOntologyImp oSubOntology = new OGOntologyImp(myURI+"-sub2");
    theOntoManager.addOntology(oSubOntology);

    // Use the method from the instance of the source ontology,
    // not the instance of the copy. Unless, you want the copy to
    // be empty.
    ((OGOntologyImp)anOntology).copySubOntology(class2,oSubOntology);
    OGUtility.printOntotology(System.out, oSubOntology);
    //
    // Create as string.
    //
    System.out.println("***********************************");
    System.out.println(" RDF String ");
    System.out.println("***********************************");
    /** STAN **/
    //m = oSubOntology.toRDFModel();
	// System.out.println(RDFUtil.dumpModel(m, f.createSerializer()));
	/* KAON */
	theRDFModel = theOntoFactory.toRDFModel(oSubOntology);
	s.addNamespacePrefix("this",oSubOntology.getURI());
	s.serialize(theRDFModel, System.out, RDFSerializer.DEFAULT_ENCODING);

    System.out.println("***********************************");
    System.out.println(" Remove Resources from SubOnto One");
    System.out.println("***********************************");
    aSubOntology.removeClass(class3);
    OGUtility.printOntotology(System.out, aSubOntology);

    } // main


} // PROG2DAML
