/*
 * @(#)RDF2DAY.java
 *
 * Ontology Graph API,  Copyright GRC International, Inc 2000, 2001
 *
 */
 package com.att.gs.codip.ograph.examples;

import java.io.File;
import java.io.FileOutputStream;
import java.io.PrintStream;
import java.util.Iterator;

import com.att.gs.codip.ograph.DAML;
import com.att.gs.codip.ograph.OGLiteral;
import com.att.gs.codip.ograph.OGOntology;
import com.att.gs.codip.ograph.OGPropInstance;
import com.att.gs.codip.ograph.OGProperty;
import com.att.gs.codip.ograph.OGResource;
import com.att.gs.codip.ograph.OGUtility;
import com.att.gs.codip.ograph.OntologyManager;
import com.att.gs.codip.ograph.imp.OntologyManagerImp;
import com.att.gs.codip.resource.namespaceResolver;

/**
 * Simple example illustrating the use of OGraph.
 * <p>
 * Usage: RDF2DAY [-{xX} [Transform]] Ontology
 * <p>
 * If the x or X option is given, an XSL transform will be
 * applied to the Ontology before further processing. If no
 * transform file is specified, then an attempt will be made to
 * use the default transform file 'preProcessOG.xsl'
 * <p>
 * Only a single ontology may be given. However, all imported
 * ontologies will also be processed as needed.
 * <p>
 * The Transform and Ontology are given as URIs, supporting
 * local files and http URLs.
 * <p>
 * err.ut a Daytona compatible format.
 *
 **/
public class RDF2DAY{

static OGOntology theDAMLCore;
static OntologyManager  theOntoManager;

    static void bailout() {
	System.out.println("Usage: java RDF2DAY " +
			   "[-X | -x Transform] Ontology");
	System.out.println("");
	System.out.println(" Will append ontology data to the file './day_stmnts'");
	System.out.println("   move away exisitng file to create new.");
	System.out.println(" Will over write namespace data to the file './day_ns'");
	System.out.println("   move away exisitng file to save old.");

	System.exit(1);
    }

public static void main(String[] args) throws Exception {

    if((args.length < 1) || (args.length > 3))
        bailout();

/*
 * First create an instance of the core DAML ontology.
 */
   System.out.println("***********************************");
   System.out.println(" Setting up.... ");
   System.out.println("***********************************");

 	theOntoManager = new OntologyManagerImp("RDF2DAY");


   // Get the core daml ontology.
   theDAMLCore = theOntoManager.getDAML();
   // Add some aliases for it....while not semantically correct, it at least
   // will attempt to do something.
   theOntoManager.addAlias(DAML.DAML_URI_2000_10, theDAMLCore);
   theOntoManager.addAlias(DAML.DAML_URI_2000_12, theDAMLCore);

/*
 * Process the Arguments.
 */

 	if (args.length == 3) {
		if (!"-x".equalsIgnoreCase(args[0])) bailout();
   	// Establish the default transformation.
   	theOntoManager.setXSLT(args[1]);
   	}

 	if (args.length == 2) {
		if (!"-x".equalsIgnoreCase(args[0])) bailout();
   	// Establish the default transformation.
   	theOntoManager.setXSLT("preProcessOG.xsl");
   	}

  //System.out.println("***********************************");
  //System.out.println(" The DAML+OIL Core");
  //System.out.println("***********************************");
  //OGUtility.printOntotology(System.out. theDAMLCore);

  System.out.println("***********************************");
  System.out.println(" Parsing Ontology " + args[args.length-1]);
  System.out.println("***********************************");


  // Build from the  RDF
  OGOntology theSrcOnto = theOntoManager.createOntology(args[args.length-1]);

  // Optional - Clean up the mess.
  OGUtility.fullGC();


  System.out.println("***********************************");
  System.out.println(" Loaded DAML");
  System.out.println("***********************************");

  // Echo to System.out
  //OGUtility.printOntotology(System.out. theSrcOnto);

  System.out.println("***********************************");
  System.out.println(" Generating Namespace File - 'day_ns'");
  System.out.println("***********************************");

  namespaceResolver aNSR  = new namespaceResolver(theSrcOnto.getURI());
  // Additional Common namespaces.
  aNSR.putNamespace("cyc:", "http://www.cyc.com/2002/04/08/cyc#");
  aNSR.putNamespace("wnet:","http://xmlns.com/wordnet/1.6#");
  aNSR.putNamespace("wnc:","http://www.cogsci.princeton.edu/~wn/concept#");
  aNSR.putNamespace("wns:","http://www.cogsci.princeton.edu/~wn/schema#");
  aNSR.putNamespace("dc:",  "http://purl.org/dc/elements/1.1#");
  aNSR.putNamespace("sumo:","http://teknowledge.com/ontology/SUMO.daml#");

  File F = new File("./day_ns");
  PrintStream nsfile = new PrintStream(new FileOutputStream(F));

  // Wirte header.
  nsfile.println("%msg)flds)local_alias|logical_uri");
  // Write namespace records.
  nsfile.print(aNSR.getRecords());
  // Close'm up.
  nsfile.close();

  System.out.println("***********************************");
  System.out.println(" Statement File Complete.");
  System.out.println("***********************************");

  System.out.println("***********************************");
  System.out.println(" Generating Statements File - 'day_stmnts'");
  System.out.println("***********************************");

  String aLiteral = "";


  F = new File("./day_stmnts");
  boolean oldDayFile  = F.exists();
  PrintStream dayfile = new PrintStream(new FileOutputStream(F,true));

  // Wirte header only if a new file.
  if (!oldDayFile) dayfile.println("%msg)flds)sub_nsa|subject|prp_nsa|property|obj_nsa|object");
  for (Iterator itr=theSrcOnto.properties(); itr.hasNext();) {
    OGProperty p = (OGProperty)itr.next();
    if (p.pInstance()) {
      OGPropInstance i = (OGPropInstance)p;
      dayfile.print(aNSR.getPrefix(i.getSubject().getNamespace())+'|');
      dayfile.print(i.getSubject().getLocalName()+'|');
      dayfile.print(aNSR.getPrefix(i.getType().getNamespace())+'|');
      dayfile.print(i.getType().getLocalName()+'|');
      OGResource o = i.getObject();
      if (o instanceof OGLiteral) {
        dayfile.print('|');
        aLiteral = ((OGLiteral)o).getValue().replace('\n',' ');
		if (aLiteral.indexOf('|') >= 0) aLiteral.replace('|','!');
		if (aLiteral.indexOf('[') >= 0) aLiteral.replace('[','(');
		if (aLiteral.indexOf(']') >= 0) aLiteral.replace(']',')');
		if (aLiteral.indexOf('%') >= 0) aLiteral.replace('%','~');
        dayfile.println(aLiteral);
        }
      else if (o == null) {
        dayfile.println('|');
        }
      else {
        dayfile.print(aNSR.getPrefix(o.getNamespace())+'|');
        dayfile.println(o.getLocalName());
        }
      }
    }
    dayfile.close();
  System.out.println("***********************************");
  System.out.println(" Statement File Complete.");
  System.out.println("***********************************");


  } // main

} // RDF2DAY
