/*
 * @(#)RDF2OGRPH.java
 *
 * Ontology Graph API,  Copyright GRC International, Inc 2000, 2001
 *
 */
 package com.att.gs.codip.ograph.examples;

/** KAON RDF API **/
import com.att.gs.codip.ograph.DAML;
import com.att.gs.codip.ograph.OGOntology;
import com.att.gs.codip.ograph.OGUtility;
import com.att.gs.codip.ograph.OntologyFactory;
import com.att.gs.codip.ograph.OntologyManager;
import com.att.gs.codip.ograph.XSD;
import com.att.gs.codip.ograph.imp.OWLOntologyFactoryImp;
import com.att.gs.codip.ograph.imp.OntologyManagerImp;
import edu.unika.aifb.rdf.api.model.Model;
import edu.unika.aifb.rdf.api.model.ModelException;
import edu.unika.aifb.rdf.api.syntax.RDFSerializer;
import edu.unika.aifb.rdf.api.util.ModelContext;
/**
 * Simple example illustrating the use of OGraph.
 * <p>
 * Usage: RDF2OGRPH [-{xX} [Transform]] Ontology
 * <p>
 * If the x or X option is given, an XSL transform will be
 * applied to the Ontology before further processing. If no
 * transform file is specified, then an attempt will be made to
 * use the default transform file 'preProcessOG.xsl'
 * <p>
 * Only a single ontology may be given. However, all imported
 * ontologies will also be processed as needed.
 * <p>
 * The Transform and Ontology are given as URIs, supporting
 * local files and http URLs.
 *
 **/
public class RDF2OGRPH{

static OGOntology theDAMLCore;
static OGOntology theOWLCore;
static OntologyManager  theOntoManager;

    static void bailOut() {
	System.out.println("Usage: java RDF2OGRPH " +
			   "[-{xX} [Transform]] Ontology" +
			   "\n\n -x applies transform only to source." +
			   "\n -X applies transform to source and imports.");
	System.exit(1);
    }

public static void main(String[] args) throws Exception {
/** STAN */
//     RDFFactory f;
     Model      m;

    if((args.length < 1) || (args.length > 3))
        bailOut();

/*
 * First create an instance of the core DAML ontology.
 */
   System.out.println("***********************************");
   System.out.println(" Setting up.... ");
   System.out.println("***********************************");

 	theOntoManager = new OntologyManagerImp("RDF2OGRPH");

   // Get the core daml ontology.
   theDAMLCore = theOntoManager.getDAML();
   // Add some aliases for it....while not semantically correct, it at least
   // will attempt to do something.
   theOntoManager.addAlias(DAML.DAML_URI_2000_10, theDAMLCore);
   theOntoManager.addAlias(DAML.DAML_URI_2000_12, theDAMLCore);

   // Get the core owl ontology.
   theOWLCore = theOntoManager.getOWL();

/*
 * Process the Arguments.
 */

 	if (args.length == 3) {
		if (!"-x".equalsIgnoreCase(args[0])) bailOut();
   	// Establish the default transformation.
   	theOntoManager.setXSLT(args[1]);
   	}

 	if (args.length == 2) {
		if (!"-x".equalsIgnoreCase(args[0])) bailOut();
   	// Establish the default transformation.
   	theOntoManager.setXSLT("preProcessOG.xsl");
   	}
   theOntoManager.setApplyToImports("-X".equals(args[0]));

  //System.out.println("***********************************");
  //System.out.println(" The DAML+OIL Core");
  //System.out.println("***********************************");
  //OGUtility.printOntotology(System.out, theDAMLCore);

  System.out.println("***********************************");
  System.out.println(" Parsing Ontology " + args[args.length-1]);
  System.out.println("***********************************");

  // Build from the  RDF
  OGOntology theSrcOnto = theOntoManager.createOntology(args[args.length-1]);

  // Optional - Clean up the mess.
  OGUtility.fullGC();


  System.out.println("***********************************");
  System.out.println(" DAML");
  System.out.println("***********************************");

  // Echo to System.out
  //OGUtility.printOntotology(System.out, theSrcOnto);

  System.out.println("***********************************");
  System.out.println(" Source RDF");
  System.out.println("***********************************");
	/** STAN **/
  // Echo to System.out
  //f = new RDFFactoryImpl();
  //m = f.createModel();
  // Parse the RDF.
  //RDFUtil.parse(args[args.length-1], f.createParser(), m);
  // and dump backout...
  //RDFUtil.dumpModel(m, System.out, f.createSerializer());
   /** KAON **/
	RDFSerializer s = new RDFSerializer();
	s.addNamespacePrefix("daml",DAML.DAML_NS);
	s.addNamespacePrefix("xsd",XSD.XSD_NS);
	s.addNamespacePrefix("this",theSrcOnto.getURI());

	Model theRDFModel;
	try {
		ModelContext ctx = new ModelContext();
		theRDFModel = ctx.getModelPhysical(theSrcOnto.getURI());
		//s.serialize(theRDFModel, System.out, RDFSerializer.DEFAULT_ENCODING);
		}
	catch (ModelException MoEx) {
	  System.err.println(" Exception in creating model.");
      System.err.println("Message:"+ MoEx.getLocalizedMessage());
	  System.err.println(MoEx);
	  }

/*
 * Now rebuild the RDF.
 */
  System.out.println("***********************************");
  System.out.println(" Rebuilt RDF");
  System.out.println("***********************************");

//  OGOntologyFactory theOntoFactory = new DAMLOntologyFactoryImp();
  OntologyFactory theOntoFactory = new OWLOntologyFactoryImp();
  theRDFModel = theOntoFactory.toRDFModel(theSrcOnto);
  s.serialize(theRDFModel, System.out, RDFSerializer.DEFAULT_ENCODING);

  } // main

} // RDF2OGRPH
