<?xml version="1.0" ?>
<xsl:stylesheet 
	xmlns:xsl ="http://www.w3.org/1999/XSL/Transform" 
	
	xmlns:daml ="http://www.daml.org/2001/03/daml+oil#"
	xmlns:daml2="http://www.daml.org/2000/12/daml+oil#"
	xmlns:daml1="http://www.daml.org/2000/10/daml-ont#"

	xmlns:rdf ="http://www.w3.org/1999/02/22-rdf-syntax-ns#"
	xmlns:web ="http://www.w3.org/1999/02/22-rdf-syntax-ns#" 
	xmlns:rdfs="http://www.w3.org/2000/01/rdf-schema#"

	xmlns:owl="http://www.w3.org/2002/07/owl#"
	
   exclude-result-prefixes="daml daml1 daml2 rdf rdfs" 
	version="2.1">

<xsl:output method="xml" indent="yes" />

<!-- Variables for the prototype WordNet NS server translation. -->
<xsl:variable name="WNET">http://xmlns.com/wordnet/1.6/</xsl:variable>
<xsl:variable name="WNNS">http://xmlns.com/wordnet/1.6#</xsl:variable>

<!-- Element matches -->
<xsl:template match="	rdfs:subPropertyOf |
						rdfs:Literal |
						rdf:value |
						rdfs:subClassOf |
						rdfs:Class ">
	<xsl:element name="owl:{local-name()}">
		<xsl:apply-templates select = "node()|@*"/>
	</xsl:element>
</xsl:template>

<!-- Attribute matches -->
<xsl:template match="	@rdfs:subPropertyOf |
						@rdfs:Literal |
						@rdf:value |
						@rdfs:subClassOf |
						@rdfs:Class">
	<xsl:attribute name="owl:{local-name()}">
		<xsl:apply-templates select = "node()|@*"/>
	</xsl:attribute >
</xsl:template>

<!-- Rebuild collecitons -->
<xsl:template match="*[@rdf:parseType='Collection']">
	<xsl:copy>
		<xsl:apply-templates select="child::*[1]" mode="inList"/>
	</xsl:copy>
</xsl:template>

<xsl:template match="*[parent::*[@rdf:parseType='Collection']]" mode="inList">
	<xsl:if test="position()=1">
	<rdf:List>
		<rdf:first> 
			<xsl:apply-templates select="."/>
		</rdf:first>
		      <xsl:if test="following-sibling::*">
				<rdf:rest>
					<xsl:apply-templates select="following-sibling::*" mode="inList"/>
				</rdf:rest>
			</xsl:if>
	</rdf:List>
	</xsl:if>
</xsl:template>


<!--Change DAML Elements to new namespace-->
<xsl:template match="daml2:*|daml1:*">
	<xsl:element name="owl:{local-name()}">
		<xsl:apply-templates select = "node()|@*"/>
	</xsl:element>
</xsl:template>

<xsl:template match="@daml2:*|@daml1:*">
	<xsl:attribute name="owl:{local-name()}">
		<xsl:apply-templates select = "node()|@*"/>
	</xsl:attribute>
</xsl:template>

<xsl:template match="@*">
	<xsl:choose>
		<xsl:when test="starts-with(string(.),$WNET)">
			<xsl:attribute name="{name(.)}">
				<xsl:value-of select="concat($WNNS,substring-after(string(.),$WNET))"/>
			</xsl:attribute>
		</xsl:when>
		<xsl:otherwise>
			<xsl:copy/>
		</xsl:otherwise>
	</xsl:choose>
</xsl:template>

<xsl:template match="node()">
	<xsl:copy>
		<xsl:apply-templates select="@* | node()"/>
	</xsl:copy>
</xsl:template>

<xsl:template match="text()">
	<xsl:copy></xsl:copy>
</xsl:template>

</xsl:stylesheet>

