/*
 * @(#)DAML.java
 *
 * Ontology Graph API,  Copyright 2000, 2001 GRCI All Rights Reserved.
 *
 */
package com.att.gs.codip.ograph;

import java.lang.reflect.Constructor;

/** **************************
 * An implementaiton of the OGOntology Interface that provides
 * static instances of the core DAML+OIL Classes and Properties.
 *
 ************************** **/

public class DAML extends OGBaseOntology implements OGOntology {
//
// Built in URIs, Classes and Properties, with initialization.
//
/**
 * The URI of the RDF syntax specificaiton.
 **/
public static final String RDF_URI  = "http://www.w3.org/1999/02/22-rdf-syntax-ns";
/**
 * The Namespace of the RDF syntax specificaiton.
 **/
public static final String RDF_NS   = RDF_URI+"#";
/**
 * The URI of the RDFS syntax specificaiton.
 **/
public static final String RDFS_URI  = "http://www.w3.org/2000/01/rdf-schema";
/**
 * The Namespace of the RDFS syntax specificaiton.
 **/
public static final String RDFS_NS   = RDFS_URI+"#";

/**
 * The URI of the DAML syntax specificaiton.
 **/
public static final String DAML_URI_2000_10 = "http://www.daml.org/2000/10/daml-ont";
public static final String DAML_URI_2000_12 = "http://www.daml.org/2000/12/daml+oil";
public static final String DAML_URI = "http://www.daml.org/2001/03/daml+oil";
/**
 * The Namespace of the DAML syntax specificaiton.
 **/
public static final String DAML_NS  = DAML_URI+"#";

public static OGOntology Ontology;

public static OGClass Thing;

public static OGClass Nothing;

public static OGDatatype Datatype;

public static OGClass Class;
public static OGClass Literal;
public static OGProperty Property;
public static OGProperty ObjectProperty;
public static OGProperty DatatypeProperty;

public static OGClass Disjoint;
public static OGClass Restriction;
// Extension to DAML.
public static OGProperty RestrictionP;

public static OGProperty UniqueProperty;
public static OGProperty TransitiveProperty;
public static OGProperty UnambiguousProperty;
public static OGClass List;

public static OGProperty equivalentTo;
public static OGProperty sameClassAs;
public static OGProperty samePropertyAs;
public static OGProperty sameIndividualAs;
public static OGProperty differentIndividualFrom;
public static OGProperty disjointWith;
public static OGProperty unionOf;
public static OGProperty disjointUnionOf;
public static OGProperty intersectionOf;
public static OGProperty complementOf;
public static OGProperty oneOf;
public static OGProperty onProperty;
public static OGProperty toClass;
public static OGProperty hasValue;
public static OGProperty hasClass;
public static OGProperty cardinality;
public static OGProperty hasClassQ;
public static OGProperty minCardinality;
public static OGProperty maxCardinality;
public static OGProperty minCardinalityQ;
public static OGProperty maxCardinalityQ;
public static OGProperty cardinalityQ;
public static OGProperty inverseOf;
public static OGProperty first;
public static OGProperty rest;
public static OGProperty item;
public static OGProperty versionInfo;
public static OGProperty imports;
public static OGProperty type;
public static OGProperty value;
public static OGProperty label;
public static OGProperty comment;
public static OGProperty domain;
public static OGProperty range;
public static OGProperty subClassOf;
public static OGProperty subPropertyOf;
public static OGProperty seeAlso;
public static OGProperty isDefinedBy;
/**
 * RDF/S type included in DAML.
 **/
public static OGProperty RDF_type;
public static OGClass    RDFS_Class;

static {
    }

//
// Creators and initializors.
//

/**
 * Creates an instance of DAML using the passed in OGOntology.
 * @parms anOntoImpl An instance of a OGOntology to use.
 * @serial
 */
public DAML(OGOntology anOntoImpl) {
    myImpl = anOntoImpl;
    initialize();
    }

/**
 * Creates the static classes.
 *
 * @serial
 */
public void initialize() {
	// Create an instance of the implementation type.
	try {
	  Object[] ConsVals = new Object[1];
	  ConsVals[0] = new String(DAML.DAML_NS + "Ontology");
	  Class[] ConsTypes = new Class[1];
	  ConsTypes[0] = ConsVals[0].getClass();
	  Constructor theCons = myImpl.getClass().getConstructor(ConsTypes);
	  DAML.Ontology = (OGOntology)theCons.newInstance(ConsVals);
	  DAML.Ontology.setComment("DAML");
	  DAML.Ontology.setType(DAML.Ontology);
	  // Finish setup of this.
	  this.addResource(DAML.Ontology);
	  this.setType(DAML.Ontology);
	  }
	catch (Exception ex) {
		System.err.println("Constructor Failed in DAML.");
		ex.printStackTrace(System.err);
		}

    // The root of all resources.
    // Class needs to be initialized first, is is used as the
    // type of classes below.
    DAML.Class = myImpl.lookupClass(DAML.DAML_NS + "Class", true);
    DAML.Class.setComment("DAML+OIL Core");
    DAML.Class.setType(DAML.Class); // Need since it is null in the creator

    DAML.Thing = myImpl.lookupClass(DAML.DAML_NS + "Thing", true);
    DAML.Thing.setComment("DAML+OIL Core");

    DAML.Nothing = myImpl.lookupClass(DAML.DAML_NS + "Nothing", true);
    DAML.Nothing.setComment("DAML+OIL Core");

    DAML.Literal = myImpl.lookupClass(DAML.DAML_NS + "Literal", true);
    DAML.Literal.setComment("DAML+OIL Core");

    DAML.Datatype = myImpl.lookupDatatype(DAML.DAML_NS + "Datatype", true);
    DAML.Datatype.setComment("DAML+OIL Core");

    DAML.Disjoint = myImpl.lookupClass(DAML.DAML_NS + "Disjoint", true);
    DAML.Disjoint.setComment("DAML+OIL Core");

    DAML.Restriction = myImpl.lookupClass(DAML.DAML_NS + "Restriction", true);
    DAML.Restriction.setComment("DAML+OIL Core");

    DAML.RestrictionP = myImpl.lookupProperty(DAML.DAML_NS + "RestrictionP", true);
    DAML.RestrictionP.setComment("DAML+OIL Extension");

    DAML.List = myImpl.lookupClass(DAML.DAML_NS + "List", true);
    DAML.List.setComment("DAML+OIL Core");

    // Properties
    DAML.Property = myImpl.lookupProperty(DAML.DAML_NS + "Property", true);
    DAML.Property.setComment("DAML+OIL Core");
    DAML.Property.setType(DAML.Property); // Need since it is null in the creator

    DAML.subPropertyOf = myImpl.lookupProperty(DAML.DAML_NS + "subPropertyOf", true);
    DAML.subPropertyOf.setComment("DAML+OIL Core");

    DAML.ObjectProperty = myImpl.lookupProperty(DAML.DAML_NS + "ObjectProperty", true);
    DAML.ObjectProperty.setComment("DAML+OIL Core");
    DAML.ObjectProperty.addSubPropertyOf(DAML.Property);

    DAML.DatatypeProperty = myImpl.lookupProperty(DAML.DAML_NS + "DatatypeProperty", true);
    DAML.DatatypeProperty.setComment("DAML+OIL Core");
    DAML.DatatypeProperty.addSubPropertyOf(DAML.Property);

    DAML.UniqueProperty = myImpl.lookupProperty(DAML.DAML_NS + "UniqueProperty", true);
    DAML.UniqueProperty.setComment("DAML+OIL Core");
    DAML.UniqueProperty.addSubPropertyOf(DAML.Property);

    DAML.TransitiveProperty = myImpl.lookupProperty(DAML.DAML_NS + "TransitiveProperty", true);
    DAML.TransitiveProperty.setComment("DAML+OIL Core");
    DAML.TransitiveProperty.addSubPropertyOf(DAML.ObjectProperty);

    DAML.UnambiguousProperty = myImpl.lookupProperty(DAML.DAML_NS + "UnambiguousProperty", true);
    DAML.UnambiguousProperty.setComment("DAML+OIL Core");
    DAML.UnambiguousProperty.addSubPropertyOf(DAML.ObjectProperty);

    DAML.equivalentTo = myImpl.lookupProperty(DAML.DAML_NS + "equivalentTo", true);
    DAML.equivalentTo.setComment("DAML+OIL Core");

    DAML.minCardinality = myImpl.lookupProperty(DAML.DAML_NS + "minCardinality", true);
    DAML.minCardinality.setComment("DAML+OIL Core");

    DAML.maxCardinality = myImpl.lookupProperty(DAML.DAML_NS + "maxCardinality", true);
    DAML.maxCardinality.setComment("DAML+OIL Core");

    DAML.sameClassAs = myImpl.lookupProperty(DAML.DAML_NS + "sameClassAs", true);
    DAML.sameClassAs.setComment("DAML+OIL Core");

    DAML.samePropertyAs = myImpl.lookupProperty(DAML.DAML_NS + "samePropertyAs", true);
    DAML.samePropertyAs.setComment("DAML+OIL Core");

    DAML.sameIndividualAs = myImpl.lookupProperty(DAML.DAML_NS + "sameIndividualAs", true);
    DAML.sameIndividualAs.setComment("DAML+OIL Core");

    DAML.differentIndividualFrom = myImpl.lookupProperty(DAML.DAML_NS + "differentIndividualFrom", true);
    DAML.differentIndividualFrom.setComment("DAML+OIL Core");

    DAML.disjointWith = myImpl.lookupProperty(DAML.DAML_NS + "disjointWith", true);
    DAML.disjointWith.setComment("DAML+OIL Core");

    DAML.unionOf = myImpl.lookupProperty(DAML.DAML_NS + "unionOf", true);
    DAML.unionOf.setComment("DAML+OIL Core");

    DAML.disjointUnionOf = myImpl.lookupProperty(DAML.DAML_NS + "disjointUnionOf", true);
    DAML.disjointUnionOf.setComment("DAML+OIL Core");

    DAML.intersectionOf = myImpl.lookupProperty(DAML.DAML_NS + "intersectionOf", true);
    DAML.intersectionOf.setComment("DAML+OIL Core");

    DAML.complementOf = myImpl.lookupProperty(DAML.DAML_NS + "complementOf", true);
    DAML.complementOf.setComment("DAML+OIL Core");

    DAML.oneOf = myImpl.lookupProperty(DAML.DAML_NS + "oneOf", true);
    DAML.oneOf.setComment("DAML+OIL Core");

    DAML.onProperty = myImpl.lookupProperty(DAML.DAML_NS + "onProperty", true);
    DAML.onProperty.setComment("DAML+OIL Core");

    DAML.toClass = myImpl.lookupProperty(DAML.DAML_NS + "toClass", true);
    DAML.toClass.setComment("DAML+OIL Core");

    DAML.hasValue = myImpl.lookupProperty(DAML.DAML_NS + "hasValue", true);
    DAML.hasValue.setComment("DAML+OIL Core");

    DAML.hasClass = myImpl.lookupProperty(DAML.DAML_NS + "hasClass", true);
    DAML.hasClass.setComment("DAML+OIL Core");

    DAML.cardinality = myImpl.lookupProperty(DAML.DAML_NS + "cardinality", true);
    DAML.cardinality.setComment("DAML+OIL Core");

    DAML.hasClassQ = myImpl.lookupProperty(DAML.DAML_NS + "hasClassQ", true);
    DAML.hasClassQ.setComment("DAML+OIL Core");

    DAML.minCardinalityQ = myImpl.lookupProperty(DAML.DAML_NS + "minCardinalityQ", true);
    DAML.minCardinalityQ.setComment("DAML+OIL Core");

    DAML.maxCardinalityQ = myImpl.lookupProperty(DAML.DAML_NS + "maxCardinalityQ", true);
    DAML.maxCardinalityQ.setComment("DAML+OIL Core");

    DAML.cardinalityQ = myImpl.lookupProperty(DAML.DAML_NS + "cardinalityQ", true);
    DAML.cardinalityQ.setComment("DAML+OIL Core");

    DAML.inverseOf = myImpl.lookupProperty(DAML.DAML_NS + "inverseOf", true);
   DAML.inverseOf.setComment("DAML+OIL Core");

    DAML.first = myImpl.lookupProperty(DAML.DAML_NS + "first", true);
    DAML.first.setComment("DAML+OIL Core");

    DAML.rest = myImpl.lookupProperty(DAML.DAML_NS + "rest", true);
    DAML.rest.setComment("DAML+OIL Core");

    DAML.item = myImpl.lookupProperty(DAML.DAML_NS + "item", true);
    DAML.item.setComment("DAML+OIL Core");

    DAML.versionInfo = myImpl.lookupProperty(DAML.DAML_NS + "versionInfo", true);
    DAML.versionInfo.setComment("DAML+OIL Core");

    DAML.imports = myImpl.lookupProperty(DAML.DAML_NS + "imports", true);
    DAML.imports.setComment("DAML+OIL Core");

    DAML.type = myImpl.lookupProperty(DAML.DAML_NS + "type", true);
    DAML.type.setComment("DAML+OIL Core");

    DAML.value = myImpl.lookupProperty(DAML.DAML_NS + "value", true);
    DAML.value.setComment("DAML+OIL Core");

    DAML.domain = myImpl.lookupProperty(DAML.DAML_NS + "domain", true);
    DAML.domain.setComment("DAML+OIL Core");

    DAML.range = myImpl.lookupProperty(DAML.DAML_NS + "range", true);
    DAML.range.setComment("DAML+OIL Core");

    DAML.subClassOf = myImpl.lookupProperty(DAML.DAML_NS + "subClassOf", true);
    DAML.subClassOf.setComment("DAML+OIL Core");

    DAML.label = myImpl.lookupProperty(DAML.DAML_NS + "label", true);
    DAML.label.setComment("DAML+OIL Core");

    DAML.comment = myImpl.lookupProperty(DAML.DAML_NS + "comment", true);
    DAML.comment.setComment("DAML+OIL Core");

    DAML.seeAlso = myImpl.lookupProperty(DAML.DAML_NS + "seeAlso", true);
    DAML.seeAlso.setComment("DAML+OIL Core");

    DAML.isDefinedBy = myImpl.lookupProperty(DAML.DAML_NS + "isDefinedBy", true);
    DAML.isDefinedBy.setComment("DAML+OIL Core");
    DAML.isDefinedBy.addSubPropertyOf(DAML.seeAlso);

    DAML.RDF_type = myImpl.lookupProperty(DAML.RDF_NS + "type", true);
    DAML.RDF_type.setComment("RDF");

    DAML.RDFS_Class = myImpl.lookupClass(DAML.RDFS_NS + "Class", true);
    DAML.RDFS_Class.setComment("RDFS");
    }

/***********************************************
 * Most DAML Ontology Methods Inherited from 
 * OGBaseOntology. Only overidded ones here.
 *********************************************/
public void morph(String URI) {
	// Can't morph this.
	}

   } // DAML
