/*
 * @(#)OWL.java
 *
 * Ontology Graph API
 * Copyright 2003 AT&T Govvernment Solution, Inc.
 * All Rights Reserved.
 *
 */
package com.att.gs.codip.ograph;

import java.util.Collection;
import java.util.Iterator;

import com.att.gs.codip.resource.LogicalResource;

/** **************************
 * An implementaiton of the OGOntology Interface that provides
 * static instances of the OWL Classes and Properties.
 *
 ************************** **/

public class OGBaseOntology implements OGOntology  {
//
// Built in URIs, Classes and Properties, with initialization.
//

//
// Creators and initializors.
//
/**
 * The internal Ontology implementation.
 * @serial
 */
protected OGOntology myImpl;

/************************************************
 * Base Ontology Methods.
 ************************************************/
public void morph(String URI) {
	// Can't morph this.
	}

public OGOntology getOntology (){
    return myImpl.getOntology();
   }

public String getNamespace (){
    return myImpl.getNamespace();
   }

public String getLocalName (){
    return myImpl.getLocalName();
   }

public void setVersionInfo(String theVersionInfo){
    myImpl.setVersionInfo(theVersionInfo);
    }

public String getVersionInfo (){
    return myImpl.getVersionInfo();
    }

public String getID(){
    return myImpl.getID();
    }

public String getURI(){
    return myImpl.getURI();
    }

public void setSourceURI(String theSourceURI) {
	myImpl.setSourceURI(theSourceURI);
	}

public String getSourceURI(){
    return myImpl.getSourceURI();
    }

public void setLabel(String theLabel) {
	myImpl.setLabel(theLabel);
	}

public String getLabel () {
	return myImpl.getLabel();

	}
public void setComment(String theLabel) {
	myImpl.setComment(theLabel);
	}

public String getComment () {
	return myImpl.getComment();
	}

/********************************************
 * Collection Access Methods.
 *******************************************/

public void setType(LogicalResource theType) {
    myImpl.setType(theType);
    }

public LogicalResource getType() {
    return (LogicalResource)myImpl.getType();
   }

public boolean isType(LogicalResource theType) {
	return myImpl.isType(theType);
	}

public boolean isLiteralType() {
	return myImpl.isLiteralType();
	}

public boolean containsType(OGResource theType) {
	return myImpl.containsType(theType);
   }

public void addType(OGResource theType) {
	myImpl.addType(theType);
	}

public void removeType(OGResource theType) {
	myImpl.removeType(theType);
	}

public Iterator types() {
	return myImpl.types();
	}

/*
 * ---------- Property Instance Methods --------------
 */


public OGPropInstance lookupPropInstance(Object theID){
	return myImpl.lookupPropInstance(theID);
	}

public OGPropInstance addNewPropInstance(OGResource newPropType,
											  OGResource theObject) {
	return myImpl.addNewPropInstance(newPropType, theObject);
	}

public void addPropInstance(OGPropInstance thePropInstance) {
	myImpl.addPropInstance(thePropInstance);
	}

public void removePropInstance(OGPropInstance thePropInstance) {
	myImpl.removePropInstance(thePropInstance);
	}

public OGPropInstance getPropInstance(Object theID) {
	return myImpl.getPropInstance(theID);
	}

public OGPropInstance[] getPropInstances() {
	return myImpl.getPropInstances();
	}

public boolean containsPropInstance(Object theID) {
	return myImpl.containsPropInstance(theID);
	}

public Iterator propInstances() {
	return myImpl.propInstances();
	}

public Iterator propInstances(OGResource aType) {
	return myImpl.propInstances(aType);
	}

public Collection getPropInstances(OGResource aType) {
	return myImpl.getPropInstances(aType);
	}
/*
 * ---------- Class Methods --------------
 */

public OGClass lookupClass(Object theID) {
    return myImpl.lookupClass(theID);
    }


public OGClass lookupClass(Object theID, boolean pCreate){
    return myImpl.lookupClass(theID, pCreate);
    }

public OGClass createClass() {
    return myImpl.createClass();
}

public OGClass createClass(String theID) {
    return myImpl.createClass(theID);
}

public OGClass createClass(OGResource aType) {
    return myImpl.createClass(aType);
}

public OGClass createClass(OGResource aType,String theID) {
    return myImpl.createClass(aType,theID);
}

public void addClass(OGClass theClass) {
    myImpl.addClass(theClass);
    }


public void removeClass(OGClass theClass) {
    myImpl.removeClass(theClass);
    }


public OGClass getClass(Object theID){
    return myImpl.getClass(theID);
    }


public boolean containsClass(Object theID) {
    return myImpl.containsClass(theID);
    }


public Iterator classes(){
    return myImpl.classes();
    }


public Iterator classes(OGResource aType){
    return myImpl.classes(aType);
    }

public Collection getClasses(OGResource aType){
    return myImpl.getClasses(aType);
    }

/*
 * ---------- Datatype Methods --------------
 */

public OGDatatype lookupDatatype(Object theID) {
    return myImpl.lookupDatatype(theID);
    }


public OGDatatype lookupDatatype(Object theID, boolean pCreate){
    return myImpl.lookupDatatype(theID, pCreate);
    }

public OGDatatype createDatatype() {
    return myImpl.createDatatype();
}

public OGDatatype createDatatype(String theID) {
    return myImpl.createDatatype(theID);
}

public OGDatatype createDatatype(OGResource aType) {
    return myImpl.createDatatype(aType);
}

public OGDatatype createDatatype(OGResource aType,String theID) {
    return myImpl.createDatatype(aType,theID);
}

public void addDatatype(OGDatatype theDatatype) {
    myImpl.addDatatype(theDatatype);
    }


public void removeDatatype(OGDatatype theDatatype) {
    myImpl.removeDatatype(theDatatype);
    }


public OGDatatype getDatatype(Object theID){
    return myImpl.getDatatype(theID);
    }


public boolean containsDatatype(Object theID) {
    return myImpl.containsDatatype(theID);
    }


public Iterator datatypes(){
    return myImpl.datatypes();
    }


public Iterator datatypes(OGResource aType){
    return myImpl.datatypes(aType);
    }

public Collection getDatatypes(OGResource aType){
    return myImpl.getDatatypes(aType);
    }
/*
 * ---------- Property Methods --------------
 */

public OGProperty lookupProperty(Object theID){
    return myImpl.lookupProperty(theID);
    }


public OGProperty lookupProperty(Object theID, boolean pCreate){
    return myImpl.lookupProperty(theID);
    }

public OGProperty createProperty() {
    return myImpl.createProperty();
}

public OGProperty createProperty(String theID) {
    return myImpl.createProperty(theID);
}

public OGProperty createProperty(OGResource aType) {
    return myImpl.createProperty(aType);
}

public OGPropInstance createPropInstance(OGResource aType) {
    return myImpl.createPropInstance(aType);
}

public OGProperty createProperty(OGResource aType,String theID) {
    return myImpl.createProperty(aType,theID);
}

public OGPropInstance createPropInstance(OGResource aType,String theID) {
    return myImpl.createPropInstance(aType,theID);
}

public void addProperty(OGProperty theProperty){
    myImpl.addProperty(theProperty);
    }


public void removeProperty(OGProperty theProperty){
    myImpl.removeProperty(theProperty);
    }


public OGProperty getProperty(Object theID){
    return myImpl.getProperty(theID);
    }


public boolean containsProperty(Object theID){
    return myImpl.containsProperty(theID);
    }


public Iterator properties(){
    return myImpl.properties();
    }


public Iterator properties(OGResource aType){
    return myImpl.properties(aType);
    }

public Collection getProperties(OGResource aType){
    return myImpl.getProperties(aType);
    }

/*
 * ---------- Literal Methods --------------
 */

public OGLiteral lookupLiteral(Object theID){
    return myImpl.lookupLiteral(theID);
    }


public OGLiteral lookupLiteral(Object theID, boolean pCreate){
    return myImpl.lookupLiteral(theID, pCreate);
    }

public OGLiteral createLiteral() {
    return myImpl.createLiteral();
}


public OGLiteral createLiteral(String theID) {
    return myImpl.createLiteral(theID);
}

public OGLiteral createLiteralByValue(String theValue) {
    return myImpl.createLiteralByValue(theValue);
}

public OGLiteral createLiteral(OGResource aType) {
    return myImpl.createLiteral(aType);
}


public OGLiteral createLiteral(OGResource aType,String theID) {
    return myImpl.createLiteral(aType,theID);
}

public OGLiteral createLiteralByValue(OGResource aType,String theValue) {
    return myImpl.createLiteralByValue(aType,theValue);
}

public void addLiteral(OGLiteral theLiteral) {
    myImpl.addLiteral(theLiteral);
    }


public void removeLiteral(OGLiteral theLiteral){
    myImpl.removeLiteral(theLiteral);
    }


public OGLiteral getLiteral(Object theID){
    return myImpl.getLiteral(theID);
    }


public boolean containsLiteral(Object theID){
    return myImpl.containsLiteral(theID);
    }

public Iterator literals(){
    return myImpl.literals();
    }


public Iterator literals(OGResource aType){
    return myImpl.literals(aType);
    }

public Collection getLiterals(OGResource aType){
    return myImpl.getLiterals(aType);
    }

/*
/*
 * ---------- Imports Methods --------------
 */


public OGOntology lookupImport(Object theID) {
    return myImpl.lookupImport(theID);
    }


public void addImport(OGOntology theImport){
    myImpl.addImport(theImport);
    }


public void removeImport(OGOntology theImport){
    myImpl.removeImport(theImport);
    }


public OGOntology getImport(Object theID){
    return myImpl.getImport(theID);
    }

public OGOntology[] getImports(){
    return myImpl.getImports();
    }


public boolean containsImport(Object theID){
    return myImpl.containsImport(theID);
    }



public Iterator imports(){
    return myImpl.imports();
    }

/*
 * ---------- Resource Methods --------------
 */
public void addResource(OGResource theResource) {
	myImpl.addResource(theResource);
	}

public void removeResource(OGResource theResource) {
	myImpl.removeResource(theResource);
	}

public OGResource lookupResource(Object theID){
    return myImpl.lookupResource(theID);
    }

public OGResource getResource(Object theID){
    return myImpl.getResource(theID);
    }


public boolean containsResource(Object theID) {
    return myImpl.containsResource(theID);
    }


public Iterator resources(){
    return myImpl.resources();
    }


public Iterator resources(OGResource aType){
    return myImpl.resources(aType);
    }

public Collection getResources(OGResource aType){
    return myImpl.getResources(aType);
    }

   } // OWL
