/*
 * @(#)OGClass.java
 *
 * Ontology Graph API,  Copyright 2000, 2001 GRCI All Rights Reserved.
 *
 */
package com.att.gs.codip.ograph;

import java.util.Iterator;
/**
 * Provides instnaces of classes.
 *
 **/
public interface OGClass extends OGResource
{

/*
 * ---------- subClassOf Methods --------------
 */
/**
 * Looks for super classes of this class.
 *
 * Searches this class's subClassOf recursively for a super class
 * whose ID matches the provided ID.
 *
 * @param theID The ID of the sought super class.
 * @return The matching class if found, otherwise returns void.
 **/
public OGClass lookupSubClassOf(Object theID);

/**
 * Adds the provided class to the class's subClassOf collection.
 * *
 * @param theClass The super class to add to the subClassOf collection.
 **/
public void addSubClassOf(OGClass theClass);

/**
 * Removes the provided class from the class's subClassOf collection.
 * *
 * @param theClass The super class to remove from the subClassOf collection.
 **/
public void removeSubClassOf(OGClass theClass);

/**
 * Looks for the provided super class in the class's subClassOf collection.
 * *
 * @param theID The ID of the super class to get.
 * @return The matching class if found, otherwise returns void.
 **/
public OGClass getSubClassOf(Object theID);

/**
 * Accessor for the read only indexed bean property, "subClassOf"
 *
 *
 * @return The super classes of this class as an array.
 **/
public OGClass[] getSubClassOf();


/**
 * Looks for a super class in this class whose ID matches the provided ID.
 *
 * The super class must be in this class's subClassOf collection, that is
 * it must be a direct super class.
 *
 * @param theID The ID of the sought super class.
 * @return true if the super class exists in the subClassOf collection.
 **/
public boolean containsSubClassOf(Object theID);

/**
 * Provides a iterator for the all of the class's super classes.
 *
 * Only this class's direct super classes are included.
 *
 * @return An iterator over all the class's direct super classes.
 **/
public Iterator subClassOf() ;

/**
 * Looks for a super class in the subClassOf collection
 * of this resource that is a particular type.
 *
 * @param aType The type of the sought classes.
 * @return An iterator over some the resource's super classes.
 **/
public Iterator subClassOf(OGResource aType);

}
