/*
 * @(#)OGProperty.java
 *
 * Ontology Graph API,  Copyright 2000, 2001 GRCI All Rights Reserved.
 *
 */
package com.att.gs.codip.ograph;

import java.util.Iterator;

/**
 * Provides definitions of Property.
 *
 **/
public interface OGProperty extends OGResource {

/*
 * ---------- Basic Access Methods --------------
 */

/**
 * Determines if this property is a defintion or an instances
 * based on the values of its subject and object attributes.
 *
 * @returns True if this is an instance property.
 **/
public boolean pInstance();

/*
 * ---------- subPropertyOf Methods --------------
 */

/**
 * Looks for super properties of this property.
 *
 * Searches this property's subPropertyOf recursively for a super property
 * whose ID matches the provided ID.
 *
 * @param theID The ID of the sought super property.
 * @return The matching property if found, otherwise returns void.
 **/
public OGProperty lookupSubPropertyOf(Object theID);


/**
 * Adds the provided property to the property's subPropertyOf collection.
 * *
 * @param theProperty The super property to add to the subPropertyOf collection.
 **/
public void addSubPropertyOf(OGProperty theProperty);

/**
 * Removes the provided property from the property's subPropertyOf collection.
 * *
 * @param theProperty The super property to remove from the subPropertyOf collection.
 **/
public void removeSubPropertyOf(OGProperty theProperty);

/**
 * Looks for the provided super property in the property's subPropertyOf collection.
 * *
 * @param theID The ID of the super property to get.
 * @return The matching property if found, otherwise returns void.
 **/
public OGProperty getSubPropertyOf(Object theID);

/**
 * Accessor for the read only indexed bean property, "subPropertyOf"
 *
 *
 * @return The super properties of this class as an array.
 **/
public OGProperty[] getSubPropertyOf();

/**
 * Looks for a super property in this property whose ID matches the provided ID.
 *
 * The super property must be in this property's subPropertyOf collection, that is
 * it must be a direct super property.
 *
 * @param theID The ID of the sought super property.
 * @return true if the super property exists in the subPropertyOf collection.
 **/
public boolean containsSubPropertyOf(Object theID);

/**
 * Provides a iterator for the all of the property's super properties.
 *
 * Only this property's direct super properties are included.
 *
 * @return An iterator over all the property's direct super properties.
 **/
public Iterator subPropertyOf();

/**
 * Looks for a super propery in the subPropertyOf collection
 * of this resource that is a particular type.
 *
 * @param aType The type of the sought properties.
 * @return An iterator over some the resource's super properties.
 **/
public Iterator subPropertyOf(OGResource aType);
}
