/*
 * @(#)OGUtility.java
 *
 * Ontology Graph API,  Copyright 2000, 2001 GRCI All Rights Reserved.
 *
 */
package com.att.gs.codip.ograph;

import java.util.Iterator;
import java.util.HashMap;
import java.io.PrintStream;

import com.att.gs.codip.ograph.imp.OGOntologyImp;
import com.att.gs.codip.ograph.imp.OntologyManagerImp;

import jess.Rete;
import jess.Context;

/** **************************
 * A colleciton static of utility methods.
 *****************************/
public class OGUtility {

/**
 * Perform a full garbage collection.
 *
 * Repeatedly call the system garbage collection funciton until
 * free memory stops increasing. This ensures that all available
 * heap space is reclaimed.
 *
 * When working with large ontologies, this method may be useful after
 * parsing or building the model to reclaim heap space.
 **/
  public static void fullGC() {

    Runtime rt = Runtime.getRuntime();
    rt.runFinalization();
    long isFree = rt.freeMemory();
    long wasFree;
    do {
      wasFree = isFree;
      rt.gc();
      isFree = rt.freeMemory();
    } while (isFree > wasFree);
  }

/*************************
 * Display/Debug  Methods.
 **************************/


 /**
 * Prints an ontology, and all its components, to the provided PrintStream.
 *
 * @param outStream The PrintStream to use.
 * @param theOnto The ontology to print.
 *
 **/
public static void printOntotology(PrintStream outStream, OGOntology theOnto) throws Exception{
    // Dump the Ontology
    outStream.println(" " + theOnto.getID());
    outStream.println("***********************************");
    outStream.println(" VInfo : " +theOnto.getVersionInfo());
    outStream.println(" SrcURI: " +theOnto.getURI());
    outStream.print("imports: ");
    Iterator itr = theOnto.imports();
    if (itr.hasNext()) {
      while (itr.hasNext()) {
        outStream.print(((OGOntology)itr.next()).getID()+" ");
        }
      outStream.println(" ");
      }

    // Dump the Classes.
    outStream.println("\n***********************************");
    int i=0;
    outStream.println(" OGraph Classes ");
    outStream.println(" ");
    if (theOnto.classes() != null) {
      for (Iterator en = theOnto.classes(); en.hasNext();) {
        outStream.print("[" + i++ + "]");
        OGUtility.printClass(outStream, (OGClass)en.next());
        }
      }
    else outStream.println(" NONE ");
    // Dump the Properties.
    outStream.println("\n***********************************");
    i=0;
    outStream.println(" OGraph Properties ");
    outStream.println(" ");
    if (theOnto.properties().hasNext()) {
      for (Iterator en = theOnto.properties(); en.hasNext();) {
        outStream.print("[" + i++ + "]");
       OGUtility.printProperty(outStream, (OGProperty)en.next());
        }
      }
    else outStream.println(" NONE ");
    // Dump the Literals.
    outStream.println("\n***********************************");
    i=0;
    outStream.println(" OGraph Literals ");
    outStream.println(" ");
    if (theOnto.literals().hasNext()) {
      for (Iterator en = theOnto.literals(); en.hasNext();) {
        outStream.print("[" + i++ + "]");
        OGUtility.printLiteral(outStream, (OGLiteral)en.next());
        }
      }
    else outStream.println(" NONE ");
    // Dump the Datatypes.
    outStream.println("\n***********************************");
    i=0;
    outStream.println(" OGraph Datatypes ");
    outStream.println(" ");
    if (theOnto.datatypes().hasNext()) {
      for (Iterator en = theOnto.datatypes(); en.hasNext();) {
        outStream.print("[" + i++ + "]");
        OGUtility.printDatatype(outStream, (OGDatatype)en.next());
        }
      }
    else outStream.println(" NONE ");
   } // printOntology

/**
 * Prints a class to the provided PrintStream.
 *
 * @param outStream The PrintStream to use.
 * @param theClass The class to print.
 *
 **/
public static void printClass(PrintStream outStream, OGClass theClass) throws Exception {
    Iterator itr;
    outStream.println(" " + theClass.getID());
    if (theClass.getType() != null)
      outStream.println("  type : " +theClass.getType().getID());
    else
      outStream.println("  type : NULL");
    outStream.println("locName: " +theClass.getLocalName());
    outStream.println("  label: " +theClass.getLabel());
    outStream.println("  comnt: " +theClass.getComment());
    outStream.print(" subCOf: ");
    itr = theClass.subClassOf();
    if (itr.hasNext()) {
      while (itr.hasNext()) {
        outStream.print(((OGResource)itr.next()).getID()+" ");
        }
      outStream.println(" ");
      }
    else
      outStream.println("none");
    outStream.print("  props: ");
    itr = theClass.propInstances();
    if (itr.hasNext()) {
      while (itr.hasNext()) {
        outStream.print(((OGResource)itr.next()).getID()+" ");
        }
      outStream.println(" ");
      }
    else
      outStream.println("none");
    } // ShowClass


/**
 * Prints a property to the provided PrintStream.
 *
 * @param outStream The PrintStream to use.
 * @param theProperty The property to print.
 *
 **/
public static void printProperty(PrintStream outStream, OGProperty theProperty)
		 throws Exception {
    outStream.println(" " + theProperty.getID());
    if (theProperty.getType() != null)
      outStream.println("  type : " +theProperty.getType().getID());
    else
      outStream.println("  type : NULL");
    outStream.println("locName: " +theProperty.getLocalName());
    outStream.println("  pInst: " + theProperty.pInstance());
    if (theProperty.pInstance()) {
      OGResource r;
      r = ((OGPropInstance)theProperty).getSubject();
      if (r != null)
        outStream.println("subject: " + r.getID());
      else
        outStream.println("subject: " + r);
      r = ((OGPropInstance)theProperty).getObject();
      if (r != null)
        outStream.println(" object: " + r.getID());
      else
        outStream.println(" object: " + r);
      }
    else { // not an instance
        outStream.println("  label: " +theProperty.getLabel());
        outStream.println("  comnt: " +theProperty.getComment());
        outStream.print(" subPOf: ");
        Iterator en = theProperty.subPropertyOf();
        if (en.hasNext()) {
          while (en.hasNext()) {
            outStream.print(((OGResource)en.next()).getID()+" ");
            }
          outStream.println(" ");
        }
        else
          outStream.println("none");
      outStream.print("  props: ");
      Iterator itr = theProperty.propInstances();
      if (itr.hasNext()) {
        while (itr.hasNext()) {
          outStream.print(((OGResource)itr.next()).getID()+" ");
          }
        outStream.println(" ");
        }
      else
        outStream.println("none");
      } // not instance
    } // printProperty

/**
 * Prints a literal to the provided PrintStream.
 *
 * @param outStream The PrintStream to use.
 * @param theLiteral The literal to print.
 *
 **/
public static void printLiteral(PrintStream outStream, OGLiteral theLiteral)
		 throws Exception {
    outStream.println(" " + theLiteral.getID());
    outStream.println("   type: " +theLiteral.getType().getID());
    outStream.println("locName: " +theLiteral.getLocalName());
    outStream.println("  label: " +theLiteral.getLabel());
    outStream.println("  comnt: " +theLiteral.getComment());
    outStream.println("  value: " +theLiteral.getValue());
    } // printLiteral

/**
 * Prints a datatype to the provided PrintStream.
 *
 * @param outStream The PrintStream to use.
 * @param theDatatype The datatype to print.
 *
 **/
public static void printDatatype(PrintStream outStream, OGDatatype theDatatype)
		 throws Exception {
    outStream.println(" " + theDatatype.getID());
    outStream.println("  type : " +theDatatype.getType().getID());
    outStream.println("locName: " +theDatatype.getLocalName());
    outStream.println("  label: " +theDatatype.getLabel());
    outStream.println("  comnt: " +theDatatype.getComment());
} // printDatatype


/*************************
 * Jess Interface  Methods.
 **************************/

static HashMap jessOntos = new HashMap();

/**
 * Adds an ontology, and all its compoents, to the provided Jess Rete Object.
 * Duplicate instances of the ontolgoy will not be added.
 * <P>
 * Note that a suitable jess defclass calss must have be executed before
 * using this method. A Jess script with appropriate calls
 * is in OGRTemplates.clp
 *
 * @param rete The Jess Rete object to add facts to.
 * @param theOnto The ontology to print.
 *
 **/
public static void addJessOntotology(Rete rete, OGOntology theOnto) throws Exception {
	OGUtility.addJessOntotology(rete, theOnto, false);
   }

/**
 * Adds an ontology, and all its compoents, to the provided Jess Rete Object.
 * Duplicate instances of the ontolgoy will only be added if force is true.
 * <P>
 * Note that a suitable jess defclass calss must have be executed before
 * using this method. A Jess script with appropriate calls
 * is in OGRTemplates.clp
 *
 * @param rete The Jess Rete object to add facts to.
 * @param theOnto The ontology to print.
 * @param forceReload A flag force reloading a previously loaded ontology.
 *
 **/
public static void addJessOntotology(Rete rete,
				     OGOntology theOnto,
				     boolean forceReload) throws Exception {

    // Adding the Ontology
    System.out.println("***********************************");
    System.out.println(" ONT "+ theOnto.getID());

    // Determine if the ontology should be loaded.
    if (!forceReload &&
    	   jessOntos.containsKey(theOnto.getID())) {
   	 System.out.println("     Already Loaded.");
    	 return;
    	 }

    Context context = rete.getGlobalContext();

    // Jess 6 and later
    rete.definstance("OGOntology",theOnto, false, context);

    // Add the Classes.
    //System.out.println("***********************************");
    int i=0;
    if (theOnto.classes() != null) {
      for (Iterator en = theOnto.classes(); en.hasNext();) {
        OGUtility.addJessClass(rete, context, (OGClass)en.next());
        i++;
        }
      System.out.println(" Added " + i + " Classes.");
      }
    else
      System.out.println(" Added no Classes.");
    // Add the Properties.
    //System.out.println("***********************************");
    i=0;
    if (theOnto.properties().hasNext()) {
      for (Iterator en = theOnto.properties(); en.hasNext();) {
        OGUtility.addJessProperty(rete, context, (OGProperty)en.next());
        i++;
        }
      System.out.println(" Added " + i + " Properties.");
      }
    else
      System.out.println(" Added no Properties.");
    // Add the Literals.
    //System.out.println("***********************************");
    i=0;
    if (theOnto.literals().hasNext()) {
      for (Iterator en = theOnto.literals(); en.hasNext();) {
        OGUtility.addJessLiteral(rete, context, (OGLiteral)en.next());
        i++;
        }
      System.out.println(" Added " + i + " Literals.");
      }
    else
      System.out.println(" Added no Literals.");
    // Add the Datatypes.
    //System.out.println("***********************************");
    i=0;
    if (theOnto.datatypes().hasNext()) {
      for (Iterator en = theOnto.datatypes(); en.hasNext();) {
        OGUtility.addJessDatatype(rete, context, (OGDatatype)en.next());
        i++;
        }
      System.out.println(" Added " + i + " Datatypes.");
      }
    else
      System.out.println(" Added no Datatypes.");

    // Successfully loaded, make a not of it.
    jessOntos.put(theOnto.getID(),"Loaded");

   } // addJessOntotology

/**
 * Adds a class to the provided Jess Rete Object.
 * <P>
 * Note that a suitable jess defclass must have be executed before
 * using this method.
 *
 * @param rete The Jess Rete object to add facts to.
 * @param theClass The class to print.
 *
 **/
public static void addJessClass(Rete rete, Context context,
                                OGClass theClass) throws Exception {
    //System.out.println(" CLS "+ theClass.getID());
    // Jess 6 and later
    rete.definstance("OGClass",theClass, false, context);
    } // addJessClass


/**
 * Adds a property to the provided Jess Rete Object.
 * <P>
 * Note that a suitable jess defclass must have be executed before
 * using this method.
 *
 * @param rete The Jess Rete object to add facts to.
 * @param theProperty The property to print.
 *
 **/
public static void addJessProperty(Rete rete,  Context context,
                                   OGProperty theProperty)
		 throws Exception {
    //System.out.print("\naddJessProperty: adding ");
    //OGUtility.printProperty(System.out, theProperty);

    //System.out.println(" PRP "+ theProperty.getID());
    // Jess 6 and later
    if (theProperty instanceof OGPropInstance)
      addJessInstance(rete, context, (OGPropInstance)theProperty);
    else
      rete.definstance("OGProperty",theProperty, false, context);
    } // addJessProperty

/**
 * Adds a literal to the provided Jess Rete Object.
 * <P>
 * Note that a suitable  ess defclass must have be executed before
 * using this method.
 *
 * @param rete The Jess Rete object to add facts to.
 * @param theLiteral The literal to add.
 *
 **/
public static void addJessLiteral(Rete rete,  Context context,
                                  OGLiteral theLiteral)
		 throws Exception {
    //System.out.println(" LIT "+ theLiteral.getID());
    // Jess 6 and later
    rete.definstance("OGLiteral",theLiteral, false, context);
    } // addJessLiteral

/**
 * Adds a datatype to the provided Jess Rete Object.
 * <P>
 * Note that a suitable jess defclass must have be executed before
 * using this method.
 *
 * @param rete The Jess Rete object to add facts to.
 * @param theDatatype The Datatype to add.
 *
 **/
public static void addJessDatatype(Rete rete,  Context context,
                                   OGDatatype theDatatype)
		 throws Exception {
    //System.out.println(" DTY "+ theDatatype.getID());
    // Jess 6 and later
    rete.definstance("OGDatatype",theDatatype, false, context);
    } // addJessDatatype

/**
 * Adds the instances in an ontology to the provided Jess Rete Object.
 * Instances from duplicates of the ontology will not be added.
 * <P>
 * Note that a suitable jess defclass must have be executed before
 * using this method.
 *
 * @param rete The Jess Rete object to add facts to.
 * @param theOnto The ontology to print.
 *
 **/
public static void addJessInstances(Rete rete, OGOntology theOnto) throws Exception {
	OGUtility.addJessInstances(rete, theOnto, false);
   }

/**
 * Adds the instances in an ontology to the provided Jess Rete Object.
 * Instances from duplicates of the ontology will not be added unless
 * forceReload is true.
 * <P>
 * Note that a suitable jess defclass must have be executed before
 * using this method.
 *
 * @param rete The Jess Rete object to add facts to.
 * @param theOnto The ontology to print.
 * @param forceReload A flag force reloading a previously loaded ontology.
 *
 **/
public static void addJessInstances(Rete rete,
                                    OGOntology theOnto,
				    boolean forceReload) throws Exception {
    // Adding the Ontology
    System.out.println("***********************************");
    System.out.println(" ONT (I) "+ theOnto.getID());

    // Determine if the ontology should be loaded.
    if (!forceReload &&
    	   jessOntos.containsKey(theOnto.getID())) {
   	 System.out.println("     Already Loaded.");
    	 return;
    	 }

    Context context = rete.getGlobalContext();


    int i=0;
    if (theOnto.properties().hasNext()) {
      for (Iterator en = theOnto.properties(); en.hasNext();) {
        OGProperty aProp = (OGProperty)en.next();
        if (aProp.pInstance())
          OGUtility.addJessInstance(rete, context, (OGPropInstance)aProp);
        i++;
        }
      System.out.println(" Added " + i + " Instances.");
      }
    else
      System.out.println(" Added no Instances.");
    }
/**
 * Adds an instance to the provided Jess Rete Object.
 * <P>
 * Note that a suitable jess defclass must have be executed before
 * using this method.
 *
 * @param rete The Jess Rete object to add facts to.
 * @param theProperty The property instance to add.
 *
 **/
public static void addJessInstance(Rete rete,  Context context,
                                   OGPropInstance theProperty)
		 throws Exception {
    //System.out.print("\naddJessProperty: adding ");
    //OGUtility.printProperty(System.out, theProperty);

    //System.out.println(" PRP "+ theProperty.getID());
    // Jess 6 and later
    rete.definstance("OGInstance",theProperty, false, context);
   } // addJessProperty

/**
 * Removes an ontology, and all its compoents, from the provided Jess Rete Object.
 *
 * @param rete The Jess Rete object to add facts to.
 * @param theOnto The ontology to print.
 *
 **/
public static void removeJessOntotology(Rete rete,
				     OGOntology theOnto) throws Exception {

    // Removing the Ontology
    System.out.println("***********************************");
    System.out.println(" ONT "+ theOnto.getID());

    // Determine if the ontology should be unloaded.
    if (!jessOntos.containsKey(theOnto.getID())) {
   	 System.out.println("     Not Loaded.");
    	 return;
    	 }
    jessOntos.remove(theOnto.getID());

    // Jess 6 and later
    rete.undefinstance(theOnto);

    // Remove the Classes.
    //System.out.println("***********************************");
    int i=0;
    if (theOnto.classes() != null) {
      for (Iterator en = theOnto.classes(); en.hasNext();) {
        rete.undefinstance((OGClass)en.next());
        i++;
        }
      System.out.println(" Removed " + i + " Classes.");
      }
    else
      System.out.println(" Removed no Classes.");
    // Remove the Properties.
    //System.out.println("***********************************");
    i=0;
    if (theOnto.properties().hasNext()) {
      for (Iterator en = theOnto.properties(); en.hasNext();) {
        rete.undefinstance((OGProperty)en.next());
        i++;
        }
      System.out.println(" Removed " + i + " Properties.");
      }
    else
      System.out.println(" Removed no Properties.");
    // Remove the Literals.
    //System.out.println("***********************************");
    i=0;
    if (theOnto.literals().hasNext()) {
      for (Iterator en = theOnto.literals(); en.hasNext();) {
        rete.undefinstance((OGLiteral)en.next());
        i++;
        }
      System.out.println(" Removed " + i + " Literals.");
      }
    else
      System.out.println(" Removed no Literals.");
    // Remove the Datatypes.
    //System.out.println("***********************************");
    i=0;
    if (theOnto.datatypes().hasNext()) {
      for (Iterator en = theOnto.datatypes(); en.hasNext();) {
        rete.undefinstance((OGDatatype)en.next());
        i++;
        }
      System.out.println(" Removed " + i + " Datatypes.");
      }
    else
      System.out.println(" Removed no Datatypes.");

    // Successfully removed, make a not of it.
    jessOntos.put(theOnto.getID(),"Unloaded");

   } // addJessOntotology

public static void removeJessInstances(Rete rete,
                                    OGOntology theOnto) throws Exception {
    // Removing the Ontology
    System.out.println("***********************************");
    System.out.println(" ONT (I) "+ theOnto.getID());

    // Determine if the ontology should be unloaded.
    if (!jessOntos.containsKey(theOnto.getID())) {
   	 System.out.println("     Not Loaded.");
    	 return;
    	 }
    jessOntos.remove(theOnto.getID());

    int i=0;
    if (theOnto.properties().hasNext()) {
      for (Iterator en = theOnto.properties(); en.hasNext();) {
        OGProperty aProp = (OGProperty)en.next();
        if (aProp.pInstance())
        rete.undefinstance((OGPropInstance)en.next());
        i++;
        }
      System.out.println(" Removed " + i + " Instances.");
      }
    else
      System.out.println(" Removed no Instances.");
    }
/**
 * Main added for basic testing.
 **/
public static void main(String[] args) throws Exception {

	String myURI = "urn:mem:foo";
		if (args.length >=  1) myURI = args[0];
		String myNS = myURI+"#";

	OGOntology coreOntology;
	OntologyManager  theOntoManager;

	theOntoManager = new OntologyManagerImp("OGUtilityMain");

    // Get the core ontology.
    coreOntology = theOntoManager.getOWL();

    System.out.println("***********************************");
    System.out.println(" Testing imports.... ");
    System.out.println("***********************************");
    System.out.println("owl:Class " + coreOntology.lookupResource(OWL.OWL_NS + "Class"));
    System.out.println("rdf:type " + coreOntology.lookupResource(RDFS.RDF_NS + "type"));
    System.out.println("rdfs:Class " + coreOntology.lookupResource(RDFS.RDFS_NS + "Class"));
    System.out.println("xsd:double " + coreOntology.lookupResource(XSD.XSD_NS + "double"));

	System.out.println("***********************************");
	System.out.println(" Programatic build of "+myNS);
	System.out.println("***********************************");
	OGOntology anOntology =theOntoManager.createOntology(myURI, false);
	anOntology.addImport(coreOntology);
	anOntology.setVersionInfo("This is a test, only a test, of PROG2DAML");

	OGClass classTop = anOntology.createClass(myNS+"classTop");
	classTop.addSubClassOf(OWL.Class);

	OGClass class1 = anOntology.createClass(myNS+"class1");
	class1.addSubClassOf(classTop);

	OGClass class2 = anOntology.createClass(myNS+"class2");
	class2.addSubClassOf(classTop);

	// Seperate class tree.
	OGClass class3 = anOntology.createClass(myNS+"class3");
	OGClass class4 = anOntology.createClass(myNS+"class4");

	OGProperty propObj = anOntology.createProperty(OWL.ObjectProperty,myNS+"myObjProp");
	OGProperty instPO12 = anOntology.createProperty(propObj); // anonymous.

	// Class 3 to Class 4
	OGPropInstance instPO34 = anOntology.createPropInstance(propObj);
	instPO34.setSubject(class3);
	class3.addPropInstance(instPO34);
	instPO34.setObject(class4);

	OGProperty propDT = anOntology.createProperty(OWL.DatatypeProperty, myNS+"myDTProp");
	OGPropInstance instDT = anOntology.createPropInstance(propDT);

	instDT.addSubPropertyOf(OWL.DatatypeProperty);
	OGLiteral lit = anOntology.createLiteralByValue("57");

	instDT.setSubject(class2);
	class2.addPropInstance(instDT);
	instDT.setObject(lit);

	OGClass theRestriction = anOntology.createClass(OWL.Restriction);
	OGPropInstance onProperty = anOntology.createPropInstance(OWL.onProperty);
	onProperty.setSubject(theRestriction);
	onProperty.setObject(propDT);

	OGPropInstance toClass = anOntology.createPropInstance(OWL.allValuesFrom);
	toClass.setSubject(theRestriction);
	toClass.setObject(anOntology.lookupDatatype(XSD.XSD_NS+"string",true));

	theRestriction.addPropInstance(onProperty);
	theRestriction.addPropInstance(toClass);
	class2.addSubClassOf(theRestriction);

	//*******************************
	// Show the properties of class2
	//*******************************
	System.out.println("Class " + class2.getID());
	for (Iterator thePropInstances = class2.propInstances();
				  thePropInstances.hasNext();) {
		 OGInstance anInstance = (OGInstance)thePropInstances.next();
		 System.out.print(" has property "+ anInstance.getSubject() + " " +
											  anInstance.getProperty() + " ");
		 if (anInstance.getObject() instanceof OGLiteral)
		   System.out.println(" L:"+ ((OGLiteral)anInstance.getObject()).getValue());
		 else
		   System.out.println(" R:"+ anInstance.getObject());
		}
	for (Iterator myClassRestrictions = class2.subClassOf(OWL.Restriction);
				  myClassRestrictions .hasNext();) {
	OGResource aResource = (OGResource)myClassRestrictions .next();
	System.out.println(" Restiction " + aResource.getID());
	for (Iterator thePropInstances = aResource.propInstances();
				  thePropInstances.hasNext();) {
		 OGInstance anInstance = (OGInstance)thePropInstances.next();
		 System.out.println("   "+ anInstance.getSubject() + " " +
								   anInstance.getProperty() + " " +
								   anInstance.getObject());
		}
	}


	System.out.println("***********************************");
	System.out.println(" OWL ");
	System.out.println("***********************************");
	OGUtility.printOntotology(System.out, anOntology);


	System.out.println("***********************************");
	System.out.println(" Build SubOntology One");
	System.out.println("***********************************");
	// Copy a sub ontology tree...
	//
	// Currently must use the Imp!!!
	//
	OGOntologyImp aSubOntology = new OGOntologyImp(myURI+"-sub1");
	theOntoManager.addOntology(aSubOntology);

	// Use the method from the instance of the source ontology,
	// not the instance of the copy. Unless, you want the copy to
	// be empty.
	((OGOntologyImp)anOntology).copySubOntology(class3,aSubOntology);
	OGUtility.printOntotology(System.out, aSubOntology);

	System.out.println("***********************************");
	System.out.println(" Remove Resources from SubOnto One");
	System.out.println("***********************************");
	aSubOntology.removeClass(class3);
	OGUtility.printOntotology(System.out, aSubOntology);

	} // main

} // OGUtilities
