/*
 * @(#)OWL.java
 *
 * Ontology Graph API
 * Copyright 2003 AT&T Govvernment Solution, Inc.
 * All Rights Reserved.
 *
 */
package com.att.gs.codip.ograph;

import java.lang.reflect.Constructor;


/** **************************
 * An implementaiton of the OGOntology Interface that provides
 * static instances of the OWL Classes and Properties.
 *
 ************************** **/

public class OWL extends OGBaseOntology implements OGOntology  {
//
// Built in URIs, Classes and Properties, with initialization.
//

/**
 * The URI of the OWL syntax specificaiton.
 **/
public static final String OWL_URI = "http://www.w3.org/2002/07/owl";
/**
 * The Namespace of the OWL syntax specificaiton.
 **/
public static final String OWL_NS  = OWL_URI+"#";

public static OGOntology Ontology;

public static OGClass Class;

public static OGClass Thing;
public static OGClass Nothing;

public static OGDatatype Datatype;

public static OGClass Literal;
public static OGProperty ObjectProperty;
public static OGProperty DatatypeProperty;

public static OGClass AllDifferent;
public static OGClass Restriction;

public static OGProperty FunctionalProperty;
public static OGProperty InverseFunctionalProperty;
public static OGProperty SymmetricProperty;
public static OGProperty TransitiveProperty;

public static OGProperty allValuesFrom;
public static OGProperty backwardCompatibleWith;
public static OGProperty cardinality;
public static OGProperty complementOf;
public static OGProperty differentFrom;
public static OGProperty disjointWith;
public static OGProperty distinctMembers;
public static OGProperty equivalentClass;
public static OGProperty equivalentProperty;
public static OGProperty hasValue;
public static OGProperty imports;
public static OGProperty incompatibleWith;
public static OGProperty intersectionOf;
public static OGProperty inverseOf;
public static OGProperty maxCardinality;
public static OGProperty minCardinality;
public static OGProperty oneOf;
public static OGProperty onProperty;
public static OGProperty priorVersion;
public static OGProperty sameAs;
public static OGProperty sameIndividualAs;
public static OGProperty someValuesFrom;
public static OGProperty unionOf;
public static OGProperty versionInfo;

static {
    }

//
// Creators and initializors.
//

/**
 * Creates an instance of OWL using the passed in OGOntology.
 * @parms anOntoImpl An instance of a OGOntology to use.
 * @serial
 */
public OWL(OGOntology anOntoImpl) {
    myImpl = anOntoImpl;
    initialize();
    }

/**
 * Creates the static classes.
 *
 * @serial
 */
public void initialize() {
	// Create an instance of the implementation type.
    try {
      Object[] ConsVals = new Object[1];
      ConsVals[0] = new String(OWL.OWL_NS + "Ontology");
      Class[] ConsTypes = new Class[1];
      ConsTypes[0] = ConsVals[0].getClass();
      Constructor theCons = myImpl.getClass().getConstructor(ConsTypes);
      OWL.Ontology = (OGOntology)theCons.newInstance(ConsVals);
	  OWL.Ontology.setComment("OWL");
	  OWL.Ontology.setType(OWL.Ontology);
	  // Finish setup of this.
	  this.addResource(OWL.Ontology);
	  this.setType(OWL.Ontology);
      }
    catch (Exception ex) {
    	System.err.println("Constructor Failed in OWL.");
    	ex.printStackTrace(System.err);
    	}
    // The root of all resources.
    // Class needs to be initialized first, is is used as the
    // type of classes below.
    OWL.Class = myImpl.lookupClass(OWL.OWL_NS + "Class", true);
    OWL.Class.setComment("OWL");
    OWL.Class.setType(OWL.Class); // Need since it is null in the creator

    OWL.Thing = myImpl.lookupClass(OWL.OWL_NS + "Thing", true);
    OWL.Thing.setComment("OWL");

    OWL.Nothing = myImpl.lookupClass(OWL.OWL_NS + "Nothing", true);
    OWL.Nothing.setComment("OWL");

    OWL.Literal = myImpl.lookupClass(OWL.OWL_NS + "Literal", true);
    OWL.Literal.setComment("OWL");
    //OWL.Literal.addSubClassOf(RDFS.Literal);

    OWL.Datatype = myImpl.lookupDatatype(OWL.OWL_NS + "Datatype", true);
    OWL.Datatype.setComment("OWL");

    OWL.AllDifferent = myImpl.lookupClass(OWL.OWL_NS + "AllDifferent", true);
    OWL.AllDifferent.setComment("OWL");

    OWL.Restriction = myImpl.lookupClass(OWL.OWL_NS + "Restriction", true);
    OWL.Restriction.setComment("OWL");

    // Properties
    OWL.ObjectProperty = myImpl.lookupProperty(OWL.OWL_NS + "ObjectProperty", true);
    OWL.ObjectProperty.setComment("OWL");
    OWL.ObjectProperty.addSubPropertyOf(RDFS.Property);

    OWL.DatatypeProperty = myImpl.lookupProperty(OWL.OWL_NS + "DatatypeProperty", true);
    OWL.DatatypeProperty.setComment("OWL");
    OWL.DatatypeProperty.addSubPropertyOf(RDFS.Property);

    OWL.FunctionalProperty = myImpl.lookupProperty(OWL.OWL_NS + "FunctionalProperty", true);
    OWL.FunctionalProperty.setComment("OWL");
    OWL.FunctionalProperty.addSubPropertyOf(RDFS.Property);

    OWL.InverseFunctionalProperty = myImpl.lookupProperty(OWL.OWL_NS + "InverseFunctionalProperty", true);
    OWL.InverseFunctionalProperty.setComment("OWL");
    OWL.InverseFunctionalProperty.addSubPropertyOf(RDFS.Property);

    OWL.SymmetricProperty = myImpl.lookupProperty(OWL.OWL_NS + "SymmetricProperty", true);
    OWL.SymmetricProperty.setComment("OWL");
    OWL.SymmetricProperty.addSubPropertyOf(OWL.ObjectProperty);

    OWL.TransitiveProperty = myImpl.lookupProperty(OWL.OWL_NS + "TransitiveProperty", true);
    OWL.TransitiveProperty.setComment("OWL");
    OWL.TransitiveProperty.addSubPropertyOf(OWL.ObjectProperty);

    OWL.allValuesFrom = myImpl.lookupProperty(OWL.OWL_NS + "allValuesFrom", true);
    OWL.allValuesFrom.setComment("OWL");

    OWL.backwardCompatibleWith = myImpl.lookupProperty(OWL.OWL_NS + "backwardCompatibleWith", true);
    OWL.backwardCompatibleWith.setComment("OWL");

    OWL.cardinality = myImpl.lookupProperty(OWL.OWL_NS + "cardinality", true);
    OWL.cardinality.setComment("OWL");

    OWL.complementOf = myImpl.lookupProperty(OWL.OWL_NS + "complementOf", true);
    OWL.complementOf.setComment("OWL");

    OWL.differentFrom = myImpl.lookupProperty(OWL.OWL_NS + "differentFrom", true);
    OWL.differentFrom.setComment("OWL");

    OWL.disjointWith = myImpl.lookupProperty(OWL.OWL_NS + "disjointWith", true);
    OWL.disjointWith.setComment("OWL");

    OWL.distinctMembers = myImpl.lookupProperty(OWL.OWL_NS + "distinctMembers", true);
    OWL.distinctMembers.setComment("OWL");

    OWL.equivalentProperty = myImpl.lookupProperty(OWL.OWL_NS + "equivalentProperty", true);
    OWL.equivalentProperty.setComment("OWL");

    OWL.hasValue = myImpl.lookupProperty(OWL.OWL_NS + "hasValue", true);
    OWL.hasValue.setComment("OWL");

    OWL.imports = myImpl.lookupProperty(OWL.OWL_NS + "imports", true);
    OWL.imports.setComment("OWL");

    OWL.incompatibleWith = myImpl.lookupProperty(OWL.OWL_NS + "incompatibleWith", true);
    OWL.incompatibleWith.setComment("OWL");

    OWL.intersectionOf = myImpl.lookupProperty(OWL.OWL_NS + "intersectionOf", true);
    OWL.intersectionOf.setComment("OWL");

    OWL.inverseOf = myImpl.lookupProperty(OWL.OWL_NS + "inverseOf", true);
    OWL.inverseOf.setComment("OWL");

    OWL.maxCardinality = myImpl.lookupProperty(OWL.OWL_NS + "maxCardinality", true);
    OWL.maxCardinality.setComment("OWL");

    OWL.minCardinality = myImpl.lookupProperty(OWL.OWL_NS + "minCardinality", true);
    OWL.minCardinality.setComment("OWL");

    OWL.oneOf = myImpl.lookupProperty(OWL.OWL_NS + "oneOf", true);
    OWL.oneOf.setComment("OWL");

    OWL.onProperty = myImpl.lookupProperty(OWL.OWL_NS + "onProperty", true);
    OWL.onProperty.setComment("OWL");

    OWL.priorVersion = myImpl.lookupProperty(OWL.OWL_NS + "priorVersion", true);
    OWL.priorVersion.setComment("OWL");

    OWL.sameAs = myImpl.lookupProperty(OWL.OWL_NS + "sameAs", true);
    OWL.sameAs.setComment("OWL");

    OWL.sameIndividualAs = myImpl.lookupProperty(OWL.OWL_NS + "sameIndividualAs", true);
    OWL.sameIndividualAs.setComment("OWL");

    OWL.someValuesFrom = myImpl.lookupProperty(OWL.OWL_NS + "someValuesFrom", true);
    OWL.someValuesFrom.setComment("OWL");

    OWL.unionOf = myImpl.lookupProperty(OWL.OWL_NS + "unionOf", true);
    OWL.unionOf.setComment("OWL");

    OWL.versionInfo = myImpl.lookupProperty(OWL.OWL_NS + "versionInfo", true);
    OWL.versionInfo.setComment("OWL");

    }

/***********************************************
 * Most OWL Ontology Methods Inherited from 
 * OGBaseOntology. Only overidded ones here.
 *********************************************/
public void morph(String URI) {
	// Can't morph this.
	}

   } // OWL
