package com.att.gs.codip.ograph;

import edu.unika.aifb.rdf.api.model.Model;

/** KAON RDF API **/

/****************************
 * Ontology Factory
 ****************************/
public interface OntologyFactory {

/**
 * Boolen to determin if transforms are applied to imports.
 *
 **/
public void setApplyToImports(boolean pTransform);

/****************************************************
 * Factory Methods.
 *****************************************************/
/**
 * Builds ontologies from  RDF models.
 *
 * Adds the triples in an RDF model to this ontology.
 *
 * @param theRDF The RDF Model  to use.
 **/
public OGOntology createFromRDF(Model theRDF); // Change to use RDF_API

/**
 * Builds ontologies from a model identified by the URI.
 *
 * @param theURI The URI of the ontology to use.
 **/
public OGOntology createFromURI(String theURI);

/**
 * Builds ontologies from an ontology identified by the URI,
 * after applying the XSL transformation specified in the
 * XSL URI.
 *
 * @param theURI The URI of the ontology  to use.
 * @param theXSLURI The URI of the XSL specification to use.
 **/
public OGOntology createFromURI(String theURI, String theXSLURI);

/**
 * Creates an RDF model from an ontology.
 *
 *
 * @return The RDF model.
 **/
public Model toRDFModel(OGOntology theWorkingOntology) throws Exception;



} // OntoFactory
