/*
 * @(#)RDFS.java
 *
 * Ontology Graph API
 * Copyright 2003 AT&T Govvernment Solution, Inc.
 * All Rights Reserved.
 *
 */
package com.att.gs.codip.ograph;


/** **************************
 * An implementaiton of the OGOntology Interface that provides
 * static instances of the RDFS Classes and Properties.
 *
 ************************** **/

public class RDFS extends OGBaseOntology implements OGOntology  {
//
// Built in URIs, Classes and Properties, with initialization.
//
/**
 * The URI of the RDF syntax specificaiton.
 **/
public static final String RDF_URI  = "http://www.w3.org/1999/02/22-rdf-syntax-ns";
/**
 * The Namespace of the RDF syntax specificaiton.
 **/
public static final String RDF_NS   = RDF_URI+"#";
/**
 * The URI of the RDFS syntax specificaiton.
 **/
public static final String RDFS_URI  = "http://www.w3.org/2000/01/rdf-schema";
/**
 * The Namespace of the RDFS syntax specificaiton.
 **/
public static final String RDFS_NS   = RDFS_URI+"#";

public static OGOntology Ontology;


/**
 * RDF/S as an OGONtology
 **/
public static OGClass Class;
public static OGClass Literal;
public static OGClass Datatype;
public static OGClass List;
public static OGClass nil;

public static OGProperty Property;
public static OGProperty label;
public static OGProperty comment;
public static OGProperty isDefinedBy;
public static OGProperty first;
public static OGProperty rest;
public static OGProperty domain;
public static OGProperty range;
public static OGProperty subClassOf;
public static OGProperty subPropertyOf;

public static OGProperty RDF_type;

static {
    }

//
// Creators and initializors.
//
/**
 * Creates an instance of using the passed in OGOntology.
 * @parms anOntoImpl An instance of a OGOntology to use.
 * @serial
 */
public RDFS(OGOntology anOntoImpl) {
    myImpl = anOntoImpl;
    initialize();
    }

/**
 * Creates the static classes.
 *
 * @serial
 */
public void initialize() {

    OGClass theClass;

    RDFS.Ontology = this;
    RDFS.Ontology.setComment("RDFS");

    // The root of all resources.
    // Class needs to be initialized first, is is used as the
    // type of classes below.
    RDFS.Class = myImpl.lookupClass(RDFS.RDFS_NS + "Class", true);
    RDFS.Class.setComment("RDFS");
    RDFS.Class.setType(RDFS.Class); // Need since it is null in the creator


    RDFS.Class = myImpl.lookupClass(RDFS.RDFS_NS + "Class", true);
    RDFS.Class.setComment("RDFS");

    RDFS.Literal = myImpl.lookupClass(RDFS.RDFS_NS + "Literal", true);
    RDFS.Literal.setComment("RDFS");

    RDFS.Datatype = myImpl.lookupClass(RDFS.RDFS_NS + "Datatype", true);
    RDFS.Datatype.setComment("RDFS");

    RDFS.List = myImpl.lookupClass(RDFS.RDF_NS + "List", true);
    RDFS.List.setComment("RDF");

	RDFS.nil = myImpl.lookupClass(RDFS.RDF_NS + "nil", true);
	RDFS.nil.setComment("RDF");

    RDFS.Property = myImpl.lookupProperty(RDFS.RDF_NS + "Property", true);
    RDFS.Property.setComment("RDF");
    RDFS.Property.setType(RDFS.Property); // Need since it is null in the creator

    RDFS.label = myImpl.lookupProperty(RDFS.RDFS_NS + "label", true);
    RDFS.label.setComment("RDFS");

    RDFS.comment = myImpl.lookupProperty(RDFS.RDFS_NS + "comment", true);
    RDFS.comment.setComment("RDFS");

    RDFS.isDefinedBy = myImpl.lookupProperty(RDFS.RDFS_NS + "isDefinedBy", true);
    RDFS.isDefinedBy.setComment("RDFS");

    RDFS.first = myImpl.lookupProperty(RDFS.RDF_NS + "first", true);
    RDFS.first.setComment("RDF");

    RDFS.rest = myImpl.lookupProperty(RDFS.RDF_NS + "rest", true);
    RDFS.rest.setComment("RDF");

    RDFS.domain = myImpl.lookupProperty(RDFS.RDFS_NS + "domain", true);
    RDFS.domain.setComment("RDFS");

    RDFS.range = myImpl.lookupProperty(RDFS.RDFS_NS + "range", true);
    RDFS.range.setComment("RDFS");

    RDFS.subClassOf = myImpl.lookupProperty(RDFS.RDFS_NS + "subClassOf", true);
    RDFS.subClassOf.setComment("RDFS");

    RDFS.subPropertyOf = myImpl.lookupProperty(RDFS.RDFS_NS + "subPropertyOf", true);
    RDFS.subPropertyOf.setComment("RDFS");

    RDFS.RDF_type = myImpl.lookupProperty(RDFS.RDF_NS + "type", true);
    RDFS.RDF_type.setComment("RDF");

    }

/***********************************************
 * Most RDFS Ontology Methods Inherited from 
 * OGBaseOntology. Only overidded ones here.
 *********************************************/
public void morph(String URI) {
	// Can't morph this.
	}


   } // RDFS
