/*
 * @(#)XSD.java
 *
 * Ontology Graph API,  Copyright 2002, AT&T All Rights Reserved.
 *
 */
package com.att.gs.codip.ograph;


/** **************************
 * An implementaiton of the OGOntology Interface that provides
 * static instances of the XML Schema Data types.
 *
 ************************** **/

public class XSD extends OGBaseOntology implements OGOntology  {
//
// Built in URIs, Classes and Properties, with initialization.
//
/**
 * The URI of the XMLSchema Datatype specificaiton.
 **/
public static final String XSD_URI_2000  = "http://www.w3.org/2000/10/XMLSchema";
public static final String XSD_URI  = "http://www.w3.org/2001/XMLSchema";
/**
 * The Namespace of the XMLSchema Datatype specificaiton.
 **/
public static final String XSD_NS  = XSD_URI+"#";

/**
 * Common XSD data types are predefined.
 **/

public static OGOntology Ontology;

public static OGDatatype xsd_string;
public static OGDatatype xsd_double;
public static OGDatatype xsd_long;
public static OGDatatype xsd_real;
public static OGDatatype xsd_integer;
public static OGDatatype xsd_decimal;

static {
    }

//
// Creators and initializors.
//

/**
 * Creates an instance using the passed in OGOntology.
 * @parms anOntoImpl An instance of a OGOntology to use.
 */
public XSD(OGOntology anOntoImpl) {
    myImpl = anOntoImpl;
    initialize();
    }

/**
 * Creates the static classes.
 */
public void initialize() {

    XSD.Ontology = this;
    XSD.Ontology.setComment("XML Schema Datatypes");

    XSD.xsd_string = myImpl.lookupDatatype(XSD.XSD_NS + "string", true);
    XSD.xsd_string.setComment("XML Schema Datatypes");

    XSD.xsd_double = myImpl.lookupDatatype(XSD.XSD_NS + "double", true);
    XSD.xsd_double.setComment("XML Schema Datatypes");

    XSD.xsd_long = myImpl.lookupDatatype(XSD.XSD_NS + "long", true);
    XSD.xsd_long.setComment("XML Schema Datatypes");

    XSD.xsd_real = myImpl.lookupDatatype(XSD.XSD_NS + "real", true);
    XSD.xsd_real.setComment("XML Schema Datatypes");

    XSD.xsd_integer = myImpl.lookupDatatype(XSD.XSD_NS + "integer", true);
    XSD.xsd_integer.setComment("XML Schema Datatypes");

    XSD.xsd_decimal = myImpl.lookupDatatype(XSD.XSD_NS + "decimal", true);
    XSD.xsd_decimal.setComment("XML Schema Datatypes");

    }

/***********************************************
 * Most XSD Ontology Methods Inherited from
 * OGBaseOntology. Only overidded ones here.
 *********************************************/

/**
 * XSD ID can not be morphed. Override any
 * possible implelmetation morph method.
 **/
public void morph(String URI) {
	// Can't morph this.
	}

/**
 * Magic lookup Resoruce that will create datatype
 * if does not exist in XSD ontologiy.
 **/
public OGResource lookupResource(Object theID){
    // XSD will match any XSD namespace datatype.
    OGResource theResource = null;
    if (myImpl.containsResource(theID)) {
    	 theResource = myImpl.getResource(theID);
        }
    // I not found, then create a datatype.
    else if (theID instanceof String) {
        String theIDStr = (String)theID;
    	if (theIDStr.startsWith(XSD.XSD_NS)) {
    	  theResource = myImpl.lookupDatatype(theID, true);
    	  }
    }
    return theResource;
    }

   } // XSD
