/*
 * BookmarksMozilla.java
 *
 * Created on Aug 5, 2004
 */

package cz.cvut.km.io;

import cz.cvut.km.model.*;
import cz.cvut.km.model.iterators.*;
import cz.cvut.km.utils.*;

import java.io.*;
import java.util.*;
import java.net.*;

import javax.swing.text.html.parser.*;
import javax.swing.text.html.*;
import javax.swing.text.*;

//import javax.xml.parsers.*;
//import javax.xml.transform.*;
//import javax.xml.transform.stream.*;
//import javax.xml.transform.dom.*;
//import org.w3c.dom.*;

/**
 * BookmarksMozilla
 * @author  aubrech
 */
public class BookmarksMozilla implements IoInterface {
    Vector parents = new Vector();
    KnowledgeBase kb = new KnowledgeBase();
    
    /** Creates a new instance of a BookmarksMozilla I/O. */
    public BookmarksMozilla() {
    }
    
    /**
     * load a knowledge base from the I/O module.
     * @return the loaded knoledge base
     * @param parameters properties to customise the kb load, must contain <pre>source</pre> for the filename, may contain <pre>namespace</pre> with the namespace of the loaded concepts
     * @throws KMException all other exception are covered by KMException (not existing file, bad format, etc.)
     */
    public cz.cvut.km.model.KnowledgeBase loadKB(java.util.Properties parameters) throws cz.cvut.km.utils.KMException {
        try {
            String source = parameters.getProperty("source");
            //System.out.println("Loading "+source);
            //Document doc = DocumentBuilderFactory.newInstance().newDocumentBuilder().parse(source);
            
            kb = new KnowledgeBase();
            parents = new Vector();
            Concept root = new Concept("bookmarks","Bookmark");
            kb.addConcept(root); parents.add(root);
            
            ParserDelegator dp = new ParserDelegator();
            HTMLEditorKit.ParserCallback callback = new HTMLEditorKit.ParserCallback() {
                public static final String STATE_BOOKMARK = "bookmark";
                public static final String STATE_FOLDER = "folder";
                String state = null;
                URL url = null;
                public void handleText(char[] data, int pos) {
                    //System.out.println(data);
                    String str = new String(data);
                    if(state==STATE_BOOKMARK)
                        newBookmark(url,str);
                    else if(state==STATE_FOLDER)
                        newFolder(str);
                }
                public void handleError(String errorMsg, int pos){
                    //System.out.println("error: "+errorMsg);
                }
                public void handleStartTag(javax.swing.text.html.HTML.Tag t, MutableAttributeSet attrSet, int pos) {
                    //System.out.println("<"+t+">");
                    if(t==javax.swing.text.html.HTML.Tag.A) {
                        state = STATE_BOOKMARK;
                        try {url = new URL((String)attrSet.getAttribute(javax.swing.text.html.HTML.Attribute.HREF)); } 
                        catch(MalformedURLException e) {e.printStackTrace();url = null;}
                    } else if(t==javax.swing.text.html.HTML.Tag.H3)
                        state = STATE_FOLDER;
                    //else
                    //    state = null;
                }
                
                public void handleEndTag(javax.swing.text.html.HTML.Tag t, int pos) {
                    //System.out.println("</"+t+">");
                    if(t==javax.swing.text.html.HTML.Tag.DL)
                        parents.removeElementAt(parents.size()-1); // remove the last element
                }
                
                public void handleSimpleTag(javax.swing.text.html.HTML.Tag t, MutableAttributeSet a, int pos) {
                    //System.out.println("<"+t+" />");
                }
            };
            Reader reader = new BufferedReader(new FileReader(source));
            dp.parse(reader,callback,true);
            
            return kb;
        } catch(Exception e) {
            throw new cz.cvut.km.utils.KMException("Unable to load Mozilla bookmarks.",e);
        }
    }
    
    // create a new bookmark
    protected void newBookmark(URL url, String desc) {
        String name = url.toExternalForm();
        Concept bookm = new Concept("bookmarks",name);
        //bookm.setCaption(desc);
        kb.addConcept(bookm);
        kb.addRelation(new InstanceOf(bookm,getLastParent()));
    }
    
    // create a new folder
    protected void newFolder(String name) {
        Concept folder = new Concept("bookmarks",name);
        kb.addConcept(folder); 
        kb.addRelation(new SubclassOf(folder,getLastParent()));
        parents.add(folder);
    }
    
    protected Concept getLastParent() {
        return (Concept)parents.get(parents.size()-1);
    }
    
    /**
     * save a knoledge base
     * @param kb the knowledge base to save
     * @param parameters parameters: filename (file name to save to)
     * @throws IOException IO problems
     * @throws KMException all exceptions are covered by KMException
     */
    public void saveKB(cz.cvut.km.model.KnowledgeBase kb, java.util.Properties parameters) throws java.io.IOException, cz.cvut.km.utils.KMException {
        //throw new cz.cvut.km.utils.NotImplementedException();
        String filename = parameters.get("filename").toString(); // filename to store the content
        
        /*
        try {
            Document doc = DocumentBuilderFactory.newInstance().newDocumentBuilder().newDocument();
            Element root = doc.createElement("knowledgebase");
            ConceptIterator cIt = kb.getConcepts();
            while(cIt.hasNext()) {
                Concept c = cIt.nextConcept();
                saveConceptToXML(root,c);
            }
            RelationIterator rIt = kb.getRelations();
            while(rIt.hasNext()) {
                Relation r = rIt.nextRelation();
                saveRelationToXML(root,r);
            }
            Transformer trans = TransformerFactory.newInstance().newTransformer();
            trans.setOutputProperty(OutputKeys.INDENT,"yes");
            DOMSource src = new DOMSource(root);
            StreamResult res = new StreamResult(new FileWriter(filename));
            trans.transform(src, res);
        } catch(Exception e) {
            throw new KMException("Unable to export ontology to BabelFish XML! "+e);
        }
         */
    }
    
    public boolean supports(String action) {
        return action.equals(SUPPORTS_LOAD);
    }
    
}
