/*
 * FileSystem.java
 *
 * Created on June 15, 2004, 11:22 PM
 */

package cz.cvut.km.io;

import cz.cvut.km.model.*;
import java.io.*;
import java.util.*;

// OGraph
import com.att.gs.codip.ograph.*;
import com.att.gs.codip.ograph.imp.*;
// KAON
import edu.unika.aifb.rdf.api.syntax.*;
import edu.unika.aifb.rdf.api.model.*;
import edu.unika.aifb.rdf.api.util.*;

/**
 * DAML I/O using OGraph
 * @author  aubrech
 */
public class DAML implements IoInterface {
    // process only directories
    protected boolean onlyDirs = false;
    
    /** Creates a new instance of FileSystem */
    public DAML() {
    }
    
    /**
     * save a knoledge base
     */
    public void saveKB(cz.cvut.km.model.KnowledgeBase kb, java.util.Properties parameters) throws java.io.IOException, cz.cvut.km.utils.KMException {
        throw new cz.cvut.km.utils.NotImplementedException();
    }
    
    /**
     * load a knowledge base from the I/O module
     */
    public cz.cvut.km.model.KnowledgeBase loadKB(java.util.Properties parameters) throws java.io.IOException, cz.cvut.km.utils.KMException {
        KnowledgeBase kb = new KnowledgeBase();
        
        String source = parameters.get("source").toString();
        boolean doSystemConcepts = false;
        String doSystemConceptsStr = (String)parameters.get("system-concepts");
        if(doSystemConceptsStr!=null && doSystemConceptsStr.equalsIgnoreCase("true"))
            doSystemConcepts = true;
        Concept cClass = null;
        Concept cProperty = null;
        if(doSystemConcepts) {
            cClass = new Concept("system:daml","Class");
            kb.addConcept(cClass);
            cProperty = new Concept("system:daml","Property");
            kb.addConcept(cProperty);
        }
        
        OGOntology theDAMLCore;
        //OGOntology theOWLCore;
        OntologyManager  theOntoManager = new OntologyManagerImp("RDF2OGRPH");
        theDAMLCore = theOntoManager.getDAML();
        OGOntology theSrcOnto = theOntoManager.createOntology(source);
        
        // first define all classes
        Iterator it = theSrcOnto.classes();
        while(it.hasNext()) {
            OGClass cls = (OGClass)it.next();
            //System.out.println("label: "+cls.getLabel()+", localname: "+cls.getLocalName()+", ns: "+cls.getNamespace());
            Concept concept = new Concept(cls.getNamespace(),cls.getLocalName());
            concept.setCaption(cls.getLocalName());
            kb.addConcept(concept);
            if(doSystemConcepts) {
                kb.addRelation(new InstanceOf(concept,cClass));
            }
        }
        
        // now define all properties
        Iterator itProp = theSrcOnto.properties();
        while(itProp.hasNext()) {
            OGProperty prop = (OGProperty)itProp.next();
            //System.out.println("label: "+prop.getLabel()+", localname: "+prop.getLocalName()+", ns: "+prop.getNamespace());
            Concept concept = new Concept(prop.getNamespace(),prop.getLocalName());
            concept.setCaption(prop.getLocalName());
            kb.addConcept(concept);
            if(doSystemConcepts) {
                kb.addRelation(new InstanceOf(concept,cProperty));
            }
            
            //TODO: process properties of the property!!!
            //OGProperty props[] = prop.getSubPropertyOf();
        }
        
        /*
        //OGUtility.printOntotology(System.out, theSrcOnto);
        // KAON
        RDFSerializer s = new RDFSerializer();
        s.addNamespacePrefix("daml",com.att.gs.codip.ograph.DAML.DAML_NS);
        s.addNamespacePrefix("xsd",XSD.XSD_NS);
        s.addNamespacePrefix("this",theSrcOnto.getURI());
         
        Model theRDFModel;
        try {
            ModelContext ctx = new ModelContext();
            theRDFModel = ctx.getModelPhysical(theSrcOnto.getURI());
            //s.serialize(theRDFModel, System.out, RDFSerializer.DEFAULT_ENCODING);
        }
        catch (ModelException MoEx) {
            System.err.println(" Exception in creating model.");
            System.err.println("Message:"+ MoEx.getLocalizedMessage());
            System.err.println(MoEx);
        }
         */
        
        return kb;
    }
    
    public boolean supports(String action) {
        return action.equals(SUPPORTS_LOAD);
    }
    
}
