/*
 * DAMLJena.java
 *
 * Created on 29. ervenec 2004, 13:22
 */

package cz.cvut.km.io;

import cz.cvut.km.model.*;
import java.io.*;
import java.util.*;

import com.hp.hpl.jena.rdf.model.*;
import com.hp.hpl.jena.ontology.*;


public class DAMLJena implements IoInterface  {
    
    /** name of property of concepts according to DAML */
    public static final String DAMLTYPE = "daml-type";
    /** property of concepts according to DAML -- class*/
    public static final String TYPE_DAML_CLASS = "OwlClass";
    /** property of concepts according to DAML -- class*/
    public static final String TYPE_RDFS_CLASS = "RdfsClass";
    /** property of concepts according to DAML -- Property*/
    public static final String TYPE_PROPERTY = "Property";
    /** property of concepts according to DAML -- DatatypeProperty*/
    public static final String TYPE_DATATYPE_PROPERTY = "DatatypeProperty";
    /** property of concepts according to DAML -- ObjectProperty*/
    public static final String TYPE_OBJECT_PROPERTY = "ObjectProperty";    
   /** property of concepts according to DAML -- Resource*/
    public static final String TYPE_RESOURCE = "Resource";    

    
    // gate-specific concepts
    public Concept cDamlClass = null;
    public Concept cRdfsClass = null;
    public Concept cProperty = null;
    public Concept cObjectProperty = null;
    public Concept cDatatypeProperty = null;
    public Concept cFunctionalProperty = null;
    public Concept cSubProperty = null;
    public Concept cSubClass = null;
    public Concept cDomain = null;
    public Concept cRange = null;
    public Concept cResource = null;
    
     
    /** Creates a new instance of Jena */
    public DAMLJena() {
    }
    
    /**
     * load a knowledge base from the I/O module
     */
    public cz.cvut.km.model.KnowledgeBase loadKB(java.util.Properties parameters) throws java.io.IOException, cz.cvut.km.utils.KMException {
        KnowledgeBase kb = new KnowledgeBase();
        
        String source = parameters.get("source").toString();
        boolean doSystemConcepts = false;
        String doSystemConceptsStr = (String)parameters.get("system-concepts");
        if(doSystemConceptsStr!=null && doSystemConceptsStr.equalsIgnoreCase("true"))
            doSystemConcepts = true;
        
        // add concepts of the system ontology to the knowledge base
        kb = createSystemOntology();
        
        // create ontlogy models
        OntModel model = ModelFactory.createOntologyModel(OntModelSpec.DAML_MEM, null );
        model.read(source);
        
        OntModel modelRdfs = ModelFactory.createOntologyModel(OntModelSpec.RDFS_MEM, null);
        modelRdfs.read(source);
                
        // define all classes
        Iterator it = model.listNamedClasses();
        while(it.hasNext()) {
            OntClass cls = (OntClass)it.next();
            System.out.println("label: "+cls.getLabel(null)+", localname: "+cls.getLocalName()+", ns: "+cls.getNameSpace());
            Concept concept = new Concept(cls.getNameSpace(),cls.getLocalName());
            concept.setCaption(cls.getLocalName());
            if ( cls.getLocalName() != null ){
                // add concept to the knowledge base
                concept.setProperty(DAMLTYPE, TYPE_DAML_CLASS);
                kb.addConcept(concept);
                if(doSystemConcepts) {
                    kb.addRelation(new InstanceOf(concept,cDamlClass));
                }
            }
        }
 
       
        // now define all object properties
        Iterator itProp = model.listObjectProperties();
        loadProperties(kb,itProp,cObjectProperty);

        itProp = model.listTransitiveProperties();
        loadProperties(kb,itProp,cObjectProperty);
        
        // now define all datatype properties
        itProp = model.listDatatypeProperties();
        loadProperties(kb,itProp,cDatatypeProperty);
         
        itProp = model.listOntProperties();
        loadProperties(kb,itProp,cProperty);
        
        // iterate all classes to get subClassOf relations
        it = model.listClasses();
        while(it.hasNext()) {
            OntClass cls = (OntClass)it.next();
            System.out.println(cls.getLocalName());
            if(cls.getLocalName() != null){
                Iterator itSub = cls.listSubClasses(true);
                while(itSub.hasNext()) {
                    OntClass sub = (OntClass)itSub.next();
                    Concept from = kb.getConcept(new Concept(sub.getNameSpace(),sub.getLocalName()));
                    Concept to = kb.getConcept(new Concept(cls.getNameSpace(),cls.getLocalName()));
                    if(sub.getLocalName() != null)
                        kb.addRelation(new SubclassOf(from, to));
                }
            }
         }
        
        // TODO: add superclasses from other non-imported ontologies 
        
        itProp = model.listDatatypeProperties();
        processProperties(kb,itProp, doSystemConcepts);
        
        itProp = model.listObjectProperties();
        processProperties(kb,itProp, doSystemConcepts);
        
        itProp = model.listOntProperties();
        processProperties(kb,itProp, doSystemConcepts);

            
        // write it to standard out
        //model.write(System.out);
       
        return kb;
   
    }
    
    
   /**
    * load properties into knowledge base 
    */
    private void loadProperties(KnowledgeBase kb, Iterator itProp, Concept type){
        System.out.println("Type: "+type.getName());
        while(itProp.hasNext()) {
        OntProperty prop = (OntProperty)itProp.next();
        System.out.println("label: "+prop.getLabel(null)+", localname: "+prop.getLocalName()+", ns: "+prop.getNameSpace());
        Concept concept = new Concept(prop.getNameSpace(),prop.getLocalName());
        concept.setCaption(prop.getLocalName());
        concept.setProperty(DAMLTYPE, type.getProperty(DAMLTYPE));
        kb.addConcept(concept);
        kb.addRelation(new InstanceOf(concept,type));
        }
    }
    
    /**
    * process properties to get subproperties, domain and range
    */
    private void processProperties(KnowledgeBase kb, Iterator itProp, boolean doSystemConcepts){
        while(itProp.hasNext()) {
            OntProperty prop = (OntProperty)itProp.next();
            System.out.println(prop.getLocalName());
            if(prop.getLocalName() != null){
                // get subproperties
                Iterator itSub = prop.listSubProperties(true);
                while(itSub.hasNext()) {
                    OntProperty sub = (OntProperty)itSub.next();
                    Concept from = kb.getConcept(new Concept(sub.getNameSpace(),sub.getLocalName()));
                    Concept to = kb.getConcept(new Concept(prop.getNameSpace(),prop.getLocalName()));
                    if(sub.getLocalName() != null)
                        kb.addRelation(new SubclassOf(from, to));
                }
                // get domain 
                Concept from = kb.getConcept(new Concept(prop.getNameSpace(),prop.getLocalName()));
                OntResource domain = (OntResource)prop.getDomain();
                if(domain != null){
                    Concept to = kb.getConcept(new Concept(domain.getNameSpace(),domain.getLocalName()));
                    System.out.println(domain.getLocalName());
                    if(to == null){
                        to = new Concept(domain.getNameSpace(),domain.getLocalName());
                        to.setProperty(DAMLTYPE, TYPE_DAML_CLASS);
                        kb.addConcept(to);
                       if(doSystemConcepts) 
                            kb.addRelation(new InstanceOf(to,cDamlClass));
                    }
                    kb.addRelation(new HasDomain(from, to));
                }
                // get range
                OntResource range = (OntResource)prop.getRange();
                if(range != null){
                    Concept to = kb.getConcept(new Concept(range.getNameSpace(),range.getLocalName()));
                    System.out.println(range.getLocalName());
                    if (to == null){
                        to = new Concept(range.getNameSpace(),range.getLocalName());
                        to.setProperty(DAMLTYPE, TYPE_DAML_CLASS);
                        kb.addConcept(to);
                        if(doSystemConcepts) 
                            kb.addRelation(new InstanceOf(to,cDamlClass));
                    }
                    kb.addRelation(new HasRange(from, to));
                }
                
            }
        }
    }
    
    public KnowledgeBase createSystemOntology() {
        KnowledgeBase kb = new KnowledgeBase();
        kb.addSystemNamespace("system:daml");
        
        cDamlClass = new Concept("system:daml","Class");
        cDamlClass.setProperty(DAMLTYPE, TYPE_DAML_CLASS);
        kb.addConcept(cDamlClass);
        cObjectProperty = new Concept("system:daml","ObjectProperty");
        cObjectProperty.setProperty(DAMLTYPE, TYPE_OBJECT_PROPERTY);
        kb.addConcept(cObjectProperty);
        cDatatypeProperty = new Concept("system:daml","DatatypeProperty");
        cDatatypeProperty.setProperty(DAMLTYPE, TYPE_DATATYPE_PROPERTY);
        kb.addConcept(cDatatypeProperty);
        cResource = new Concept("system:rdf","resource");
        cResource.setProperty(DAMLTYPE, TYPE_RESOURCE);
        kb.addConcept(cResource);
        cProperty = new Concept("system:rdfs","Property");
        cProperty.setProperty(DAMLTYPE, TYPE_PROPERTY);
        kb.addConcept(cProperty);
          
        
        return kb;
    }
    
    public void saveKB(cz.cvut.km.model.KnowledgeBase kb, java.util.Properties parameters) throws java.io.IOException, cz.cvut.km.utils.KMException {
        throw new cz.cvut.km.utils.NotImplementedException();
    }
    
    public boolean supports(String action) {
        return action.equals(SUPPORTS_LOAD) || action.equals(SUPPORTS_SAVE);
    }
    
}

