/*
 * GraphViz.java
 *
 * Created on May 5, 2004, 1:33 PM
 */

package cz.cvut.km.io;

import cz.cvut.km.model.*;

import java.io.*;
import java.util.*;

/**
 *
 * @author  aubrech
 */
public class GraphViz implements IoInterface {
    
    /** Creates a new instance of GraphViz */
    public GraphViz() {
        
    }
    
    /**
     * load a knowledge base from the I/O module
     */
    public cz.cvut.km.model.KnowledgeBase loadKB(java.util.Properties parameters) throws java.io.IOException, cz.cvut.km.utils.KMException {
        throw new cz.cvut.km.utils.NotImplementedException();
    }
    
    /**
     * save a knoledge base
     * @param parameters use: filename for the file to save the content, showNS (TRUE/false) for showing/hiding namespace
     */
    public void saveKB(cz.cvut.km.model.KnowledgeBase kb, java.util.Properties parameters) throws java.io.IOException, cz.cvut.km.utils.KMException {
        String filename = (String)parameters.get("filename");
        //String headers = (String)parameters.get("headers");
        boolean showNS = ((String)parameters.getProperty("showNS","true")).equalsIgnoreCase("true");
        PrintWriter pw = new PrintWriter(new FileWriter(filename));
        
        pw.println("// generated by GraphViz, author Petr Aubrecht and Monika Zakova, CTU in Prague");
        pw.println("// "+new java.util.Date().toString());
        pw.println("digraph {");
        //pw.println("size =\"10,5\"; ratio=compress;");
        pw.println("node [shape=box];");
        pw.println("// CLASSES *****************************************");
        
        Iterator it = kb.getConcepts();
        while(it.hasNext()) {
            Concept c = (Concept)it.next();
            Vector rs = kb.getRelationsFrom(c);
            String shape = "";
            for(int i = 0; i < rs.size(); i++){
                Relation r = (Relation)rs.elementAt(i);
                if(r instanceof HasDomain || r instanceof HasRange || r instanceof HasValue || r instanceof PropertyOf){
                    shape = ",shape=ellipse";
                    break;
                }
            }
            String cColor = c.getProperty("color");
            if(cColor==null)
                cColor="";
            else
                cColor="color="+cColor;
            pw.println("\""+(showNS?c.toString():c.getName())+"\" [label=\""+c.getCaption()+"\""+shape+","+cColor+"];");
            
            Iterator rels = kb.getRelationsFrom(c).iterator();
            while(rels.hasNext()) {
                Relation r = (Relation)rels.next();
                // set the color in GraphViz form
                String color = r.getProperty("color");
                if(color==null)
                    color="";
                else
                    color="color="+color;
                String source = showNS ? r.getSource().toString() : r.getSource().getName();
                String target = showNS ? r.getTarget().toString() : r.getTarget().getName();
                if(r instanceof SubclassOf) {
                    pw.println("\""+source+"\" -> \""+target+"\" ["+color+"];");
                } else if(r instanceof InstanceOf) {
                    pw.println("\""+source+"\" -> \""+target+"\" [arrowhead=odot, "+color+"];");
                } else if(r instanceof HasDomain) {
                    pw.println("\""+source+"\" -> \""+target+"\" [arrowhead=inv,"+color+"];");
                } else if(r instanceof HasRange) {
                    pw.println("\""+source+"\" -> \""+target+"\" [arrowhead=box,"+color+"];");
                } else if(r instanceof HasValue) {
                    pw.println("\""+source+"\" -> \""+target+"\" [arrowhead=open, "+color+"];");
                } else if(r instanceof PropertyOf) {
                    pw.println("\""+source+"\" -> \""+target+"\" [arrowhead=crow, "+color+"];");
                } else {
                    pw.println("// !!! Warning: unknown relation: "+r.toString());
                    pw.println("\""+source+"\" -> \""+target+"\" ["+color+"];");
                }
            }
        }
        pw.println("}");
        
        pw.close();
    }
    
    public boolean supports(String action) {
        return action.equals(SUPPORTS_SAVE);
    }
    
}
