/*
 * JDBC.java
 *
 * Created on June 18, 2004, 3:58 PM
 */

package cz.cvut.km.io;

import cz.cvut.km.model.*;
import java.sql.*;

/**
 *
 * @author  aubrech
 */
public class JDBC implements IoInterface {
    
    /** Creates a new instance of JDBC */
    public JDBC() {
    }
    
    /**
     * load a knowledge base from the I/O module
     */
    public cz.cvut.km.model.KnowledgeBase loadKB(java.util.Properties parameters) throws java.io.IOException, cz.cvut.km.utils.KMException {
        try {
            KnowledgeBase kb = new KnowledgeBase();
            String doSystemConceptsStr = (String)parameters.get("system-concepts");
            boolean doSystemConcepts = false;
            if(doSystemConceptsStr!=null && doSystemConceptsStr.equalsIgnoreCase("true"))
                doSystemConcepts = true;
            // add system ontology
            Concept conceptTable = null;
            Concept conceptField = null;
            Concept conceptHasField = null;
            if(doSystemConcepts) {
                conceptTable = new Concept("system:jdbc","TABLE");
                kb.addConcept(conceptTable);
                conceptField = new Concept("system:jdbc","FIELD");
                kb.addConcept(conceptField);
                conceptHasField = new Concept("system:jdbc","HAS-FIELD");
                kb.addConcept(conceptHasField);
                kb.addRelation(new HasDomain(conceptHasField,conceptTable));
                kb.addRelation(new HasRange(conceptHasField,conceptField));
            }
            //throw new cz.cvut.km.utils.NotImplementedException();
            
            // create connection to the database
            Class.forName((String)parameters.get("driver"));
            Connection conn = DriverManager.getConnection((String)parameters.get("url"),(String)parameters.get("username"),(String)parameters.get("password"));
            
            
            // browse db metadata
            DatabaseMetaData dbmd = conn.getMetaData();
            ResultSet rsmd = dbmd.getTables(null, null, null, new String[] {"TABLE","VIEW"});
            while(rsmd.next()) {
                String tableName = rsmd.getString("TABLE_NAME");
                Concept conceptPTable = new Concept(null,tableName);
                kb.addConcept(conceptPTable);
                if(doSystemConcepts)
                    kb.addRelation(new InstanceOf(conceptPTable,conceptTable));
                // continue for columns
                ResultSet rscol = dbmd.getColumns(null, null, tableName, null);
                while(rscol.next()) {
                    String columnName = rscol.getString("COLUMN_NAME");
                    Concept conceptColumn = new Concept(null, tableName+"--"+columnName);
                    conceptColumn.setCaption(columnName);
                    kb.addConcept(conceptColumn);
                    if(doSystemConcepts)
                        kb.addRelation(new InstanceOf(conceptColumn,conceptField));
                    Concept conceptAssignment = new Concept(null,"has-field-"+tableName+"-"+columnName);
                    conceptAssignment.setCaption("");
                    kb.addConcept(conceptAssignment);
                    if(doSystemConcepts)
                        kb.addRelation(new InstanceOf(conceptAssignment,conceptHasField));
                    kb.addRelation(new PropertyOf(conceptPTable,conceptAssignment));
                    kb.addRelation(new HasValue(conceptAssignment,conceptColumn));
                }
                rscol.close();
            }
            rsmd.close();
            
            // finish -- close and exit
            conn.close();
            return kb;
        } catch (ClassNotFoundException e) {
            java.io.IOException ex = new java.io.IOException(e.getMessage());
            ex.initCause(e);
            throw ex;
        } catch (SQLException e) {
            java.io.IOException ex = new java.io.IOException(e.getMessage());
            ex.initCause(e);
            throw ex;
        }
    }
    
    /**
     * save a knoledge base
     */
    public void saveKB(cz.cvut.km.model.KnowledgeBase kb, java.util.Properties parameters) throws java.io.IOException, cz.cvut.km.utils.KMException {
        throw new cz.cvut.km.utils.NotImplementedException();
    }
    
    public boolean supports(String action) {
        return action.equals(SUPPORTS_LOAD);
    }
    
}
