/*
 * owl.java
 *
 * Created on May 5, 2004, 12:37 PM
 */

package cz.cvut.km.io;

import cz.cvut.km.model.*;
import java.io.*;
import java.util.*;

// OGraph
import com.att.gs.codip.ograph.*;
import com.att.gs.codip.ograph.imp.*;
import com.att.gs.codip.resource.*;
// KAON
import edu.unika.aifb.rdf.api.syntax.*;
import edu.unika.aifb.rdf.api.model.*;
import edu.unika.aifb.rdf.api.util.*;

/**
 * OWL I/O using OGraph
 * @author  aubrech
 */
public class Owl implements IoInterface {
    /** split names by hash mark */
    public static String SPLIT_BY_HASH = "#";
    /** split names by the path delimiter (as a local name use the last part of the path) */
    public static String SPLIT_BY_PATH = "#";
    /** type of name/namespace splitting:
     * # - use # mark as a delimiter
     * lastpart - the localname is the last part of the URI
     */
    protected String nameSplitting = "#"; 
    
    /** Creates a new instance of owl */
    public Owl() {
    }
    
    /**
     * load a knowledge base from the I/O module
     */
    public cz.cvut.km.model.KnowledgeBase loadKB(java.util.Properties parameters) throws java.io.IOException, cz.cvut.km.utils.KMException {
        KnowledgeBase kb = new KnowledgeBase();
                
        String source = parameters.get("source").toString();
        nameSplitting = parameters.getProperty("nameparsing","#");
        boolean doSystemConcepts = false;
        String doSystemConceptsStr = (String)parameters.get("system-concepts");
        if(doSystemConceptsStr!=null && doSystemConceptsStr.equalsIgnoreCase("true"))
            doSystemConcepts = true;
        Concept cClass = null;
        Concept cProperty = null;
        if(doSystemConcepts) {
            cClass = new Concept("system:owl","Class");
            kb.addConcept(cClass);
            cProperty = new Concept("system:owl","Property");
            kb.addConcept(cProperty);
        }
        
        //OGOntology theOwlCore;
        OntologyManager  theOntoManager = new OntologyManagerImp("RDF2OGRPH");
        //theOwlCore = theOntoManager.getOWL();
        OGOntology theSrcOnto = theOntoManager.createOntology(source);
        
        // first define all classes
        Iterator it = theSrcOnto.classes();
        while(it.hasNext()) {
            OGClass cls = (OGClass)it.next();
            String label = cls.getLabel();
            String ns = identsOwl2BFNamespace(cls);
            String ln = identsOwl2BFLocalName(cls);
            System.out.println("label: "+label+", localname: "+ln+", ns: "+ns+", ID="+cls.getID());
            Concept concept = new Concept(ns,ln);
            concept.setCaption(ln);
            kb.addConcept(concept);
            if(doSystemConcepts) {
                kb.addRelation(new InstanceOf(concept,cClass));
            }
        }
        
        // now define all properties
        Iterator itProp = theSrcOnto.properties();
        while(itProp.hasNext()) {
            OGProperty prop = (OGProperty)itProp.next();
            //System.out.println("label: "+prop.getLabel()+", localname: "+prop.getLocalName()+", ns: "+prop.getNamespace());
            String ns = identsOwl2BFNamespace(prop);
            String ln = identsOwl2BFLocalName(prop);
            Concept concept = new Concept(ns,ln);
            concept.setCaption(ln);
            kb.addConcept(concept);
            if(doSystemConcepts) {
                if(prop.pInstance())
                    // assignment 
                    ;//kb.addRelation(new InstanceOf(concept,cProperty));
                else
                    // definition of property
                    kb.addRelation(new InstanceOf(concept,cProperty));
            }
            OGPropInstance propsi[] = prop.getPropInstances();
            for(int i=0;i<propsi.length;i++) {
                OGPropInstance propi = propsi[i];
                //if(prop.getSubject()!=cls)
                //    continue; // e.g. some system properties (property ?->? owl:Class?
                //System.out.println("localName:"+prop.getLocalName());
                //System.out.println("type:"+prop.getType());
                //System.out.println("id:"+prop.getID());
                //System.out.println("label:"+prop.getLabel());
                LogicalResource type = propi.getType();
                //System.out.println("type ns:"+type.getNamespace());
                //System.out.println("type local name: "+type.getLocalName());
                //System.out.println("subject: "+propi.getSubject().getLocalName());
                System.out.println("property: "+propi.getSubject().getID()+" -"+type.getLocalName()+"-> "+propi.getObject().getID());
                
                OGResource subject = propi.getSubject();
                OGResource object = propi.getObject();
                Concept cSubject = kb.getConcept(identsOwl2BF(subject));
                Concept cObject = kb.getConcept(identsOwl2BF(object));
                if(type==RDFS.range)
                    kb.addRelation(new HasRange(cSubject,cObject));
                if(type==RDFS.domain)
                    kb.addRelation(new HasDomain(cSubject,cObject));
                //TODO: other types!!! type, subclassOf,...
            }
        }
        
        // iterate all classes for additional info
        it = theSrcOnto.classes();
        while(it.hasNext()) {
            OGClass cls = (OGClass)it.next();
            OGClass superclasses[] = cls.getSubClassOf();
            for(int i=0;i<superclasses.length;i++) {
                Concept from = kb.getConcept(identsOwl2BF(cls));
                Concept to = kb.getConcept(identsOwl2BF(superclasses[i]));
                kb.addRelation(new SubclassOf(from, to));
            }
        }
        
        // iterate properties of properties
        itProp = theSrcOnto.properties();
        while(itProp.hasNext()) {
            OGProperty prop = (OGProperty)itProp.next();
            
            // add superproperties
            OGProperty props[] = prop.getSubPropertyOf();
            for(int i=0;i<props.length;i++) {
                Concept from = kb.getConcept(identsOwl2BF(prop));
                Concept to = kb.getConcept(identsOwl2BF(props[i]));
                kb.addRelation(new SubclassOf(from, to));
            }
        }
        
        return kb;
    }
    
    /** return a name of the OGraph object in BabelFish manner
     */
    protected Concept identsOwl2BF(OGResource obj) {
        return new Concept(identsOwl2BFNamespace(obj),identsOwl2BFLocalName(obj));
        //return obj.getNamespace()+":"+obj.getLocalName();
    }
    
    /** extrace namespace from the OGResource
     */
    protected String identsOwl2BFNamespace(OGResource cls) {
        try {
            java.net.URL url = new java.net.URL(cls.getID());
            File fpath = new File(url.getPath());
            String path = fpath.getParent();
            if(url.getRef()!=null)
                return new java.net.URL(url.getProtocol(),url.getHost(),url.getPort(),url.getPath()).toString();
            return new java.net.URL(url.getProtocol(),url.getHost(),url.getPort(),path).toString();
        } catch(java.net.MalformedURLException e) {
            return cls.getNamespace();
        }
    }
    
    /** extrace localname from the OGResource
     */
    protected String identsOwl2BFLocalName(OGResource cls) {
        try {
            java.net.URL url = new java.net.URL(cls.getID());
            if(url.getRef()!=null)
                return url.getRef();
            File fpath = new File(url.getPath());
            return fpath.getName();
        } catch(java.net.MalformedURLException e) {
            return cls.getLocalName();
        }
    }
    
    /**
     * save a knoledge base
     */
    public void saveKB(cz.cvut.km.model.KnowledgeBase kb, java.util.Properties parameters) throws java.io.IOException, cz.cvut.km.utils.KMException {
        throw new cz.cvut.km.utils.NotImplementedException();
    }
    
    public boolean supports(String action) {
        return action.equals(SUPPORTS_LOAD);
    }
    
}
