/*
 * Jena.java
 *
 * Created on 24. ervenec 2004, 0:39
 */

package cz.cvut.km.io;

import cz.cvut.km.model.*;
import java.io.*;
import java.util.*;

import com.hp.hpl.jena.rdf.model.*;
import com.hp.hpl.jena.ontology.*;


public class OwlJena implements IoInterface  {
    
    /** name of property of concepts according to OWL */
    public static final String OWLTYPE = "owl-type";
    /** property of concepts according to OWL -- class*/
    public static final String TYPE_OWL_CLASS = "OwlClass";
    /** property of concepts according to OWL -- class*/
    public static final String TYPE_RDFS_CLASS = "RdfsClass";
    /** property of concepts according to OWL -- Property*/
    public static final String TYPE_PROPERTY = "Property";
    /** property of concepts according to OWL -- DatatypeProperty*/
    public static final String TYPE_DATATYPE_PROPERTY = "DatatypeProperty";
    /** property of concepts according to OWL -- ObjectProperty*/
    public static final String TYPE_OBJECT_PROPERTY = "ObjectProperty";    
    /** property of concepts according to OWL -- FunctionalProperty*/
    public static final String TYPE_FUNCTIONAL_PROPERTY = "FunctionalProperty";   
    /** property of concepts according to OWL -- TransitiveProperty*/
    public static final String TYPE_TRANSITIVE_PROPERTY = "TransitiveProperty";    
 
   /** property of concepts according to OWL -- Resource*/
    public static final String TYPE_RESOURCE = "Resource";
    public static final String TYPE_OWL_INSTANCE = "OwlInstance";
    public static final String TYPE_RDFS_INSTANCE = "RdfsInstance";
    public static final String TYPE_ASSIGNMENT = "Assignment";
    public static final String TYPE_LITERAL = "Literal";
    public static final String TYPE_DATATYPE = "Datatype";
    public static final String TYPE_RDF_TYPE = "Type";
    public static final String TYPE_THING = "Thing";
    public static final String TYPE_RESTRICTION = "Restriction";
    public static final String TYPE_LIST_MEMBER = "ListMember";
    
    // gate-specific concepts
    public Concept cClass = null;
    public Concept cProperty = null;
    public Concept cObjectProperty = null;
    public Concept cDatatypeProperty = null;
    public Concept cFunctionalProperty = null;
    public Concept cSubProperty = null;
    public Concept cSubClass = null;
    public Concept cDomain = null;
    public Concept cRange = null;
    public Concept cResource = null;
    public Concept cThing = null;
    
     
    /** Creates a new instance of Jena */
    public OwlJena() {
    }
    
    /**
     * load a knowledge base from the I/O module
     */
    public cz.cvut.km.model.KnowledgeBase loadKB(java.util.Properties parameters) throws java.io.IOException, cz.cvut.km.utils.KMException {
        KnowledgeBase kb = new KnowledgeBase();
        
        String source = parameters.get("source").toString();
        boolean doSystemConcepts = false;
        String doSystemConceptsStr = (String)parameters.get("system-concepts");
        if(doSystemConceptsStr!=null && doSystemConceptsStr.equalsIgnoreCase("true"))
            doSystemConcepts = true;
        if(doSystemConcepts) {
            kb = createSystemOntology();
        }
        
        OntModel model = ModelFactory.createOntologyModel(OntModelSpec.OWL_MEM, null );
        model.read(source);
                
        // define all classes
        Iterator it = model.listNamedClasses();
        while(it.hasNext()) {
            OntClass cls = (OntClass)it.next();
            System.out.println("label: "+cls.getLabel(null)+", localname: "+cls.getLocalName()+", ns: "+cls.getNameSpace());
            Concept concept = new Concept(cls.getNameSpace(),cls.getLocalName());
            concept.setCaption(cls.getLocalName());
            if ( cls.getLocalName() != null ){
                // add concept to the knowledge base
                concept.setProperty(OWLTYPE, TYPE_OWL_CLASS);
                kb.addConcept(concept);
                if(doSystemConcepts) {
                    kb.addRelation(new InstanceOf(concept,cClass));
                }
            }
        }
        
        
        // now define all object properties
        Iterator itProp = model.listObjectProperties();
        loadProperties(kb,itProp,cObjectProperty);
        
        itProp = model.listSymmetricProperties();
        loadProperties(kb,itProp,cObjectProperty);

        itProp = model.listInverseFunctionalProperties();
        loadProperties(kb,itProp,cObjectProperty);

        itProp = model.listTransitiveProperties();
        loadProperties(kb,itProp,cObjectProperty);
        
        // now define all datatype properties
        itProp = model.listDatatypeProperties();
        //loadProperties(kb,itProp,cDatatypeProperty);
        
        // now define all functional properties
        itProp = model.listFunctionalProperties();
        loadProperties(kb,itProp,cFunctionalProperty);
        
        // now define all annotation properties
        //itProp = model.listAnnotationProperties();
        //loadProperties(kb,itProp,cAnnotaionProperty);
         
        itProp = model.listOntProperties();
        loadProperties(kb,itProp,cProperty);
        
        // iterate all classes to get subClassOf relations
        it = model.listClasses();
        while(it.hasNext()) {
            OntClass cls = (OntClass)it.next();
            System.out.println(cls.getLocalName());
            if(cls.getLocalName() != null){
                Iterator itSub = cls.listSubClasses(true);
                while(itSub.hasNext()) {
                    OntClass sub = (OntClass)itSub.next();
                    Concept from = kb.getConcept(new Concept(sub.getNameSpace(),sub.getLocalName()));
                    Concept to = kb.getConcept(new Concept(cls.getNameSpace(),cls.getLocalName()));
                    if(sub.getLocalName() != null)
                        kb.addRelation(new SubclassOf(from, to));
                }
            }
         }
        
        // iterate all classes to get superClasses, even from different ontology
        /*it = model.listClasses();
        while(it.hasNext()) {
            OntClass cls = (OntClass)it.next();
            System.out.println(cls.getLocalName());
            if(cls.getLocalName() != null){
                Iterator itSuper = cls.listSuperClasses(true);
                while(itSuper.hasNext()) {
                    OntResource sup = (OntResource)itSuper.next();
                    Concept from = kb.getConcept(cls.getNameSpace()+":"+cls.getLocalName());
                    Concept to = kb.getConcept(sup.getNameSpace()+":"+sup.getLocalName());
                    if(to == null)
                        to = new Concept(sup.getNameSpace(),sup.getLocalName());
                    if(sup.getLocalName() != null)
                        kb.addRelation(new SubclassOf(from, to));
                    }
             }
        }*/
        
        itProp = model.listDatatypeProperties();
        processProperties(kb,itProp, doSystemConcepts);
        
        itProp = model.listObjectProperties();
        processProperties(kb,itProp, doSystemConcepts);
        
        itProp = model.listSymmetricProperties();
        processProperties(kb,itProp, doSystemConcepts);
        
        itProp = model.listOntProperties();
        processProperties(kb,itProp, doSystemConcepts);

            
        // write it to standard out
        //model.write(System.out);
       
        return kb;
   
    }
    
    
   /**
    * load properties into knowledge base 
    */
    private void loadProperties(KnowledgeBase kb, Iterator itProp, Concept type){
        System.out.println("Type: "+type.getName());
        while(itProp.hasNext()) {
        OntProperty prop = (OntProperty)itProp.next();
        System.out.println("label: "+prop.getLabel(null)+", localname: "+prop.getLocalName()+", ns: "+prop.getNameSpace());
        Concept concept = new Concept(prop.getNameSpace(),prop.getLocalName());
        concept.setCaption(prop.getLocalName());
        concept.setProperty(OWLTYPE, type.getProperty(OWLTYPE));
        kb.addConcept(concept);
        kb.addRelation(new InstanceOf(concept,type));
        }
    }
    
    /**
    * process properties to get subproperties, domain and range
    */
    private void processProperties(KnowledgeBase kb, Iterator itProp, boolean doSystemConcepts){
        while(itProp.hasNext()) {
            OntProperty prop = (OntProperty)itProp.next();
            System.out.println(prop.getLocalName());
            if(prop.getLocalName() != null){
                // get subproperties
                Iterator itSub = prop.listSubProperties(true);
                while(itSub.hasNext()) {
                    OntProperty sub = (OntProperty)itSub.next();
                    Concept from = kb.getConcept(new Concept(sub.getNameSpace(),sub.getLocalName()));
                    Concept to = kb.getConcept(new Concept(prop.getNameSpace(),prop.getLocalName()));
                    if(sub.getLocalName() != null)
                        kb.addRelation(new SubclassOf(from, to));
                }
                // get domain 
                Concept from = kb.getConcept(new Concept(prop.getNameSpace(),prop.getLocalName()));
                OntResource domain = (OntResource)prop.getDomain();
                if(domain != null){
                    Concept to = kb.getConcept(new Concept(domain.getNameSpace(),domain.getLocalName()));
                    System.out.println(domain.getLocalName());
                    if(to == null){
                        to = new Concept(domain.getNameSpace(),domain.getLocalName());
                        to.setProperty(OWLTYPE, TYPE_OWL_CLASS);
                        kb.addConcept(to);
                       if(doSystemConcepts) 
                            kb.addRelation(new InstanceOf(to,cClass));
                    }
                    kb.addRelation(new HasDomain(from, to));
                }
                // get range
                OntResource range = (OntResource)prop.getRange();
                if(range != null){
                    Concept to = kb.getConcept(new Concept(range.getNameSpace(),range.getLocalName()));
                    System.out.println(range.getLocalName());
                    if (to == null){
                        to = new Concept(range.getNameSpace(),range.getLocalName());
                        to.setProperty(OWLTYPE, TYPE_OWL_CLASS);
                        kb.addConcept(to);
                        if(doSystemConcepts) 
                            kb.addRelation(new InstanceOf(to,cClass));
                    }
                    kb.addRelation(new HasRange(from, to));
                }
                
            }
        }
    }
    
    public KnowledgeBase createSystemOntology() {
        KnowledgeBase kb = new KnowledgeBase();
        kb.addSystemNamespace("system:owl");
        kb.addSystemNamespace("system:rdf");
        kb.addSystemNamespace("system:rdfs");
        
        cClass = new Concept("system:owl","Class");
        cClass.setProperty(OWLTYPE, TYPE_OWL_CLASS);
        kb.addConcept(cClass);
        cObjectProperty = new Concept("system:owl","ObjectProperty");
        cObjectProperty.setProperty(OWLTYPE, TYPE_OBJECT_PROPERTY);
        kb.addConcept(cObjectProperty);
        cDatatypeProperty = new Concept("system:owl","DatatypeProperty");
        cDatatypeProperty.setProperty(OWLTYPE, TYPE_DATATYPE_PROPERTY);
        kb.addConcept(cDatatypeProperty);
        cFunctionalProperty = new Concept("system:owl","FunctionalProperty");
        cFunctionalProperty.setProperty(OWLTYPE, TYPE_FUNCTIONAL_PROPERTY);
        kb.addConcept(cFunctionalProperty);
        cResource = new Concept("system:rdf","resource");
        cResource.setProperty(OWLTYPE, TYPE_RESOURCE);
        kb.addConcept(cResource);
        cProperty = new Concept("system:rdfs","Property");
        cProperty.setProperty(OWLTYPE, TYPE_PROPERTY);
        kb.addConcept(cProperty);
        cThing = new Concept("system:owl","Thing");
                
        /*
        cSubProperty = new Concept("system:rdfs","subPropertyOf");
        kb.addConcept(cSubProperty);
        cSubClass = new Concept("system:rdfs","subClassOf");
        kb.addConcept(cSubClass);
        cDomain = new Concept("system:rdfs","domain");
        kb.addConcept(cDomain);
        cRange = new Concept("system:rdfs","range");
        kb.addConcept(cRange);
        */
        
        return kb;
    }
    
    public void saveKB(cz.cvut.km.model.KnowledgeBase kb, java.util.Properties parameters) throws java.io.IOException, cz.cvut.km.utils.KMException {
        throw new cz.cvut.km.utils.NotImplementedException();
    }
    
    public boolean supports(String action) {
        return action.equals(SUPPORTS_LOAD);
    }
    
}
