/*
 * OwlRdfsJena.java
 *
 * Created on 29. ervenec 2004, 11:09
 */

package cz.cvut.km.io;

import cz.cvut.km.model.*;
import cz.cvut.km.model.iterators.*;
import cz.cvut.km.utils.*;
import java.io.*;
import java.util.*;
import java.net.*;

import com.hp.hpl.jena.rdf.model.*;
import com.hp.hpl.jena.ontology.*;
import com.hp.hpl.jena.ontology.impl.OntPropertyImpl;
import com.hp.hpl.jena.vocabulary.*;

/**
 * default constructor
 */
public class OwlRdfsJena implements IoInterface  {
    protected boolean dodebug = true;
    
        /** name of property of concepts according to OWL */
    public static final String OWLTYPE = "owl-type";

    public static final String OWL_MODEL = "Owl";
    
    public static final String RDFS_MODEL = "Rdfs";
    
    public static final String TYPE_CLASS = "class";
    /** property of concepts according to Apollo -- instance*/
    public static final String TYPE_INSTANCE = "instance";
    /** property of concepts according to Apollo -- property (slot)*/
    public static final String TYPE_PROPERTY = "property";
    /** property of concepts according to Apollo -- assignment (value to slot)*/
    public static final String TYPE_ASSIGNMENT = "assignment";
    
    public static final String TYPE_LITERAL = "literal";
    
    public static final String SYSTEM = "system:";

    
    /**
     * property storing value of a literal
     */    
    public static final String VALUE = "Value";
    
    // gate-specific concepts
    /**
     * concept representing owl:Class
     */    
    public static Concept cOwlClass = null;
    /**
     * concept representing rdfs:Class
     */    
    public static Concept cRdfsClass = null;
    /**
     * concept representing rdf:Property
     */    
    public static Concept cProperty = null;
    /**
     * concept representing owl:ObjectProperty
     */    
    public static Concept cObjectProperty = null;
    /**
     * concept representing owl:DatatypeProperty
     */    
    public static Concept cDatatypeProperty = null;
    /**
     * concept representing owl:FunctionalProperty
     */    
    public static Concept cFunctionalProperty = null;
    public static Concept cTransitiveProperty = null;
    /**
     * concept representing rdfs:subProperty
     */    
    public static Concept cSubProperty = null;
    /**
     * concept representing rdfs:subClass
     */    
    public static Concept cSubClass = null;
    /**
     * concept representing rdfs:domain
     */    
    public static Concept cDomain = null;
    /**
     * concept representing rdfs:range
     */    
    public static Concept cRange = null;
    /**
     * concept representing rdf:Resource
     */    
    public static Concept cResource = null;
    
    /**
     * concept representing rdf:Datatype
     */    
    public static Concept cDatatype = null;
    
    /**
     * concept representing rdf:Literal
     */    
    public static Concept cLiteral = null;
    /**
     * concept representing rdf:Type
     */    
    public static Concept cType = null;
    /**
     * concept representing owl:Thing
     */    
    public static Concept cThing = null;
    /**
     * concept representing owl:Restriction
     */    
    public static Concept cRestriction = null;
    /**
     * concept representing owl:onProperty
     */    
    public static Concept cOnProperty = null;
    /**
     * concept representing owl:hasValue
     */    
    public static Concept cHasValue = null;
    /**
     * concept representing owl:allValuesFrom
     */    
    public static Concept cAllValuesFrom = null;
    /**
     * concept representing owl:someValuesFrom
     */    
    public static Concept cSomeValuesFrom = null;
    /**
     * concept representing owl:intersectionOf
     */    
    public static Concept cIntersectionOf = null;
    /**
     * concept representing owl:unionOf
     */    
    public static Concept cUnionOf = null;
    
    public static Concept cOneOf = null;
    
    /**
     * owl namespace
     */    
    public static String owlNS = null;
    /**
     * rdfs namespace
     */    
    public static String rdfsNS = null;
    /**
     * rdf namespace
     */    
    public static String rdfNS = null;
    
    protected String baseNS = "http://www.cvut.cz/test_out.owl#";
     
    /** Creates a new instance of Jena */
    public OwlRdfsJena() {
    }
    
   
    /**
     * lad owl ontology into the knowledge base
     * @param parameters parameters e.g. source of the ontology
     * @throws IOException cannot load the ontology from source
     * @throws KMException cannot invalid ontology
     * @return knowledge base
     */    
    public cz.cvut.km.model.KnowledgeBase loadKB(java.util.Properties parameters) throws java.io.IOException, cz.cvut.km.utils.KMException {
        KnowledgeBase kb = new KnowledgeBase();
        
        String source = parameters.get("source").toString();
        boolean doSystemConcepts = false;
        String doSystemConceptsStr = parameters.getProperty("system-concepts","false");
        if(doSystemConceptsStr.equalsIgnoreCase("true"))
            doSystemConcepts = true;
                        
        OntModel model = ModelFactory.createOntologyModel(OntModelSpec.OWL_MEM, null );
	if(dodebug) System.out.println("DEBUG: OWL parsing start -- "+new Date());
        model.read(source);
        owlNS = model.getNsPrefixURI("owl");
                        
	if(dodebug) System.out.println("DEBUG: RDFS parsing start -- "+new Date());
        OntModel modelRdfs = ModelFactory.createOntologyModel(OntModelSpec.RDFS_MEM, null);
        modelRdfs.read(source);
        rdfsNS = modelRdfs.getNsPrefixURI("rdfs");
        rdfNS = modelRdfs.getNsPrefixURI("rdf");
	if(dodebug) System.out.println("DEBUG: OWL parsing end -- "+new Date());
        
        // create ontology defining the used model
        kb = createSystemOntology(doSystemConcepts);
        
        if(dodebug) System.out.println("DEBUG: OWL define  owl classes -- "+new Date());
        // define  owl classes
        Iterator it = model.listNamedClasses();
        while(it.hasNext()) {
            OntClass cls = (OntClass)it.next();
            //System.out.println("label: "+cls.getLabel(null)+", localname: "+cls.getLocalName()+", ns: "+cls.getNameSpace());
            Concept concept = new Concept(cls.getNameSpace(),cls.getLocalName());
            concept.setCaption(cls.getLocalName());
            concept.setProperty(OwlJena.OWLTYPE, OwlJena.TYPE_OWL_CLASS);
            // add concept to the knowledge base
            kb.addConcept(concept);
            if(doSystemConcepts) {
                kb.addRelation(new InstanceOf(concept,cOwlClass));
            }
        }
        
        if(dodebug) System.out.println("DEBUG: OWL define rdfs classes -- "+new Date());
        // define rdfs classes
        it = modelRdfs.listClasses();
        while(it.hasNext()) {
            OntClass cls = (OntClass)it.next();
            //System.out.println("label: "+cls.getLabel(null)+", localname: "+cls.getLocalName()+", ns: "+cls.getNameSpace());
            Concept concept = new Concept(cls.getNameSpace(),cls.getLocalName());
            concept.setCaption(cls.getLocalName());
            if ( cls.getLocalName() != null ){
                // add concept to the knowledge base
                concept.setProperty(OwlJena.OWLTYPE, OwlJena.TYPE_RDFS_CLASS);
                kb.addConcept(concept);
                if(doSystemConcepts) {
                    kb.addRelation(new InstanceOf(concept,cRdfsClass));
                }
            }
        }
        
        if(dodebug) System.out.println("DEBUG: OWL now define all object properties -- "+new Date());
        // now define all object properties
        Iterator itProp = model.listObjectProperties();
        loadProperties(kb,itProp,cObjectProperty, doSystemConcepts);
        
        if(dodebug) System.out.println("DEBUG: OWL now define all datatype properties -- "+new Date());
        // now define all datatype properties
        itProp = model.listDatatypeProperties();
        loadProperties(kb,itProp,cDatatypeProperty, doSystemConcepts);
        
        // now define all annotation properties
        //itProp = model.listAnnotationProperties();
        //loadProperties(kb,itProp,cAnnotaionProperty);
         
        if(dodebug) System.out.println("DEBUG: OWL loadProperties -- "+new Date());
        itProp = modelRdfs.listOntProperties();
        loadProperties(kb,itProp,cProperty, doSystemConcepts);
        
        if(dodebug) System.out.println("DEBUG: OWL iterate all OWL classes to get subClassOf relations -- "+new Date());
        // iterate all classes to get subClassOf relations
        it = model.listClasses();
        while(it.hasNext()) {
            OntClass cls = (OntClass)it.next();
            //System.out.println(cls.getLocalName());
            if(cls.getLocalName() != null){
                Iterator itSub = cls.listSubClasses(true);
                while(itSub.hasNext()) {
                    OntClass sub = (OntClass)itSub.next();
                    Concept from = kb.getConcept(new Concept(sub.getNameSpace(),sub.getLocalName()));
                    Concept to = kb.getConcept(new Concept(cls.getNameSpace(),cls.getLocalName()));
                    if(sub.getLocalName() != null)
                        kb.addRelation(new SubclassOf(from, to));
                }
            }
         }
        
        if(dodebug) System.out.println("DEBUG: OWL iterate all RDFS classes to get subClassOf relations -- "+new Date());
        // iterate all classes to get subClassOf relations
        it = modelRdfs.listClasses();
        while(it.hasNext()) {
            OntClass cls = (OntClass)it.next();
            //System.out.println(cls.getLocalName());
            if(cls.getLocalName() != null){
                Iterator itSub = cls.listSubClasses(true);
                while(itSub.hasNext()) {
                    OntClass sub = (OntClass)itSub.next();
                    Concept from = kb.getConcept(new Concept(sub.getNameSpace(),sub.getLocalName()));
                    Concept to = kb.getConcept(new Concept(cls.getNameSpace(),cls.getLocalName()));
                    if(sub.getLocalName() != null)
                        kb.addRelation(new SubclassOf(from, to));
                }
            }
         }
        //TODO: add superclasses from other ontologies
        
        if(dodebug) System.out.println("DEBUG: OWL processProperties listDatatypeProperties -- "+new Date());
        itProp = model.listDatatypeProperties();
        processProperties(kb,itProp, doSystemConcepts);
        
        if(dodebug) System.out.println("DEBUG: OWL processProperties listObjectProperties -- "+new Date());
        itProp = model.listObjectProperties();
        processProperties(kb,itProp, doSystemConcepts);

        if(dodebug) System.out.println("DEBUG: OWL processProperties listOntProperties -- "+new Date());
        itProp = modelRdfs.listOntProperties();
        processProperties(kb,itProp, doSystemConcepts);
        
        if(dodebug) System.out.println("DEBUG: OWL loadInstances OWL -- "+new Date());
        loadInstances(kb, model, OWL_MODEL);
        if(dodebug) System.out.println("DEBUG: OWL loadInstances RDFS -- "+new Date());
        loadInstances(kb, modelRdfs, RDFS_MODEL);
        
        if(dodebug) System.out.println("DEBUG: OWL processClassDescriptions -- "+new Date());
        // add info about union, intersection, restrictions
        processClassDescriptions(kb, model, doSystemConcepts);
       
        return kb;
   
    }
    
    private void processClassDescriptions(KnowledgeBase kb, OntModel model, boolean informed){
        // restrictions
        Iterator it = model.listNamedClasses();
        Vector errClasses = new Vector();
        
        while(it.hasNext()){
            OntClass cls = (OntClass)it.next();
            if( cls.isIntersectionClass() ){
                addClassDescription(kb,cls,cIntersectionOf,informed);
            } else if (cls.isUnionClass()){
                addClassDescription(kb,cls,cUnionOf,informed);
            } else if (cls.isEnumeratedClass()){
                addClassDescription(kb,cls,cOneOf, informed);
            }

            try{
                for (Iterator i = cls.listSuperClasses(true); i.hasNext(); ) {
                    OntClass sup = (OntClass) i.next();
                    if (sup.isRestriction()) {
                        addRestriction(kb,cls,sup.asRestriction(),informed);
                    }
                }
            }catch( Exception e){
                // list classes producing conversion error
                errClasses.addElement(cls);
            }
        }
        
        // process classes producing conversion error in listing of superclasses
        for(Iterator i = model.listRestrictions(); i.hasNext();){
            Restriction res = (Restriction)i.next();
            for(int j = 0; j < errClasses.size(); j++){
                OntClass errCls = (OntClass)errClasses.elementAt(j);
                if(res.hasSubClass(errCls)){
                    addRestriction(kb, errCls, res, informed);
                }
            }
        }

    }
    
    /**
     * add a more complex class description such as boolean operation
     * to the knowledge base
     * @param kb knowledge base
     * @param ocls class defined by the class description
     * @param type type of the class description e.g. union, intersection
     * @param informed flag indicating type whether system concepts should be used
     */    
    public void addClassDescription(KnowledgeBase kb, OntClass ocls, Concept type, boolean informed){
        Concept cls = kb.getConcept(new Concept(ocls.getNameSpace(),ocls.getLocalName()));
        Concept node;
        if(informed){
            // create assignment
            node = new Concept(ocls.getNameSpace(), 
                    "ASSIGN-"+cls.getName()+":"+type.getName());
            node.setCaption("ASSIGN-"+cls.getName()+":"+type.getName());
            node.setProperty(OwlJena.OWLTYPE, OwlJena.TYPE_ASSIGNMENT);
            kb.addConcept(node);
            kb.addRelation(new InstanceOf(node, type));
            kb.addRelation(new PropertyOf(node, cls));
        } else {
            // create an internal class node
            node = new Concept(ocls.getNameSpace(),
                    cls.getName()+"-"+type.getName());
            node.setCaption(cls.getName()+"-"+type.getName());
            node.setProperty(OwlJena.OWLTYPE, OwlJena.TYPE_OWL_CLASS);
            kb.addConcept(node);
            kb.addRelation(new SubclassOf(cls,node));
        }
            
        // add classes from the defined set
        Iterator it = null;
        if(ocls.isIntersectionClass()){
            IntersectionClass icls = ocls.asIntersectionClass();
            it = icls.listOperands();
        } else if(ocls.isUnionClass()){
            UnionClass ucls = ocls.asUnionClass();
            it = ucls.listOperands();
        } 
        
        while(it != null && it.hasNext()){
            OntClass acls = (OntClass)it.next();
            if(!acls.isRestriction()){
                Concept c = kb.getConcept(new Concept(acls.getNameSpace(),acls.getLocalName()));
                if(c == null){
                    c = new Concept(acls.getNameSpace(),acls.getLocalName());
                    c.setCaption(acls.getLocalName());
                    c.setProperty(OwlJena.OWLTYPE,  OwlJena.TYPE_OWL_CLASS);
                    kb.addConcept(c);
                }
                if(informed){
                    kb.addRelation(new HasValue(node,c));
                } else {
                    if(type.equals(cUnionOf))
                        kb.addRelation(new SubclassOf(c,node));
                    else if(type.equals(cIntersectionOf))
                        kb.addRelation(new SubclassOf(node,c));
                }
            } else {
                addRestriction(kb, ocls, acls.asRestriction(),informed);
            }
        }
        
        // process enumerated class
        if(ocls.isEnumeratedClass()){
            EnumeratedClass ecls = ocls.asEnumeratedClass();
            Iterator ei = ecls.getOneOf().iterator();
            while(ei.hasNext()){
                RDFNode anode = (RDFNode)ei.next();
                if(anode.canAs(Individual.class)){
                    Individual mem = (Individual)anode.as(Individual.class);
                    Concept c = new Concept(mem.getNameSpace(), mem.getLocalName());
                    c.setCaption(mem.getLocalName());
                    kb.addConcept(c);
                    
                    if(informed){
                        kb.addRelation(new HasValue(node,c));
                        kb.addRelation(new InstanceOf(c,cThing));
                        c.setProperty(OwlJena.OWLTYPE,  OwlJena.TYPE_LIST_MEMBER);
                    } else {
                        kb.addRelation(new InstanceOf(c,node));
                        c.setProperty(OwlJena.OWLTYPE,  OwlJena.TYPE_OWL_INSTANCE);
                    }

                }
            }
        }
    }
    
    
    /**
     * Add a restriction to the knowledge base
     * @param kb knowledge base
     * @param cls class that is a subclass of the restriction
     * @param res restriction to be added
     * @param informed flag indicating use of system concepts
     */    
    public void addRestriction(KnowledgeBase kb, OntClass cls, Restriction res, boolean informed){
        
        // TODO: handle the case of intersection one class and one restriction 
        if(informed){
            addRestrictionInformed(kb, cls, res);
        } else {
            //handle allValuesFrom and hasValue restrictions only
            //TODO: handle someValuesFrom restriction
            if( !res.isAllValuesFromRestriction() && !res.isHasValueRestriction())
                return;
        
            Concept c = kb.getConcept(new Concept(cls.getNameSpace(),cls.getLocalName()));
            Concept prop = kb.getConcept(new Concept(res.getOnProperty().getNameSpace(), 
                            res.getOnProperty().getLocalName()));
            
            // create assignement to property
            Concept assign = new Concept(prop.getNameSpace(),"ASSIGN-"+c.getName()+":"+prop.getName());
            assign.setProperty(OwlJena.OWLTYPE, OwlJena.TYPE_ASSIGNMENT);
            assign.setCaption("ASSIGN-"+c.getName()+":"+prop.getName());
            kb.addConcept(assign);
            kb.addRelation(new PropertyOf(assign, c));
            kb.addRelation(new InstanceOf(assign, prop));
        
            // create instaces of hasValue or allValuesFrom
            if(res.isAllValuesFromRestriction()){
                AllValuesFromRestriction av = res.asAllValuesFromRestriction();

                // value of all values from is a class or class description
                try{
                    OntClass values = (OntClass)av.getAllValuesFrom();
                    // value of all values is an enumerated class
                    if(values.isEnumeratedClass()){
                        EnumeratedClass ec = values.asEnumeratedClass();
                        for (Iterator li = ec.getOneOf().iterator(); li.hasNext(); ) {
                            Resource rval = (Resource) li.next();
                            Concept val = kb.getConcept(new Concept(
                                rval.getNameSpace(),rval.getLocalName()));
                            if(val == null){
                                val = new Concept(rval.getNameSpace(), rval.getLocalName());
                                val.setProperty(OwlJena.OWLTYPE, OwlJena.TYPE_OWL_INSTANCE);
                                kb.addConcept(val);
                            }
                            kb.addRelation(new HasValue(assign, val));
                        }
                    } else {
                        // value of all values is a simple class
                        Concept val = kb.getConcept(new Concept(
                        av.getAllValuesFrom().getNameSpace(),av.getAllValuesFrom().getLocalName()));
                        if(val == null){
                            val = new Concept(av.getAllValuesFrom().getNameSpace(),
                                av.getAllValuesFrom().getLocalName());
                            val.setProperty(OwlJena.OWLTYPE, OwlJena.TYPE_OWL_CLASS);
                            kb.addConcept(val);
                        }
                        kb.addRelation(new HasValue(assign, val));
                    }
                } catch (Exception e){
                    // value of all values is identified by a resource of unspecified type
                    Concept val = new Concept(av.getAllValuesFrom().getNameSpace(),
                                av.getAllValuesFrom().getLocalName());
                    val.setProperty(OwlJena.OWLTYPE, OwlJena.TYPE_OWL_CLASS);
                    kb.addConcept(val);
                    kb.addRelation(new HasValue(assign, val));
                }
            } else if(res.isHasValueRestriction()){
                HasValueRestriction hv = res.asHasValueRestriction();

                // assign value to the property has value
                if(hv.getHasValue().canAs(Individual.class)){
                    Individual ival = (Individual)hv.getHasValue();
                    Concept val = kb.getConcept(new Concept(ival.getNameSpace(),ival.getLocalName()));
                    if(val == null){
                        val = new Concept(ival.getNameSpace(), ival.getLocalName());
                        val.setProperty(OwlJena.OWLTYPE, OwlJena.TYPE_OWL_INSTANCE);
                        kb.addConcept(val);
                    }
                    kb.addRelation(new HasValue(assign, val));


                } else if(hv.getHasValue().canAs(Literal.class)){
                    Literal lval = (Literal)hv.getHasValue();
                    Concept val = kb.getConcept(new Concept(lval.getDatatypeURI(),lval.getLexicalForm()));
                    if(val == null){
                        val = new Concept(lval.getDatatypeURI(), lval.getLexicalForm());
                        val.setCaption(lval.getLexicalForm());
                        val.setProperty(OwlJena.OWLTYPE, OwlJena.TYPE_LITERAL);
                        kb.addConcept(val);
                    }
                    kb.addRelation(new HasValue(assign, val));
                }
            }        
        }
    }
    
    /**
     * add restriction using system concepts
     * @param kb knowledge base
     * @param cls class that is a subclass of the restriction
     * @param res restriction to be added
     */    
    public void addRestrictionInformed(KnowledgeBase kb, OntClass cls, Restriction res){
    
        //handle allValuesFrom and hasValue restrictions only
        if( !res.isAllValuesFromRestriction() && !res.isHasValueRestriction() && !res.isSomeValuesFromRestriction())
            return;
        
        // create instance of owl:Restriction
        Concept r = new Concept(cls.getNameSpace(), cls.getLocalName()+"-"+cRestriction.getName()+"-"+res.getOnProperty().getLocalName());
        r.setCaption(cls.getLocalName()+"-"+cRestriction.getName()+"-"+res.getOnProperty().getLocalName());
        r.setProperty(OwlJena.OWLTYPE, OwlJena.TYPE_RESTRICTION);
        kb.addConcept(r);
        kb.addRelation(new InstanceOf(r, cRestriction));
        
        // add class as a subclass of the restriction
        Concept c = kb.getConcept(new Concept(cls.getNameSpace(),cls.getLocalName()));
        kb.addRelation(new SubclassOf(c,r));
        
        //create instances of onProperty 
        Concept onProp = new Concept(cls.getNameSpace(), "ASSIGN-"+r.getName()+":"+cOnProperty.getName());
        onProp.setCaption("ASSIGN-"+r.getName()+":"+cOnProperty.getName());
        onProp.setProperty(OwlJena.OWLTYPE, OwlJena.TYPE_ASSIGNMENT);
        kb.addConcept(onProp);
        kb.addRelation(new InstanceOf(onProp, cOnProperty));
        
        Concept prop = kb.getConcept(new Concept(res.getOnProperty().getNameSpace(), 
            res.getOnProperty().getLocalName()));
        kb.addRelation(new PropertyOf(onProp,r));
        kb.addRelation(new HasValue(onProp, prop));
        
        // create instaces of hasValue or allValuesFrom or someValuesFrom
        if(res.isAllValuesFromRestriction() || res.isSomeValuesFromRestriction()){

            Resource values = null;
            Concept typeAssign = null;

            if(res.isAllValuesFromRestriction()){
                AllValuesFromRestriction av = res.asAllValuesFromRestriction();
                typeAssign = new Concept(cls.getNameSpace(), "ASSIGN-"+r.getName()+":"+cAllValuesFrom.getName());
                typeAssign.setCaption("ASSIGN-"+r.getName()+":"+cAllValuesFrom.getName());
                onProp.setProperty(OwlJena.OWLTYPE, OwlJena.TYPE_ASSIGNMENT);
                kb.addConcept(typeAssign);
                kb.addRelation(new InstanceOf(typeAssign, cAllValuesFrom));

                values = av.getAllValuesFrom();
            } else{
                SomeValuesFromRestriction sv = res.asSomeValuesFromRestriction();
                typeAssign = new Concept(cls.getNameSpace(), "ASSIGN-"+r.getName()+":"+cSomeValuesFrom.getName());
                typeAssign.setCaption("ASSIGN-"+r.getName()+":"+cSomeValuesFrom.getName());
                onProp.setProperty(OwlJena.OWLTYPE, OwlJena.TYPE_ASSIGNMENT);
                kb.addConcept(typeAssign);
                kb.addRelation(new InstanceOf(typeAssign, cSomeValuesFrom));

                values = sv.getSomeValuesFrom();
            }
            
            if(values instanceof OntClass){
                OntClass valCls = (OntClass)values;
                            
                // value of all values is an enumerated class
                if(valCls.isEnumeratedClass()){
                    EnumeratedClass ec = valCls.asEnumeratedClass();
                    kb.addRelation(new PropertyOf(typeAssign,r));
                    for (Iterator li = ec.getOneOf().iterator(); li.hasNext(); ) {
                        Resource rval = (Resource) li.next();
                        Concept val = kb.getConcept(new Concept(
                            rval.getNameSpace(),rval.getLocalName()));
                        if(val == null){
                            val = new Concept(rval.getNameSpace(), rval.getLocalName());
                            val.setProperty(OwlJena.OWLTYPE, OwlJena.TYPE_OWL_INSTANCE);
                            kb.addConcept(val);
                        }
                        kb.addRelation(new HasValue(typeAssign, val));
                    }
                } else {
                    // value of all values is a simple class
                    Concept val = kb.getConcept( new Concept(
                    valCls.getNameSpace(),valCls.getLocalName()));
                    if(val == null){
                        val = new Concept(valCls.getNameSpace(),
                            valCls.getLocalName());
                        val.setProperty(OwlJena.OWLTYPE, OwlJena.TYPE_OWL_CLASS);
                        kb.addConcept(val);
                    }
                    kb.addRelation(new PropertyOf(typeAssign,r));
                    kb.addRelation(new HasValue(typeAssign,val));
                }
            } else {
                // value of all values is identified by a resource of unspecified type
                Concept val = new Concept(values.getNameSpace(),
                                    values.getLocalName());
                val.setProperty(OwlJena.OWLTYPE, OwlJena.TYPE_OWL_CLASS);
                kb.addConcept(val);
                kb.addRelation(new PropertyOf(typeAssign,r));
                kb.addRelation(new HasValue(typeAssign, val));
            }
        } else if(res.isHasValueRestriction()){
            HasValueRestriction hv = res.asHasValueRestriction();
            
            // create an instance of has value
            Concept hasVal = new Concept(cls.getNameSpace(), "ASSIGN-"+r.getName()+":"+cHasValue.getName());
            hasVal.setCaption("ASSIGN-"+r.getName()+":"+cHasValue.getName());
            onProp.setProperty(OwlJena.OWLTYPE, OwlJena.TYPE_ASSIGNMENT);
            kb.addConcept(hasVal);
            kb.addRelation(new InstanceOf(hasVal, cHasValue));
            
            // assign value to the property has value
            if(hv.getHasValue().canAs(Individual.class)){
                Individual ival = (Individual)hv.getHasValue();
                Concept val = kb.getConcept(new Concept(ival.getNameSpace(),ival.getLocalName()));
                if(val == null){
                    val = new Concept(ival.getNameSpace(), ival.getLocalName());
                    val.setProperty(OwlJena.OWLTYPE, OwlJena.TYPE_OWL_INSTANCE);
                    kb.addConcept(val);
                }
                kb.addRelation(new PropertyOf(hasVal,r));
                kb.addRelation(new HasValue(hasVal, val));

                    
            } else if(hv.getHasValue().canAs(Literal.class)){
                Literal lval = (Literal)hv.getHasValue();
                Concept val = kb.getConcept(new Concept(lval.getDatatypeURI(),lval.getLexicalForm()));
                if(val == null){
                    val = new Concept(lval.getDatatypeURI(), lval.getLexicalForm());
                    val.setCaption(lval.getLexicalForm());
                    val.setProperty(OwlJena.OWLTYPE, OwlJena.TYPE_LITERAL);
                    kb.addConcept(val);
                }
                kb.addRelation(new PropertyOf(hasVal,r));
                kb.addRelation(new HasValue(hasVal, val));
            }
        }        
    }
    
    /**
     * load properties into knowledge base
     * @param kb - knowledge base
     * @param itProp - properties iterator
     * @param type - type of the properties
     */
    private void loadProperties(KnowledgeBase kb, Iterator itProp, Concept type, boolean doSystemConcepts){
        while(itProp.hasNext()) {
            OntProperty prop = (OntProperty)itProp.next();
            Concept c = new Concept(prop.getNameSpace(),prop.getLocalName());
            if(kb.getConcept(c) != null) 
                continue;
            Concept concept = new Concept(prop.getNameSpace(),prop.getLocalName());
            concept.setCaption(prop.getLocalName());
            /*if(prop.isTransitiveProperty())
                type = cTransitiveProperty;
            else if(prop.isFunctionalProperty())
                type = cFunctionalProperty;
            */            
            concept.setProperty(OwlJena.OWLTYPE, type.getProperty(OwlJena.OWLTYPE));
            
            kb.addConcept(concept);
            if(doSystemConcepts)
                kb.addRelation(new InstanceOf(concept,type));
        }
    }
    
    
    /**
     * process properties to get subproperties, domain and range
     * @param kb knowledge base
     * @param itProp property iterator
     * @param doSystemConcepts use concepts of the system ontology
     */
    private void processProperties(KnowledgeBase kb, Iterator itProp, boolean doSystemConcepts){
        while(itProp.hasNext()) {
            OntProperty prop = (OntProperty)itProp.next();
            //System.out.println(prop.getLocalName());
            if(prop.getLocalName() != null){
                // get subproperties
                Iterator itSub = prop.listSubProperties(true);
                while(itSub.hasNext()) {
                    OntProperty sub = (OntProperty)itSub.next();
                    Concept from = kb.getConcept(new Concept(sub.getNameSpace(),sub.getLocalName()));
                    Concept to = kb.getConcept(new Concept(prop.getNameSpace(),prop.getLocalName()));
                    if(sub.getLocalName() != null)
                        kb.addRelation(new SubclassOf(from, to));
                }
                // get domain 
                Concept from = kb.getConcept(new Concept(prop.getNameSpace(),prop.getLocalName()));
                OntResource domain = (OntResource)prop.getDomain();
                if(domain != null){
                    Concept to = kb.getConcept(new Concept(domain.getNameSpace(),domain.getLocalName()));
                    //System.out.println(domain.getLocalName());
                    if(to == null){
                        to = new Concept(domain.getNameSpace(),domain.getLocalName());
                        to.setProperty(OwlJena.OWLTYPE, OwlJena.TYPE_RDFS_CLASS);
                        kb.addConcept(to);
                        if(doSystemConcepts) 
                            kb.addRelation(new InstanceOf(to,cRdfsClass));
                    }
                    kb.addRelation(new HasDomain(from, to));
                }
                // get range
                OntResource range = (OntResource)prop.getRange();
                if(range != null){
                    Concept to = kb.getConcept(new Concept(range.getNameSpace(),range.getLocalName()));
                    //System.out.println(range.getLocalName());
                    if(to == null){
                        to = new Concept(range.getNameSpace(),range.getLocalName());
                        if(from.getProperty(OwlJena.OWLTYPE).equals(OwlJena.TYPE_DATATYPE_PROPERTY))
                            to.setProperty(OwlJena.OWLTYPE, OwlJena.TYPE_DATATYPE);
                        else
                            to.setProperty(OwlJena.OWLTYPE, OwlJena.TYPE_RDFS_CLASS);
                        //kb.addConcept(to);
                        if(doSystemConcepts){ 
                            kb.addConcept(to);
                            if(from.getProperty(OwlJena.OWLTYPE).equals(OwlJena.TYPE_DATATYPE_PROPERTY))
                                kb.addRelation(new InstanceOf(to,cDatatype));
                            else
                                kb.addRelation(new InstanceOf(to,cRdfsClass));
                            kb.addRelation(new HasRange(from, to));
                        } else {
                            if(!from.getProperty(OwlJena.OWLTYPE).equals(OwlJena.TYPE_DATATYPE_PROPERTY)){
                                kb.addConcept(to);
                                kb.addRelation(new HasRange(from, to));
                            }
                                
                        }
                    } else
                        kb.addRelation(new HasRange(from, to));
                }
                
            }
        }
    }
    
    /**
     * Load instances into the knowledge base
     * @param kb knowledge base
     * @param model Jena ontology model
     * @param modelType type of the ontology model
     */    
    private void loadInstances(KnowledgeBase kb, OntModel model, String modelType){
        if(dodebug) System.out.println("DEBUG: OWL loadInstances listNamedClasses -- "+new Date());
        Iterator it = model.listNamedClasses();
        while(it.hasNext()){
            OntClass cls = (OntClass)it.next();
            Iterator itInst = cls.listInstances();
            while(itInst.hasNext()){
                Individual ind = (Individual)itInst.next();
                // add instance to the knowledge base
                Concept inst = new Concept(ind.getNameSpace(),ind.getLocalName());
                inst.setCaption(ind.getLocalName());
                inst.setProperty(OwlJena.OWLTYPE, OwlJena.TYPE_OWL_INSTANCE);
                kb.addConcept(inst);
                kb.addRelation(new InstanceOf(inst,kb.getConcept(new Concept(cls.getNameSpace(),cls.getLocalName()))));
            }
        }
        
        if(dodebug) System.out.println("DEBUG: OWL add values of properties to instances -- "+new Date());
        // add values of properties to instances
        it = model.listNamedClasses();
        while(it.hasNext()){
            OntClass ocls = (OntClass)it.next();
            Iterator itInst = ocls.listInstances();
            while(itInst.hasNext()){
                Individual ind = (Individual)itInst.next();
                Concept inst = kb.getConcept(new Concept(ind.getNameSpace(),ind.getLocalName()));
                StmtIterator itProp = ind.listProperties();
                while(itProp.hasNext()){
                    Statement stmt = itProp.nextStatement();
                    Property oprop = stmt.getPredicate();
                    Concept tempprop = new Concept(oprop.getNameSpace(),oprop.getLocalName());
                    Concept prop = kb.getConcept(tempprop);
                    
                    // property rdf:type
                    if( prop != null && oprop.getLocalName().equals("type") && isSystemConcept(prop)){
                        if(stmt.getObject() instanceof Resource){
                            //if(dodebug) System.out.println("DEBUG: OWL property rdf:type -- "+new Date());
                            Resource res = stmt.getResource();
                            Concept cls = kb.getConcept(new Concept(res.getNameSpace(),res.getLocalName()));
                            if(cls == null){
                                cls = new Concept(res.getNameSpace(),res.getLocalName());
                                cls.setCaption(res.getLocalName());
                                cls.setProperty(OwlJena.OWLTYPE, OwlJena.TYPE_OWL_CLASS);
                                kb.addConcept(cls);
                            }
                            Relation instanceOf = new InstanceOf(inst, cls);
                            if(kb.getRelation(instanceOf) == null)
                                kb.addRelation(instanceOf);
                            continue;
                        }
                    // skip other system properties
                    } else if(prop == null || isSystemConcept(prop))
                        continue;
                    
                    if(dodebug) System.out.println("DEBUG: OWL create assigment i.e. instance of a property -- "+new Date());
                    // create assigment i.e. instance of a property
                    Concept assign = new Concept(oprop.getNameSpace(),"ASSIGN-"+inst.getName()+":"+oprop.getLocalName());
                    assign.setProperty(OwlJena.OWLTYPE, OwlJena.TYPE_ASSIGNMENT);
                    kb.addConcept(assign);
                    kb.addRelation(new InstanceOf(assign,
                            kb.getConcept(new Concept(oprop.getNameSpace(),oprop.getLocalName()))));
                    kb.addRelation(new PropertyOf(assign, inst));
                    
                    if(dodebug) System.out.println("DEBUG: OWL add value of the assignment -- "+new Date());
                    // add value of the assignment
                    Concept val;
                    if(stmt.getObject() instanceof Resource){
                        // instances
                        Resource res = stmt.getResource();
                        val = kb.getConcept(new Concept(res.getNameSpace(),res.getLocalName()));
                        if(val == null){
                            val = new Concept(res.getNameSpace(),res.getLocalName());
                            val.setCaption(res.getLocalName());
                            val.setProperty(OwlJena.OWLTYPE, OwlJena.TYPE_OWL_INSTANCE);
                            kb.addConcept(val);
                        }
                    } else {
                        // literals
                        Literal lit = stmt.getLiteral();
                        val = kb.getConcept(new Concept(lit.getDatatypeURI(),lit.getLexicalForm()));
                        if(val == null){
                            val = new Concept(lit.getDatatypeURI(), lit.getLexicalForm());
                            val.setCaption(lit.getLexicalForm());
                            val.setProperty(OwlJena.OWLTYPE, OwlJena.TYPE_LITERAL);
                            kb.addConcept(val);
                        }
                    }
                    kb.addRelation(new HasValue(assign,val));
                    Concept objClass = kb.getConcept(new Concept(oprop.getNameSpace(),oprop.getLocalName()));
                    Vector rels = kb.getRelationsFrom(objClass);
                    for(int i = 0; i < rels.size(); i++){
                        Relation rel = (Relation)rels.elementAt(i);
                        if(rel instanceof HasRange)
                            kb.addRelation(new InstanceOf(val,rel.getTarget()));
                    }
                    if(dodebug) System.out.println("DEBUG: OWL goint to the next -- "+new Date());
                    
                }
            }
        }

    }
    
    /**
     * process properties to get subproperties, domain and range
     * @return knowledge base containing concepts of the system ontology
     */
    public KnowledgeBase createSystemOntology( boolean doSystemConcepts ) {
        
        //TODO: separate creating of system ontology and adding it to the kb
        
        KnowledgeBase kb = new KnowledgeBase();
        if(owlNS == null) owlNS = "system:owl";
        if(rdfsNS == null) rdfsNS = "system:rdfs";
        if(rdfNS == null) rdfNS = "system:rdf";
        
        kb.addSystemNamespace(owlNS);
        kb.addSystemNamespace(rdfNS);
        kb.addSystemNamespace(rdfsNS);
        
        cOwlClass = new Concept(owlNS,"Class");
        cOwlClass.setProperty(OwlJena.OWLTYPE, OwlJena.TYPE_OWL_CLASS);

        cRdfsClass = new Concept(rdfsNS,"Class");
        cRdfsClass.setProperty(OwlJena.OWLTYPE, OwlJena.TYPE_RDFS_CLASS);
        
        cObjectProperty = new Concept(owlNS,"ObjectProperty");
        cObjectProperty.setProperty(OwlJena.OWLTYPE, OwlJena.TYPE_OBJECT_PROPERTY);

        cDatatypeProperty = new Concept(owlNS,"DatatypeProperty");
        cDatatypeProperty.setProperty(OwlJena.OWLTYPE, OwlJena.TYPE_DATATYPE_PROPERTY);
        
        cFunctionalProperty = new Concept(owlNS,"FunctionalProperty");
        cFunctionalProperty.setProperty(OwlJena.OWLTYPE, OwlJena.TYPE_FUNCTIONAL_PROPERTY);
        
        cTransitiveProperty = new Concept(owlNS,"TransitiveProperty");
        cTransitiveProperty.setProperty(OwlJena.OWLTYPE, OwlJena.TYPE_TRANSITIVE_PROPERTY);
        
        cResource = new Concept(rdfNS,"Resource");
        cResource.setProperty(OwlJena.OWLTYPE, OwlJena.TYPE_RESOURCE);
        
        cProperty = new Concept(rdfsNS,"Property");
        cProperty.setProperty(OwlJena.OWLTYPE, OwlJena.TYPE_PROPERTY);
        
        cDatatype = new Concept(rdfsNS,"Datatype");
        cDatatype.setProperty(OwlJena.OWLTYPE, OwlJena.TYPE_DATATYPE);
        
        cLiteral = new Concept(rdfsNS,"Literal");
        cLiteral.setProperty(OwlJena.OWLTYPE, OwlJena.TYPE_LITERAL);
        
        cType = new Concept(rdfNS,"type");
        cType.setProperty(OwlJena.OWLTYPE, OwlJena.TYPE_PROPERTY);
        
        cThing = new Concept(owlNS,"Thing");
        cThing.setProperty(OwlJena.OWLTYPE, OwlJena.TYPE_OWL_CLASS);
        
        cRestriction = new Concept(owlNS,"Restriction");
        cRestriction.setProperty(OwlJena.OWLTYPE, OwlJena.TYPE_OWL_CLASS);
        
        cOnProperty = new Concept(owlNS,"onProperty");
        cOnProperty.setProperty(OwlJena.OWLTYPE, OwlJena.TYPE_PROPERTY);

        cHasValue = new Concept(owlNS,"hasValue");
        cHasValue.setProperty(OwlJena.OWLTYPE, OwlJena.TYPE_PROPERTY);
        
        cAllValuesFrom = new Concept(owlNS,"allValuesFrom");
        cAllValuesFrom.setProperty(OwlJena.OWLTYPE, OwlJena.TYPE_PROPERTY);
        
        cSomeValuesFrom = new Concept(owlNS,"someValuesFrom");
        cSomeValuesFrom.setProperty(OwlJena.OWLTYPE, OwlJena.TYPE_PROPERTY);
       
        cIntersectionOf = new Concept(owlNS,"intersectionOf");
        cIntersectionOf.setProperty(OwlJena.OWLTYPE, OwlJena.TYPE_PROPERTY);

        cUnionOf = new Concept(owlNS,"unionOf");
        cUnionOf.setProperty(OwlJena.OWLTYPE, OwlJena.TYPE_PROPERTY);
        
        cOneOf = new Concept(owlNS,"oneOf");
        cOneOf.setProperty(OwlJena.OWLTYPE, OwlJena.TYPE_PROPERTY);
        
        if(doSystemConcepts){
            kb.addConcept(cOwlClass);
            kb.addConcept(cOneOf);  
            kb.addConcept(cUnionOf);
            kb.addConcept(cIntersectionOf);
            kb.addConcept(cSomeValuesFrom);
            kb.addConcept(cAllValuesFrom);
            kb.addConcept(cHasValue);
            kb.addConcept(cOnProperty);
            kb.addConcept(cRestriction);
            kb.addRelation(new SubclassOf(cRestriction, cOwlClass));
            kb.addConcept(cThing);
            kb.addRelation(new InstanceOf(cThing, cOwlClass));
            kb.addConcept(cType);
            kb.addConcept(cRdfsClass);
            kb.addConcept(cObjectProperty);
            kb.addConcept(cDatatypeProperty);
            kb.addConcept(cFunctionalProperty);
            kb.addConcept(cTransitiveProperty);
            kb.addConcept(cResource);
            kb.addConcept(cProperty);
            kb.addConcept(cDatatype);
            kb.addConcept(cLiteral);
        }
        
        
        
        return kb;
    }
    
    /**
     * method for testing whether a given concept belongs to a system ontology
     * @param c concept to be tested
     * @return true if the concept belongs to a system ontology
     */    
    protected boolean isSystemConcept(Concept c){
        if ( c.getNameSpace().equals(OwlRdfsJena.owlNS)
                || c.getNameSpace().equals(OwlRdfsJena.rdfNS)
                || c.getNameSpace().equals(OwlRdfsJena.rdfsNS)){
            return true;
        } else {
            return false;
        }
    }
    
    /**
     * Classify concepts of the knowledge base for Apollo
     * @param kb the knowledge base to classify
     * @throws KMException fails if the classification is not possible
     */
    public void classifyKB(cz.cvut.km.model.KnowledgeBase kb) throws cz.cvut.km.utils.KMException {
        
        classifyKBByMapping(kb); // classification by Mapping
    }
    
    // Mapping used for classification
    Mapping mapping;
    
    public void classifyKBByMapping(cz.cvut.km.model.KnowledgeBase kb) throws cz.cvut.km.utils.KMException {
        //cz.cvut.km.utils.Mapping 
	mapping = createValidator();
        mapping.doMapping(kb);
    }
    
        /** create set of validation rules
     */
    public Mapping createValidator() {
        Mapping mapping = new Mapping(OWLTYPE);
        mapping.addRule(OwlJena.TYPE_OWL_CLASS,      SubclassOf.class, OwlJena.TYPE_OWL_CLASS);
        mapping.addRule(OwlJena.TYPE_OBJECT_PROPERTY,   SubclassOf.class,   OwlJena.TYPE_OWL_CLASS);
        mapping.addRule(OwlJena.TYPE_OBJECT_PROPERTY,   HasRange.class,   OwlJena.TYPE_OWL_CLASS);
        mapping.addRule(OwlJena.TYPE_OBJECT_PROPERTY,   HasDomain.class,  OwlJena.TYPE_OWL_CLASS);
        mapping.addRule(OwlJena.TYPE_OBJECT_PROPERTY,   HasRange.class,   OwlJena.TYPE_OBJECT_PROPERTY);
        mapping.addRule(OwlJena.TYPE_OBJECT_PROPERTY,   HasDomain.class,  OwlJena.TYPE_OBJECT_PROPERTY);
        mapping.addRule(OwlJena.TYPE_ASSIGNMENT, PropertyOf.class, OwlJena.TYPE_OWL_INSTANCE);
        mapping.addRule(OwlJena.TYPE_ASSIGNMENT, PropertyOf.class, OwlJena.TYPE_OWL_CLASS);
        mapping.addRule(OwlJena.TYPE_ASSIGNMENT, HasValue.class,   OwlJena.TYPE_OWL_INSTANCE);
        mapping.addRule(OwlJena.TYPE_ASSIGNMENT, HasValue.class,   OwlJena.TYPE_OWL_CLASS);
        mapping.addRule(OwlJena.TYPE_OBJECT_PROPERTY,   HasValue.class,   OwlJena.TYPE_OWL_INSTANCE);
        mapping.addRule(OwlJena.TYPE_OBJECT_PROPERTY,   HasValue.class,   OwlJena.TYPE_OWL_CLASS);
        mapping.addRule(OwlJena.TYPE_OWL_INSTANCE,   InstanceOf.class, OwlJena.TYPE_OWL_CLASS);
        mapping.addRule(OwlJena.TYPE_ASSIGNMENT, InstanceOf.class, OwlJena.TYPE_OBJECT_PROPERTY);
        return mapping;
    }


    
    
    /**
     * save knowledge base
     * @param kb knowledge base to be saved
     * @param parameters parameters e.g. where the knowledge base is to be saved
     * @throws IOException problems with target file
     * @throws KMException invalid knowledge base
     */    
    public void saveKB(cz.cvut.km.model.KnowledgeBase kb, java.util.Properties parameters) throws java.io.IOException, cz.cvut.km.utils.KMException {
        
        String target = parameters.get("filename").toString();
        String baseUri = null;
        
        baseUri = "http://www.cvut.cz/apollo#";
        
        boolean doSystemConcepts = false;
        String doSystemConceptsStr = parameters.getProperty("system-concepts","false");
        if(doSystemConceptsStr.equalsIgnoreCase("true"))
            doSystemConcepts = true;
        
        if(!doSystemConcepts) {
            classifyKB(kb);
            // save log
    	    FileWriter fw = new FileWriter(target+".log");
	    fw.write(mapping.getErrorLog());
            fw.close();
	}
        
        OntModel model = ModelFactory.createOntologyModel( OntModelSpec.OWL_MEM, null);
                
        // TODO: save uri of the main ontology into KB
        // TODO: add XML document header

        ConceptIterator ci = kb.getConcepts();
        Vector classes = new Vector();
        
        // add classes to the model
        while(ci.hasNext()){
            Concept concept = ci.nextConcept();
            //TODO: find a better way to combine 2 models into one file
            if(OwlJena.TYPE_OWL_CLASS.equals(concept.getProperty(OwlJena.OWLTYPE))
                || OwlJena.TYPE_RDFS_CLASS.equals(concept.getProperty(OwlJena.OWLTYPE))){
                if(!isSystemConcept(concept)){
                    OntClass cls = model.createClass(getFullConceptName(concept.getNameSpace(), concept.getName()));
                    //System.out.println(cls.getLocalName());
                    //System.out.println(cls.getNameSpace());
                    
                    classes.addElement(cls);
                     
                }
            }
        }
                              
        // add superclasses to each class
        for(int i = 0; i < classes.size(); i++){
            OntClass ocls = (OntClass)classes.elementAt(i);
            Concept cls = kb.getConcept(new Concept(ocls.getNameSpace(), ocls.getLocalName()));
            Iterator it = kb.getRelationsFrom(cls).iterator();
            while(it.hasNext()){
                Relation rel = (Relation)it.next();
                if(rel instanceof SubclassOf){
                    OntClass acls = model.getOntClass(getFullConceptName(rel.getTarget().getNameSpace(),rel.getTarget().getName()));
                    if(acls != null)
                        ocls.addSuperClass(acls);
                }
            }
        }
        
        // add properties to the model
        saveProperties(kb, model);
        
        // add instances to the model
        saveInstances(kb, model);
        
        // add restrictions to classes
        saveRestrictions(kb, model, classes);
        
        // handle complex class descriptions
        saveClassDescriptions(kb, model, classes);
        
        // write model to the owl file
        RDFWriter writer = model.getWriter("RDF/XML-ABBREV");
        writer.setProperty("showXmlDeclaration", "true");
        writer.setProperty("relativeURIs", "same-document");
        
        // write model to the owl file
        if(baseUri != null){
            model.setNsPrefix("", baseUri);
            writer.setProperty("xmlBase", baseUri);
            writer.write(model, new FileOutputStream(target), baseUri);
        } else 
            
        model.write(new FileOutputStream(target), "RDF/XML-ABBREV");
        
        
        
    }
    
    private void saveClassDescriptions(cz.cvut.km.model.KnowledgeBase kb, OntModel model, Vector classes){
    
        ConceptIterator ci = kb.getConcepts();
        while(ci.hasNext()){
            Concept assign = ci.nextConcept();
            if(OwlJena.TYPE_ASSIGNMENT.equals(assign.getProperty(OwlJena.OWLTYPE))){
                Iterator it = kb.getRelationsFrom(assign).iterator();
                OntClass cls = null;
                Property prop = null;
                while(it.hasNext()){
                    Relation rel = (Relation)it.next();
                    if(rel instanceof PropertyOf){
                        cls = model.getOntClass(rel.getTarget().getNameSpace()+rel.getTarget().getName());
                    } else if(rel instanceof InstanceOf){
                        prop = model.getProperty(rel.getTarget().getNameSpace(), rel.getTarget().getName());
                    }
                }
                
                // get values of operands
                if(prop.equals(OWL.intersectionOf) || prop.equals(OWL.unionOf) || prop.equals(OWL.oneOf)){
                    it = kb.getRelationsFrom(assign).iterator();
                    RDFList members = model.createList();
                    while(it.hasNext()){
                        Relation rel = (Relation)it.next();
                        if(rel instanceof HasValue)
                            members = members.cons(model.getResource(rel.getTarget().getNameSpace()+rel.getTarget().getName()));
                    }
                
                    if(cls != null && prop != null ){
                        // if only one operand is a class add restrictions to operand list
                        if(members.size() < 2 && (prop.equals(OWL.intersectionOf) || prop.equals(OWL.unionOf))){
                            Iterator si = cls.listSuperClasses(true);
                            while(si.hasNext()){
                                OntClass sup = (OntClass)si.next();
                                if(sup.isRestriction()){
                                    cls.removeSuperClass(sup);
                                    members = members.cons(sup);
                                }
                            }
                        }
                        // if there are still less than 2 operands - ivalid -> skip
                        if(members.size() < 2)
                            continue;
                    
                        // handle different types of expressions
                        if(prop.equals(OWL.intersectionOf)){
                            cls.convertToIntersectionClass(members);
                        } else if(prop.equals(OWL.unionOf)){
                            cls.convertToUnionClass(members);
                        } else if(prop.equals(OWL.oneOf)){
                            cls.convertToEnumeratedClass(members);
                        }
                    }
                }
            }
        }
    }
    
    private void saveRestrictions(cz.cvut.km.model.KnowledgeBase kb, OntModel model, Vector classes){
        
        // add restriction to classes where appropriate
        for(int i = 0; i < classes.size(); i++){
            OntClass ocls = (OntClass)classes.elementAt(i);
            Concept cls = kb.getConcept(new Concept(ocls.getNameSpace(), ocls.getLocalName()));
            Iterator it = kb.getRelationsFrom(cls).iterator();
            while(it.hasNext()){
                Property oprop = null;
                Vector vals = new Vector();
                Individual ind = null;
                Concept restrictType = null;
                Relation rel = (Relation)it.next();
                if(rel instanceof SubclassOf){
                    Concept res = rel.getTarget();
                    if(OwlJena.TYPE_RESTRICTION.equals(res.getProperty(OwlJena.OWLTYPE))){
                        Iterator ai = kb.getRelationsTo(res).iterator();
                        while(ai.hasNext()){
                            Relation po = (Relation)ai.next();
                            if(po instanceof PropertyOf){
                                Concept assign = po.getSource();
                                Iterator rf = kb.getRelationsFrom(assign).iterator();
                                Concept type = null;
                                Vector values = new Vector();
                                while(rf.hasNext()){
                                    Relation r = (Relation)rf.next();
                                    if(r instanceof InstanceOf){
                                        type = r.getTarget();
                                    } else if(r instanceof HasValue){
                                        values.addElement(r.getTarget());
                                    }
                                }
                                // handle case of onProperty, and restriction types
                                if(type.equals(OwlRdfsJena.cOnProperty)){
                                    Concept val = (Concept)values.firstElement();
                                    oprop = model.getProperty(val.getNameSpace(), val.getName());
                                } else if(type.equals(OwlRdfsJena.cAllValuesFrom) 
                                            || type.equals(OwlRdfsJena.cSomeValuesFrom)){
                                    restrictType = type;
                                    vals.addAll(values);
                                } else if(type.equals(OwlRdfsJena.cHasValue)){
                                    restrictType = type;
                                    Concept val = (Concept)values.firstElement();
                                    ind = model.getIndividual(getFullConceptName(val.getNameSpace(),val.getName()));
                                }
                            }
                        }
                    }
                }
                
                //process values of restrictions
                OntClass acls = null;
                if(vals.size() == 1){
                    Concept aval = (Concept)vals.firstElement();
                    acls = model.getOntClass(getFullConceptName(aval.getNameSpace(),aval.getName()));
                } else {
                    RDFList list = model.createList();
                    for(int j = 0; j < vals.size(); j++){
                        Concept aval = (Concept)vals.elementAt(j);
                        Individual ival = model.getIndividual(getFullConceptName(aval.getNameSpace(),aval.getName()));
                        if(ival != null)
                            list = list.cons(ival);
                    }
                    if(!list.isEmpty())
                        acls = model.createEnumeratedClass(null, list);
                }    

                // create the appropriate restriction as a superclass of the given class
                if(oprop != null && restrictType != null){
                    Restriction res = null;
                    if(restrictType.equals(OwlRdfsJena.cAllValuesFrom) && acls != null){
                        res = model.createAllValuesFromRestriction(null, oprop, acls);
                    } else if(restrictType.equals(OwlRdfsJena.cSomeValuesFrom) && acls != null){
                        res = model.createSomeValuesFromRestriction(null, oprop, acls);
                    } else if(restrictType.equals(OwlRdfsJena.cHasValue) && ind != null){
                        res = model.createHasValueRestriction(null, oprop, ind);
                    }
                    
                    // in case some information is missing skip
                    if(res != null)
                        ocls.addSuperClass(res);
                    
                }
            }
        }
    }
    
    private void saveProperties(cz.cvut.km.model.KnowledgeBase kb, OntModel model){
        
        ConceptIterator ci = kb.getConcepts();
        Vector props = new Vector();
        
        // add properties to the model
        while(ci.hasNext()){
            Concept prop = ci.nextConcept();
            ObjectProperty oprop = null;
            
            // skip system concepts
            if(isSystemConcept(prop))
                continue;
            
            // create property of the appropariate type
            if(OwlJena.TYPE_OBJECT_PROPERTY.equals(prop.getProperty(OwlJena.OWLTYPE))){
                oprop = model.createObjectProperty(getFullConceptName(prop.getNameSpace(), prop.getName()));
            } else if(OwlJena.TYPE_FUNCTIONAL_PROPERTY.equals(prop.getProperty(OwlJena.OWLTYPE))){
                oprop = model.createObjectProperty(getFullConceptName(prop.getNameSpace(), prop.getName()), true);
            } else if(OwlJena.TYPE_TRANSITIVE_PROPERTY.equals(prop.getProperty(OwlJena.OWLTYPE))){
                oprop = model.createTransitiveProperty(getFullConceptName(prop.getNameSpace(), prop.getName()));
            }
            
            props.addElement(oprop);
            
            // add domain and range to the property
            if(oprop != null){
                Iterator it = kb.getRelationsFrom(prop).iterator();
                while(it.hasNext()){
                    Relation rel = (Relation) it.next();
                    if(rel instanceof HasDomain){
                        oprop.addDomain(model.getResource(getFullConceptName(rel.getTarget().getNameSpace(),rel.getTarget().getName())));
                    } else if(rel instanceof HasRange){
                        oprop.addRange(model.getResource(getFullConceptName(rel.getTarget().getNameSpace() ,rel.getTarget().getName())));
                    }
                }
            }
        }
        
        // add superproperties to each property
        for(int i = 0; i < props.size(); i++){
            OntProperty oprop = (OntProperty)props.elementAt(i);
            if(oprop == null)
                continue;
            Concept prop = kb.getConcept(new Concept(oprop.getNameSpace(), oprop.getLocalName()));
            Iterator it = kb.getRelationsFrom(prop).iterator();
            while(it.hasNext()){
                Relation rel = (Relation)it.next();
                if(rel instanceof SubclassOf){
                    OntProperty aprop = model.getOntProperty(getFullConceptName(rel.getTarget().getNameSpace(),rel.getTarget().getName()));
                    if(aprop != null)
                        oprop.addProperty(RDFS.subPropertyOf,aprop);
                }
            }
        }

    }
    
    private void saveInstances(cz.cvut.km.model.KnowledgeBase kb, OntModel model){
        
        ConceptIterator ci = kb.getConcepts();
        // add properties to the model
        while(ci.hasNext()){
            Concept inst = ci.nextConcept();
            if(OwlJena.TYPE_OWL_INSTANCE.equals(inst.getProperty(OwlJena.OWLTYPE))){
                Iterator it = kb.getRelationsFrom(inst).iterator();
                while(it.hasNext()){
                    Relation rel = (Relation) it.next();
                    if(rel instanceof InstanceOf){
                        Concept cls = rel.getTarget();
                        Resource ocls = model.getOntClass(getFullConceptName(cls.getNameSpace(),cls.getName()));
                        if(ocls == null)
                            ocls = OWL.Thing;
                        Individual ind = model.createIndividual(getFullConceptName(inst.getNameSpace(),inst.getName()),ocls);
                    }
                }
            }
        }
        
        // add assignments to the model
        ci = kb.getConcepts();
        while(ci.hasNext()){
            Concept assign = ci.nextConcept();
            if(OwlJena.TYPE_ASSIGNMENT.equals(assign.getProperty(OwlJena.OWLTYPE))){
                Iterator it = kb.getRelationsFrom(assign).iterator();
                Individual ind = null, obj = null;
                Property prop = null;
                while(it.hasNext()){
                    Relation rel = (Relation)it.next();
                    if(rel instanceof PropertyOf){
                       ind = model.getIndividual(getFullConceptName(rel.getTarget().getNameSpace(),rel.getTarget().getName()));
                    } else if(rel instanceof HasValue){
                       obj = model.getIndividual(getFullConceptName(rel.getTarget().getNameSpace(),rel.getTarget().getName()));
                    } else if(rel instanceof InstanceOf){
                        prop = model.getProperty(rel.getTarget().getNameSpace(), rel.getTarget().getName());
                    }
                }
                if(ind != null && prop != null && obj != null && !isSystemConcept(new Concept(prop.getNameSpace(),prop.getLocalName())))
                    ind.addProperty(prop, obj);
            }
        }
    }
    
    public boolean supports(String action) {
        return action.equals(SUPPORTS_LOAD) || action.equals(SUPPORTS_SAVE);
    }
    
    private String getFullConceptName(String namespace, String localName){
        String fullName;
        if ( !namespace.endsWith("#") && !namespace.endsWith("/"))
            fullName = namespace+ "#" + localName;
        else
            fullName = namespace + localName;
	return fullName;	
    }
}
    

