/*
 * Concept.java
 *
 * Created on May 5, 2004, 11:44 AM
 */

package cz.cvut.km.model;

import java.util.*;

/**
 * A concept in the model language
 * @author  Petr Aubrecht
 */
public class Concept extends GraphEntity implements Comparable {
    protected String namespace = null;
    protected String name = "";
    protected String caption = null; // human readable caption; if null, use name
    
    /** Creates a new instance of Concept */
    public Concept() {
    }
    
    /** Creates a new instance of Concept */
    public Concept(String ns, String n) {
        namespace = ns;
        name = n;
    }
    
    /** set name of the concept
     */
    public void setName(String n) {
        name = n;
    }
    /** set namespace (or null if none) of the concept
     */
    public void setNameSpace(String ns) {
        namespace = ns;
    }
    
    /** get name of the concept
     */
    public String getName() {
        return name;
    }
    /** get namespace (or null if none) of the concept
     */
    public String getNameSpace() {
        return namespace;
    }
    
    /** set caption of the concept (human readable label)
     */
    public void setCaption(String c) {
        caption = c;
    }
    
    /** return human readable caption. If caption is null, returns toString()
     */
    public String getCaption() {
        if(caption==null)
            return toString();
        else
            return caption;
    }
    
    /**
     * Returns a string representation of the object. 
     * @return  a string representation of the object.
     */
    public String toString() {
        return (namespace==null?"":(namespace+":"))+name;
    }    
    
    /**
     * Indicates whether some other object is "equal to" this one.
     * <p>
     * The <code>equals</code> method implements an equivalence relation
     * on non-null object references:
     * <ul>
     * <li>It is <i>reflexive</i>: for any non-null reference value
     *     <code>x</code>, <code>x.equals(x)</code> should return
     *     <code>true</code>.
     * <li>It is <i>symmetric</i>: for any non-null reference values
     *     <code>x</code> and <code>y</code>, <code>x.equals(y)</code>
     *     should return <code>true</code> if and only if
     *     <code>y.equals(x)</code> returns <code>true</code>.
     * <li>It is <i>transitive</i>: for any non-null reference values
     *     <code>x</code>, <code>y</code>, and <code>z</code>, if
     *     <code>x.equals(y)</code> returns <code>true</code> and
     *     <code>y.equals(z)</code> returns <code>true</code>, then
     *     <code>x.equals(z)</code> should return <code>true</code>.
     * <li>It is <i>consistent</i>: for any non-null reference values
     *     <code>x</code> and <code>y</code>, multiple invocations of
     *     <tt>x.equals(y)</tt> consistently return <code>true</code>
     *     or consistently return <code>false</code>, provided no
     *     information used in <code>equals</code> comparisons on the
     *     objects is modified.
     * <li>For any non-null reference value <code>x</code>,
     *     <code>x.equals(null)</code> should return <code>false</code>.
     * </ul>
     * <p>
     * The <tt>equals</tt> method for class <code>Object</code> implements
     * the most discriminating possible equivalence relation on objects;
     * that is, for any non-null reference values <code>x</code> and
     * <code>y</code>, this method returns <code>true</code> if and only
     * if <code>x</code> and <code>y</code> refer to the same object
     * (<code>x == y</code> has the value <code>true</code>).
     * <p>
     * Note that it is generally necessary to override the <tt>hashCode</tt>
     * method whenever this method is overridden, so as to maintain the
     * general contract for the <tt>hashCode</tt> method, which states
     * that equal objects must have equal hash codes.
     *
     * @param   obj   the reference object with which to compare.
     * @return  <code>true</code> if this object is the same as the obj
     *          argument; <code>false</code> otherwise.
     * @see     #hashCode()
     * @see     java.util.Hashtable
     */
    public boolean equals(Object obj) {
        if(obj==this)
            return true;
        if(obj instanceof Concept) {
            Concept c = (Concept)obj;
//            return getNameSpace().equalsIgnoreCase(c.getNameSpace()) 
//              && getName().equalsIgnoreCase(c.getName());
            return getNameSpace().equals(c.getNameSpace()) 
              && getName().equals(c.getName());
        }
        return false;
    }

    /**
     * Creates and returns a copy of this object.  The precise meaning
     * of "copy" may depend on the class of the object. The general
     * intent is that, for any object <tt>x</tt>, the expression:
     * @return     a clone of this instance.
     */
    public Object clone() {
        Concept c = new Concept(getNameSpace(),getName());
        c.setCaption(getCaption());
        c.setProperties(this);
        return c;
    }
    
    public int compareTo(Object o) {
        Concept c = (Concept)o;
        String ns1 = namespace;
        String ns2 = c.getNameSpace();
        if(ns1==null)
            ns1="";
        if(ns2==null)
            ns2="";
        int c1 = ns1.compareTo(ns2);
        if(c1!=0)
            return c1;
        return getName().compareTo(c.getName());
    }
    
}
