/*
 * GraphEntity.java
 *
 * Created on June 25, 2004, 9:08 AM
 */

package cz.cvut.km.model;

import java.util.*;

/**
 * Class used for property handling
 * @author  aubrech
 */
public class GraphEntity extends Properties {
    
    /** Creates a new instance of GraphEntity */
    public GraphEntity() {
    }
    
    /**
     * Returns a string representation of the object with an extra debug info (list of properties).
     * @return  a string representation of the object.
     */
    public String toDebugString() {
        return toDebugString(true,true);
    }
    
    /**
     * Returns a string representation of the object with an extra debug info (list of properties).
     * @return  a string representation of the object.
     * @param showId show the identifier (toString)
     * @param showAttributes show the set of attributes
     */
    public String toDebugString(boolean showId, boolean showAttributes) {
        String id = toString();
        String str = "";
        String delim = "";
        Enumeration keys = keys();
        while(keys.hasMoreElements()) {
            String key = (String)keys.nextElement();
            str+=delim+key+"="+getProperty(key);
            delim=",";
        }
        String ret = null;
        if(showId)
            ret=id;
        if(showAttributes) {
            if(ret==null)
                ret = str;
            else
                ret = ret+" ["+str+"]";
        }
        return ret;
    }
    
    /**
     * Indicates whether some other object is "equal to" this one.
     * <p>
     * The <code>equals</code> method implements an equivalence relation
     * on non-null object references:
     * <ul>
     * <li>It is <i>reflexive</i>: for any non-null reference value
     *     <code>x</code>, <code>x.equals(x)</code> should return
     *     <code>true</code>.
     * <li>It is <i>symmetric</i>: for any non-null reference values
     *     <code>x</code> and <code>y</code>, <code>x.equals(y)</code>
     *     should return <code>true</code> if and only if
     *     <code>y.equals(x)</code> returns <code>true</code>.
     * <li>It is <i>transitive</i>: for any non-null reference values
     *     <code>x</code>, <code>y</code>, and <code>z</code>, if
     *     <code>x.equals(y)</code> returns <code>true</code> and
     *     <code>y.equals(z)</code> returns <code>true</code>, then
     *     <code>x.equals(z)</code> should return <code>true</code>.
     * <li>It is <i>consistent</i>: for any non-null reference values
     *     <code>x</code> and <code>y</code>, multiple invocations of
     *     <tt>x.equals(y)</tt> consistently return <code>true</code>
     *     or consistently return <code>false</code>, provided no
     *     information used in <code>equals</code> comparisons on the
     *     objects is modified.
     * <li>For any non-null reference value <code>x</code>,
     *     <code>x.equals(null)</code> should return <code>false</code>.
     * </ul>
     * <p>
     * The <tt>equals</tt> method for class <code>Object</code> implements
     * the most discriminating possible equivalence relation on objects;
     * that is, for any non-null reference values <code>x</code> and
     * <code>y</code>, this method returns <code>true</code> if and only
     * if <code>x</code> and <code>y</code> refer to the same object
     * (<code>x == y</code> has the value <code>true</code>).
     * <p>
     * Note that it is generally necessary to override the <tt>hashCode</tt>
     * method whenever this method is overridden, so as to maintain the
     * general contract for the <tt>hashCode</tt> method, which states
     * that equal objects must have equal hash codes.
     *
     * @param   obj   the reference object with which to compare.
     * @return  <code>true</code> if this object is the same as the obj
     *          argument; <code>false</code> otherwise.
     * @see     #hashCode()
     * @see     java.util.Hashtable
     */
    public boolean equals(Object obj) {
        if(obj==this)
            return true;
        return false;
    }
    
    /** Set all the properties
     * @param props the set of properties to be set
     */
    public void setProperties(Properties props) {
        Enumeration keys = props.keys();
        while(keys.hasMoreElements()) {
            String next = (String)keys.nextElement();
            setProperty(next, props.getProperty(next));
        }
    }
    
    /**
     * Returns an enumeration of the keys of the properties.
     * @return  an enumeration of the keys in this dictionary.
     */
    public Enumeration getPropertyKeys() {
        return keys();
    }    
    
    
}
