/*
 * KnowledgeBase.java
 *
 * Created on May 5, 2004, 12:37 PM
 */

package cz.cvut.km.model;

import cz.cvut.km.model.iterators.*;
import cz.cvut.km.utils.*;
import java.util.*;
import java.io.*;

/**
 *
 * @author  aubrech
 */
public class KnowledgeBase {
    String name = "";
    /** name of the system namespace
     */
    Vector systemNamespaces = new Vector();
    Vector concepts = new Vector();
    TreeMap conceptsMap = new TreeMap();
    Vector relations = new Vector();
    
    /** Creates an empty Knowledge Base */
    public KnowledgeBase() {
    }
    
    /** get name of the knoledge base
     */
    public String getName() {
        return name;
    }
    
    /** set name of the knowledge base
     * @param name new name of the knowledge base
     */
    public void setName(String name) {
        this.name = name;
    }
    /** add the concept to a knowledge base
     */
    public void addConcept(Concept c) {
        concepts.add(c);
        conceptsMap.put(c,c);
    }
    
    /** remove the concept from a knowledge base
     */
    public void removeConcept(Concept c) {
        concepts.remove(c);
        conceptsMap.remove(c);
    }
    
    /** remove the relation from the kb
     */
    public boolean removeRelation(Relation r) {
        return relations.remove(r);
    }
    
    public void removeConceptWithRelations(Concept c){
        Vector rels = new Vector();
        Iterator it = relations.iterator();
        while(it.hasNext()) {
            RelationInterface rel = (RelationInterface)it.next();
            if(rel.getSource().equals(c) || rel.getTarget().equals(c))
                rels.addElement(rel);
        }
        for(int i = 0; i < rels.size(); i++){
            RelationInterface rel = (RelationInterface)rels.elementAt(i);
            relations.remove(rel);
        }
        concepts.remove(c);
        conceptsMap.remove(c);
    }
    
    /** get all concepts
     */
    public ConceptIterator getConcepts() {
        return new ConceptIterator(concepts.iterator());
    }
    
    /** get all concepts in a raw form
     */
    public Vector getRawConcepts() {
        return concepts;
    }
    
    /** get all concepts in a raw form
     */
    public Vector getRawRelations() {
        return relations;
    }
    
    /** return a particular concept (or null if not found) based on equals
     * @param template a template of the concept
     */
    public Concept getConcept(Concept template) {
        return (Concept)conceptsMap.get(template);
        /*
        int index = concepts.indexOf(template);
        if(index<0)
            return null;
        else
            return (Concept)concepts.get(index);
         */
    }
    
    public void addRelation(RelationInterface rel) {
        relations.add(rel);
    }
    
    /** get all concepts.
     */
    public RelationIterator getRelations() {
        return new RelationIterator(relations.iterator());
    }
    
    /**
     * Returns a particular relation (or null if not found).
     * @param name a name of the relation in form [namespace:]name
     * @deprecated This method should be replaced by getRelation(Relation).
     */
    public Relation getRelation(String name) {
        for(int i=0;i<relations.size();i++)
            if(relations.get(i).toString().equals(name))
                return (Relation)relations.get(i);
        return null;
    }
    
    /** return a particular relation (or null if not found)
     * @param template the template relation (can be from another kb
     * @returns tre retured relation for which equals(template) is true
     */
    public Relation getRelation(Relation template) {
        for(int i=0;i<relations.size();i++)
            if(relations.get(i).equals(template))
                return (Relation)relations.get(i);
        return null;
    }
    
    /** return Vector of RelationInterfaces starting from the specified concept
     */
    public Vector getRelationsFrom(Concept c) {
        Vector res = new Vector();
        Iterator it = relations.iterator();
        while(it.hasNext()) {
            RelationInterface rel = (RelationInterface)it.next();
            if(rel.getSource()==c)
                res.add(rel);
        }
        return res;
    }
    
       
    /** return Vector of RelationInterfaces ending in the specified concept
     */
    public Vector getRelationsTo(Concept c) {
        Vector res = new Vector();
        Iterator it = relations.iterator();
        while(it.hasNext()) {
            RelationInterface rel = (RelationInterface)it.next();
            if(rel.getTarget()==c)
                res.add(rel);
        }
        return res;
    }
    
    public Concept getInstanceOf(Concept c) {
        Iterator it = relations.iterator();
        while(it.hasNext()) {
            RelationInterface rel = (RelationInterface)it.next();
            if(rel.getSource()==c && rel instanceof InstanceOf)
                return rel.getTarget();
        }
        return null;
    }
    
    public Concept getRangeOf(Concept c){
        Iterator it = relations.iterator();
        while(it.hasNext()) {
            RelationInterface rel = (RelationInterface)it.next();
            if(rel.getSource()==c && rel instanceof HasRange)
                return rel.getTarget();
        }
        return null;
    }
 
    /** write out the whole knowledge base in a text form
     * @param out the output writer for the flush
     */
    public void textFlush(OutputStream out) {
        textFlush(new OutputStreamWriter(out));
    }
    
    /** write out the whole knowledge base in a text form
     * @param out the output writer for the flush
     */
    public void textFlush(Writer out) {
        PrintWriter pw = new PrintWriter(out);
        pw.println("Concepts:");
        ConceptIterator ci = getConcepts();
        while(ci.hasNext())
            pw.println(ci.nextConcept().toDebugString());
        pw.println("Relations:");
        RelationIterator ri = getRelations();
        while(ri.hasNext())
            pw.println(ri.nextRelation().toDebugString());
        pw.close();
    }

    /** write out the whole knowledge base in a HTML form
     * @param out the output writer for the flush
     * @param properties properties to setup the output
     */
    public void htmlFlush(OutputStream out, Properties properties) {
        htmlFlush(new OutputStreamWriter(out),properties);
    }
    
    /** write out the whole knowledge base in a HTML form
     * @param out the output writer for the flush
     * @param properties properties to setup the output
     */
    public void htmlFlush(Writer out, Properties properties) {
        String colorify = properties.getProperty("colorify");
        String title = properties.getProperty("title","");
        PrintWriter pw = new PrintWriter(out);
        pw.println("<html><head>"+properties.getProperty("headattributes","")+"<title>"+title+"</title></head><body>");
        if(title.length()>0)
            pw.println("<h1>"+title+"</h1>");
        pw.println(properties.getProperty("textbefore",""));
        pw.println("<p><table border=1><tr><th colspan=3>Concepts:</th></tr>");
        pw.println("<tr><th>namespace</th><th>name</th><th>attributes</th></tr>");
        ConceptIterator ci = getConcepts();
        while(ci.hasNext()) {
            Concept c = ci.nextConcept();
            String color = c.getProperty(colorify,""); // get the attribute
            if(color.length()>0) 
                color = properties.getProperty(color,""); // get the actual color
            if(color.length()>0)
                color = "style='"+color+"'";
            pw.println("<tr "+color+"><td>"+StringUtils.escapeHTML(c.getNameSpace())+"&nbsp</td><td>"+
                StringUtils.escapeHTML(c.getName())+"</td><td>"+
                StringUtils.escapeHTML(c.toDebugString(false,true))+"&nbsp;</td></tr>");
        }
        pw.println("</table></p>");
        pw.println();
        pw.println("<p><table border=1><tr><th colspan=3>Relations:</th></tr>");
        pw.println("<tr><th>namespace</th><th>name</th><th>attributes</th></tr>");
        RelationIterator ri = getRelations();
        while(ri.hasNext()) {
            Relation rel = ri.nextRelation();
            String color = rel.getProperty(colorify,""); // get the attribute
            if(color.length()>0) 
                color = properties.getProperty(color,""); // get the actual color
            if(color.length()>0)
                color = "style='"+color+"'";
            pw.println("<tr "+color+"><td>"+StringUtils.escapeHTML(rel.getSource().toString())+"&nbsp;</td><td>"+
                StringUtils.escapeHTML(rel.getTextArrow())+"</td><td>"+
                StringUtils.escapeHTML(rel.getTarget().toString())+"&nbsp;</td></tr>");
            //pw.println("<tr><td>"+ri.nextRelation().toDebugString()+"</td></tr>");
        }
        pw.println("</table></p>");
        pw.println(properties.getProperty("textafter",""));
        pw.println("</body></html>");
        pw.close();
    }

    /** get name of the system namespace
     */
    public Iterator getSystemNamespaces() {
        return systemNamespaces.iterator();
    }

    /** set name of the system namespace
     */
    public void addSystemNamespace(String newSystemNamespace) {
        systemNamespaces.add(newSystemNamespace);
    }
    
    /** return true if the namespace is one of the system namespaces
     */
    public boolean isSystemNamespace(String ns) {
        return systemNamespaces.contains(ns);
    }
}
