/*
 * ConceptIterator.java
 *
 * Created on July 16, 2004, 10:57 AM
 */

package cz.cvut.km.model.iterators;

import cz.cvut.km.model.*;

import java.util.*;

/**
 * ConceptIterator is a wrap around Iterator to return Concepts
 * @author  aubrech
 */
public class ConceptIterator implements Iterator {
    protected Iterator iter = null;
    
    /** Creates a new instance of ConceptIterator 
     * @param original iterator
     */
    public ConceptIterator(Iterator it) {
        iter = it;
    }
    
    /**
     * Returns <tt>true</tt> if the iteration has more elements. (In other
     * words, returns <tt>true</tt> if <tt>next</tt> would return an element
     * rather than throwing an exception.)
     *
     * @return <tt>true</tt> if the iterator has more elements.
     */
    public boolean hasNext() {
        return iter.hasNext();
    }
    
    /**
     * Returns the next element in the iteration.
     *
     * @return the next element in the iteration.
     * @exception NoSuchElementException iteration has no more elements.
     */
    public Object next() {
        return iter.next();
    }
    
    /**
     * Returns the next element in the iteration.
     *
     * @return the next element in the iteration.
     * @exception NoSuchElementException iteration has no more elements.
     */
    public Concept nextConcept() {
        return (Concept)iter.next();
    }
    
    /**
     *
     * Removes from the underlying collection the last element returned by the
     * iterator (optional operation).  This method can be called only once per
     * call to <tt>next</tt>.  The behavior of an iterator is unspecified if
     * the underlying collection is modified while the iteration is in
     * progress in any way other than by calling this method.
     *
     * @exception UnsupportedOperationException if the <tt>remove</tt>
     * 		  operation is not supported by this Iterator.
     *
     * @exception IllegalStateException if the <tt>next</tt> method has not
     * 		  yet been called, or the <tt>remove</tt> method has already
     * 		  been called after the last call to the <tt>next</tt>
     * 		  method.
     */
    public void remove() {
        iter.remove();
    }
    
    
}
