/*
 * FilterIterator.java
 *
 * Created on July 16, 2004, 3:21 PM
 */

package cz.cvut.km.model.iterators;

import java.util.*;

/** 
 * Base filter iterator for all iterators
 * @author  aubrech
 */
public abstract class FilterIterator implements Iterator {
    /** hidden iterator */
    protected Iterator iter;
    /** current object */
    protected Object curr = null;

    /** Creates a new instance of FilterIterator */
    public FilterIterator(Iterator origIter) {
        iter = origIter;
        findNext();
    }
    
    /** Check the object if it is valid (in terms of filtering)
     * @param o object to check
     * @return return true if the o object have to be taken into account
     */
    protected abstract boolean checkObject(Object o);

    /**
     * Returns <tt>true</tt> if the iteration has more elements. (In other
     * words, returns <tt>true</tt> if <tt>next</tt> would return an element
     * rather than throwing an exception.)
     *
     * @return <tt>true</tt> if the iterator has more elements.
     */
    public boolean hasNext() {
        return curr!=null;
    }

    /**
     * Returns the next element in the iteration.
     *
     * @return the next element in the iteration.
     * @exception NoSuchElementException iteration has no more elements.
     */
    public Object next() {
        Object ret = curr;
        findNext();
        return ret;
    }
    
    /** find the next acceptable element
     */
    protected void findNext() {
        curr=null;
        while(iter.hasNext()) {
            Object o = iter.next();
            if(checkObject(o)) {
                curr = o;
                return;
            }
        }
    }

    /**
     *
     * Removes from the underlying collection the last element returned by the
     * iterator (optional operation).  This method can be called only once per
     * call to <tt>next</tt>.  The behavior of an iterator is unspecified if
     * the underlying collection is modified while the iteration is in
     * progress in any way other than by calling this method.
     *
     * @exception UnsupportedOperationException if the <tt>remove</tt>
     * 		  operation is not supported by this Iterator.
     *
     * @exception IllegalStateException if the <tt>next</tt> method has not
     * 		  yet been called, or the <tt>remove</tt> method has already
     * 		  been called after the last call to the <tt>next</tt>
     * 		  method.
     */
    public void remove() {
        throw new UnsupportedOperationException("FilterIterator.remove not implemented!");
    }
}
