/*
 * diff.java
 *
 * Created on July 23, 2004, 11:03 PM
 */

package cz.cvut.km.oper;

import cz.cvut.km.model.*;
import cz.cvut.km.model.iterators.*;
import java.util.*;

/**
 *
 * @author  aubrech
 */
public class Diff {
    
    /** Creates a new instance of diff */
    protected Diff() {
    }

    /** Compare the given knowledge bases and generate a diff.
     * @param left first kb
     * @param right second kb
     * @return Returns the difference, while concepts/relations appearing in the left/right
     * knowledge base have property "origin" set to "left"/"right". The shared concepts are
     * not present unless they are connected to relations presented in one kb.
     */
    public static KnowledgeBase processDiff(KnowledgeBase left, KnowledgeBase right) {
        KnowledgeBase ret = new KnowledgeBase();
        findMissingConcepts(ret,left,right,"left");
        findMissingRelations(ret,left,right,"left");
        //System.out.println("---");
        findMissingConcepts(ret,right,left,"right");
        findMissingRelations(ret,right,left,"right");
        return ret;
    }
    
    protected static void findMissingConcepts(KnowledgeBase ret, KnowledgeBase left, KnowledgeBase right, String dir) {
        ConceptIterator leftConcepts = null;
        leftConcepts = left.getConcepts();
        while(leftConcepts.hasNext()) {
            Concept leftC = leftConcepts.nextConcept();
            Concept rightC = right.getConcept(leftC);
            if(rightC==null) {
                Concept nc = (Concept)leftC.clone();
                nc.setProperty("origin", dir);
                ret.addConcept(nc);
                //System.out.println(dir+" "+leftC.toString());
            }
        }
    }
    
    protected static void findMissingRelations(KnowledgeBase ret, KnowledgeBase left, KnowledgeBase right, String dir) {
        RelationIterator leftRelations = null;
        leftRelations = left.getRelations();
        while(leftRelations.hasNext()) {
            Relation leftR = leftRelations.nextRelation();
            Relation rightR = right.getRelation(leftR);
            if(rightR==null) {
                Relation nr = (Relation)leftR.clone();
                nr.setProperty("origin", dir);
                Concept source = nr.getSource();
                if(ret.getConcept(source)==null)
                    ret.addConcept((Concept)source.clone());
                Concept target = nr.getTarget();
                if(ret.getConcept(target)==null)
                    ret.addConcept((Concept)target.clone());
                ret.addRelation(nr);
                //System.out.println(dir+" "+leftR.toString());
            }
        }
    }
    
}
