/*
 * Subset.java
 *
 * Created on July 23, 2004, 11:03 PM
 */

package cz.cvut.km.oper;

import cz.cvut.km.model.*;
import cz.cvut.km.model.iterators.*;
import java.util.*;

/**
 *
 * @author  aubrech
 */
public class Subset {
    
    /** Creates a new instance of Subset */
    protected Subset() {
    }
    
    /** Create a subset of KB.
     * @param kb the source knowledge base
     * @param filter filter used to make the subset
     * @return Returns a subset of the source kb
     */
    public static KnowledgeBase createSubset(KnowledgeBase kb, Filter filter) {
        KnowledgeBase ret = new KnowledgeBase();
        ConceptIterator ci = kb.getConcepts();
        while(ci.hasNext()) {
            Concept c = ci.nextConcept();
            if(filter.acceptConcept(c))
                ret.addConcept(c);
        }
        
        RelationIterator ri = kb.getRelations();
        while(ri.hasNext()) {
            Relation r = ri.nextRelation();
            if(filter.acceptRelation(r))
                ret.addRelation(r);
        }
        return ret;
    }
    
    /** Create a subset of KB, where only given relation remain, others are filtered. Also
     * only the connected concepts
     * @param kb the source knowledge base
     * @param relationClass the class of the relation (subclass of Relation) to remain
     */
    public static KnowledgeBase createSubsetByRelation(KnowledgeBase kb, Class relationClass) {
        final KnowledgeBase fKb = kb;
        final Class fRelationClass = relationClass;
        Filter filter = new Filter() {
            public boolean acceptConcept(Concept c) {
                Vector rels = fKb.getRelationsFrom(c);
                rels.addAll(fKb.getRelationsTo(c));
                for(int i=0;i<rels.size();i++)
                    if(rels.get(i).getClass()==fRelationClass)
                        return true;
                return false;
            }
            public boolean acceptRelation(Relation r) {
                return r.getClass() == fRelationClass;
            }
        };
        KnowledgeBase ret = createSubset(kb,filter);
        return ret;
    }
    
    public static KnowledgeBase createNonSystemSubset(KnowledgeBase kb){
        final KnowledgeBase fKb = kb;
        Filter filter = new Filter() {
            public boolean acceptConcept(Concept c) {
                // if namespace of the concept is not included among system namespaces
                // accept the concept
                return !fKb.isSystemNamespace(c.getNameSpace());
            }
            // relation accepted if neither source nor target are system concepts
            public boolean acceptRelation(Relation r) {
                Concept source = r.getSource();
                Concept target = r.getTarget();
                return !fKb.isSystemNamespace(source.getNameSpace()) && !fKb.isSystemNamespace(target.getNameSpace());
            }
        };
        KnowledgeBase ret = createSubset(kb,filter);
        return ret;
    }
}
