/*
 * Union.java
 *
 * Created on July 23, 2004, 11:03 PM
 */

package cz.cvut.km.oper;

import cz.cvut.km.model.*;
import cz.cvut.km.model.iterators.*;
import java.util.*;

/**
 *
 * @author  aubrech
 */
public class Union {
    
    /** Creates a new instance of Union */
    protected Union() {
    }

    /** Join the given knowledge bases.
     * @param left first kb
     * @param right second kb
     * @return Returns merged bases. Concepts/relations appearing in the left/right
     * knowledge base have property "origin" set to "left"/"right". Concepts/relations
     * in both bases are set to "both".
     */
    public static KnowledgeBase processUnion(KnowledgeBase left, KnowledgeBase right) {
        KnowledgeBase ret = new KnowledgeBase();
        findMissingConcepts(ret,left,"left","");
        findMissingConcepts(ret,right,"right","both");
        findMissingRelations(ret,left,"left","");
        findMissingRelations(ret,right,"right","both");
        return ret;
    }

    /** add the missing concepts from the left kb to the ret one
     */
    protected static void findMissingConcepts(KnowledgeBase ret, KnowledgeBase left, String dir, String bothDirs) {
        ConceptIterator leftConcepts = null;
        leftConcepts = left.getConcepts();
        while(leftConcepts.hasNext()) {
            Concept leftC = leftConcepts.nextConcept();
            Concept retC = ret.getConcept(leftC);
            if(retC==null) {
                Concept nc = (Concept)leftC.clone();
                nc.setProperty("origin", dir);
                ret.addConcept(nc);
                System.out.println(dir+" "+leftC.toString());
            } else
                ret.getConcept(leftC).setProperty("origin", bothDirs);
        }
    }
    
    /** add the missing relaions from the left kb to the ret one
     */
    protected static void findMissingRelations(KnowledgeBase ret, KnowledgeBase left, String dir, String bothDirs) {
        RelationIterator leftRelations = null;
        leftRelations = left.getRelations();
        while(leftRelations.hasNext()) {
            Relation leftR = leftRelations.nextRelation();
            Concept leftSource = leftR.getSource();
            Relation rightR = ret.getRelation(leftR);
            if(rightR==null) {
                Relation nr = (Relation)leftR.clone();
                nr.setProperty("origin", dir);
                System.out.println("old: "+nr.toDebugString());
                nr.setSource(ret.getConcept(nr.getSource()));
                nr.setTarget(ret.getConcept(nr.getTarget()));
                ret.addRelation(nr);
                System.out.println("adding "+dir+" "+nr.toDebugString());
            } else
                rightR.setProperty("origin", bothDirs);
        }
    }
    
}
