/*
 * AllOutputs.java
 *
 * Created on July 29, 2004, 2:48 PM
 */

package cz.cvut.km.tests;

import cz.cvut.km.model.*;
import java.util.*;

/**
 * Class for save a KnowledgeBase in all known formalisms
 * @author  aubrech
 */
public class AllOutputs {
    protected boolean debugOutput = false; // write what is done?
    protected boolean runDot = true; // run external command -- dot (from GraphViz)
    protected String outName = "test"; // core of name of the output files
    
    /** Creates a new instance of AllOutputs */
    public AllOutputs() {
    }
    
    /** Creates a new instance of AllOutputs */
    public AllOutputs(String fileName) {
        outName = fileName;
    }
    
    /** disable running the dot command (GraphViz)
     */
    public void disableDot() {
        runDot = false;
    }
    
    /** enable/disable progress output
     */
    public void setDebugOutput(boolean debOut) {
        debugOutput = debOut;
    }
    
    public void export(KnowledgeBase kb) {
        // Text
        try {
            if(debugOutput)
                System.out.println("Saving to "+outName+".txt...");
            kb.textFlush(new java.io.FileOutputStream(outName+".txt"));
        } catch(Exception e) {
            e.printStackTrace();
        }
        
        // HTML
        try {
            if(debugOutput) System.out.println("Saving to "+outName+".html...");
            Properties htmlProps = new Properties();
            htmlProps.setProperty("textbefore","<p>Legend: items was found in the <span style='background: #ffc0c0'>left</span> and <span style='background: #c0ffc0'>right</span> knowledge base.</p>");
            htmlProps.setProperty("title","Ontology "+kb.getName());
            htmlProps.setProperty("colorify","origin"); //colorify according to the origin parameter
            htmlProps.setProperty("left","background: #ffc0c0");
            htmlProps.setProperty("right","background: #c0ffc0");
            kb.htmlFlush(new java.io.FileOutputStream(outName+".html"), htmlProps);
        } catch(Exception e) {
            e.printStackTrace();
        }

        try {
            if(debugOutput) System.out.println("Saving graph to "+outName+".bf...");
            Properties parameters = new Properties();
            parameters.put("filename",outName+".bf");
            new cz.cvut.km.io.BabelFish().saveKB(kb, parameters);
        } catch(Exception e) {
            e.printStackTrace();
        }

        // Dot
        try {
            if(debugOutput) System.out.println("Saving KB to Dot...");
            Properties parameters = new Properties();
            parameters.put("filename",outName+".dot");
            new cz.cvut.km.io.GraphViz().saveKB(kb, parameters);
            if(runDot) {
                if(debugOutput) System.out.println("Running dot to prepare "+outName+".ps...");
                cz.cvut.km.utils.FileUtils.saveProcessOuputToFile("dot -Tps "+outName+".dot", outName+".ps");
                if(debugOutput) System.out.println("... and "+outName+".gif...");
                cz.cvut.km.utils.FileUtils.saveProcessOuputToFile("dot -Tgif "+outName+".dot", outName+".gif");
                               
            } else
                if(debugOutput) System.out.println("Run dot manually: 'dot -Tps "+outName+".dot' or 'dot -Tgif "+outName+".dot'");
        } catch(Exception e) {
            e.printStackTrace();
        }
        
        // Apollo not informed
        //*
        try {
            if(debugOutput) System.out.println("Saving graphs to Apollo ("+outName+".xml)...");
            // validate it is ok to save
            cz.cvut.km.io.Apollo apollo = new cz.cvut.km.io.Apollo();
            // first classify
            apollo.classifyKB(kb);
            // then validate
            apollo.validateKB(kb);
            
            Properties parameters = new Properties();
            parameters.put("filename",outName+".xml");
            //parameters.put("prefix","sumo");
            parameters.put("prefix","ap");
            apollo.saveKB(kb, parameters);
        } catch(Exception e) {
            e.printStackTrace();
        }
        //*/
        // Prolog
        try {
            if(debugOutput) System.out.println("Saving graphs to "+outName+".pl...");
            Properties parameters = new Properties();
            parameters.put("filename",outName+".pl");
            new cz.cvut.km.io.Prolog().saveKB(kb, parameters);
        } catch(Exception e) {
            e.printStackTrace();
        }
        
        // Visualisation - Prefuse
        try {
            if(debugOutput) System.out.println("Saving graphs to Prefuse: various/prefuse/build/etc/terror.xml...");
            Properties parameters = new Properties();
            parameters.put("filename","various/prefuse/build/etc/terror.xml");
            //parameters.put("showNS","false");
            new cz.cvut.km.io.Prefuse().saveKB(kb, parameters);
            if(debugOutput) System.out.println("Run java -cp demos.jar:prefuse.jar:prefusex.jar edu.berkeley.guir.prefuse.demos.RadialGraphDemo");
        } catch(Exception e) {
            e.printStackTrace();
        }
        
        // Visualisation - Hypergraph
        try {
            if(debugOutput) System.out.println("Saving graphs to various/hypergraph/"+outName+".xml...");
            Properties parameters = new Properties();
            parameters.put("filename","various/hypergraph/"+outName+".xml");
            //parameters.put("showNS","false");
            new cz.cvut.km.io.Hypergraph().saveKB(kb, parameters);
            if(debugOutput) System.out.println("View the various/hypergraph/appletTest.html in Java-enabled browser.");
        } catch(Exception e) {
            e.printStackTrace();
        }
    }
    
}
