/*
 * AubiDisertace.java
 *
 * Created on December 16, 2004, 12:54 PM
 */

package cz.cvut.km.tests;

import cz.cvut.km.model.*;
import java.util.*;
import java.io.*;
import java.text.*;

/**
 *
 * @author  aubrech
 */
public class AubiDisertace {
    boolean doCyc = true;
    protected String benchmarkFile;
    // variables for benchmarking
    long start, end;
    // used for outputing results
    AllOutputs allOutputs;
    Properties parameters,parametersFS;
    
    /** Creates a new instance of AubiDisertace */
    public AubiDisertace() {
    }
    
    /**
     * @param args the command line arguments
     */
    public static void main(String[] args) {
        new AubiDisertace().start(1);
        //TODO: pustit dvakrat, aby se odstranilo dlouhy nacitani z benchmarku (a byly porovnatelny)
        //new AubiDisertace().start(2);
    }
    
    protected void benchmark(String category, String measurement, KnowledgeBase kb, String fname, long time) throws IOException {
        int concepts = 0;
        int relations = 0;
        if(kb!=null) {
            concepts = kb.getRawConcepts().size();
            relations = kb.getRawRelations().size();
        }
        System.out.println("BENCHMARK: "+category+",\t"+measurement+":\t"+time+" ms, "+concepts+"/"+relations+" items, stored to "+fname);
        PrintWriter fw = new PrintWriter(new FileWriter(benchmarkFile,true));
        fw.println(category+"\t"+measurement+"\t"+time+"\t"+concepts+"/"+relations+"\t"+fname);
        fw.close();
	//String maperr = kb.getErrorLog();
    }
    
    /** run all the tasks for automatical generation of implementation-relevant
     *  parts for the disertation
     */
    protected void start(int run) {
        try {
            // header
            System.out.println("Automaticky generovane vystupy pro Aubiho disertaci");
            SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd hh:mm:ss");
            System.out.println(sdf.format(Calendar.getInstance().getTime()));
            // remove the old
            benchmarkFile = "benchmark"+run+".txt";
            new File(benchmarkFile).renameTo(new File(benchmarkFile+"~"));
            System.out.println("******** RUN   "+run+"    *****************");
            System.out.println();
            
            System.out.println("Load Apollo");
            doTestApollo();
            System.out.println("Load OWL");
            doTestOwl();
            //System.out.println("TODO: Load SQL");
            //doTestSql();
            System.out.println("Load BookmarksMozilla");
            doTestBookmarks();
            System.out.println("OWL->Apollo uninformed");
            doTestOwl2Apollo();
            System.out.println("OWL->Apollo informed");
            doTestOwl2ApolloInf();
            System.out.println("Evaluating differencies between un/informed transf. OWL->Apollo");
            doTestOwl2ApolloDiff();
            System.out.println("Apollo->OWL uninformed");
            doTestApollo2Owl();
            System.out.println("Apollo->OWL informed");
            doTestApollo2OwlInf();
            //System.out.println("TODO: Evaluating differencies between un/informed transf. Apollo->OWL");
            //doTestApollo2OwlDiff();
            //*/
            System.out.println("Mapping engine");
            doTestMapping();
            System.out.println("============================================= All successfully done");
        } catch(Exception e) {
            e.printStackTrace();
        }
    }
    
    /** test Apollo input
     */
    protected void doTestApollo() throws Exception {
        // Starting warm-up
        System.out.println("Warming up Apollo");
        for(int i=0;i<10;i++) {
            start = System.currentTimeMillis();
            parametersFS = new Properties();
            parametersFS.put("source","file:testdata/historical_architecture.xml");
            parametersFS.put("system-concepts",i%2==1?"true":"false");
            KnowledgeBase kb = new cz.cvut.km.io.Apollo().loadKB(parametersFS);
            end = System.currentTimeMillis();
            System.out.println("Temperature: "+(end-start));
        }
        
        doTestApollo("historical_architecture",true);
        doTestApollo("cipher-crm",true);
        doTestApollo("historical_architecture",false);
        doTestApollo("cipher-crm",false);
    }
    
    protected void doTestApollo(String which, boolean fos) throws Exception {
        //System.out.println("Without system ontology");
        
        start = System.currentTimeMillis();
        parametersFS = new Properties();
        parametersFS.put("source","file:testdata/"+which+".xml");
        parametersFS.put("system-concepts",fos?"true":"false");
        KnowledgeBase kb = new cz.cvut.km.io.Apollo().loadKB(parametersFS);
        end = System.currentTimeMillis();
        benchmark("Apollo", "load "+which+" ontology "+(fos?"with":"without")+" system ontology",kb,"disoutApollo"+which+(fos?"Uninf":"Inf"),end-start);
        
        //kb.textFlush(System.out);
        allOutputs = new AllOutputs("disoutApollo"+which);
        allOutputs.export(kb);
    }
    
    protected void doTestBookmarks() throws Exception {
        start = System.currentTimeMillis();
        Properties parametersFS = new Properties();
        parametersFS.put("source","testdata/pabookmarks.html");
        KnowledgeBase kb = new cz.cvut.km.io.BookmarksMozilla().loadKB(parametersFS);
        end = System.currentTimeMillis();
        benchmark("Bookmarks", "load test bookmarks",kb,"disoutBookmarks",end-start);
        allOutputs = new AllOutputs("disoutBookmarks");
        allOutputs.export(kb);
    }
    
    protected void doTestApollo2Owl() throws Exception {
        doTestApollo2Owl("historical_architecture");
        doTestApollo2Owl("cipher-crm");
    }
    
    protected void doTestApollo2OwlInf() throws Exception {
        doTestApollo2OwlInf("historical_architecture");
        doTestApollo2OwlInf("cipher-crm");
    }
    
    protected void doTestApollo2Owl(String which) throws Exception {
        start = System.currentTimeMillis();
        parametersFS = new Properties();
        parametersFS.put("source","file:testdata/" + which + ".xml");
        parametersFS.put("system-concepts","false");
        KnowledgeBase kb = new cz.cvut.km.io.Apollo().loadKB(parametersFS);
        
        parameters = new Properties();
        parameters.put("filename","disoutApollo2Owl"+which+"Uninf.owl");
        parameters.put("namespace","http://www.cvut.cz/apollo#");
        new cz.cvut.km.io.OwlRdfsJena().saveKB(kb,parameters);
        end = System.currentTimeMillis();
        benchmark("Apollo2Owl", "uninformed transformation of "+which,kb,"disoutApollo2Owl"+which+"Uninf",end-start);
    }
    
    protected void doTestApollo2OwlInf(String which) throws Exception {
        start = System.currentTimeMillis();
        
        parametersFS = new Properties();
        parametersFS.put("source","file:testdata/"+ which +".xml");
        parametersFS.put("system-concepts","true");
        KnowledgeBase kb = new cz.cvut.km.io.Apollo().loadKB(parametersFS);
        
        // transform from Apollo to Owl
        cz.cvut.km.trans.Apollo2Owl trans = new cz.cvut.km.trans.Apollo2Owl();
        trans.transform(kb);
        
        kb = cz.cvut.km.oper.Subset.createNonSystemSubset(kb);
        parameters = new Properties();
        parameters.put("filename","disoutApollo2Owl"+which+"Inf.xml");
        new cz.cvut.km.io.OwlRdfsJena().saveKB(kb,parameters);
        
        end = System.currentTimeMillis();
        benchmark("Apollo2Owl", "informed transformation of "+which,kb,"disoutApollo2Owl"+which+"Inf",end-start);
        
    }
    
    protected void doTestOwl2Apollo() throws Exception {
        doTestOwl2Apollo("TrivialOntology");
        doTestOwl2Apollo("wine1");
        doTestOwl2Apollo("wine");
        doTestOwl2Apollo("SUMO");
        if(doCyc) doTestOwl2Apollo("cyc-2004-06-04");
    }
    
    protected void doTestOwl2Apollo(String which) throws Exception {
        start = System.currentTimeMillis();
        
        parametersFS = new Properties();
        parametersFS.put("source","file:testdata/"+which+".owl");
        parametersFS.put("system-concepts","false");
        parametersFS.put("nameparsing","#"); // use either # (parts/of/namespace#concept) or lastpart (parts/of/namespace/concept)
        KnowledgeBase kb = new cz.cvut.km.io.OwlRdfsJena().loadKB(parametersFS);
        System.out.println("DEBUG: OWL2ApolloUninf loaded, starting save -- "+new Date());
        cz.cvut.km.io.Apollo apollo = new cz.cvut.km.io.Apollo();
        parameters = new Properties();
        parameters.put("filename","disoutOwl2Apollo"+which+"Uninf.xml");
        parameters.put("prefix","gof_");
        apollo.saveKB(kb, parameters);
        end = System.currentTimeMillis();
        benchmark("Owl2Apollo", "uninformed transformation of "+which,kb,"disoutOwl2Apollo"+which+"Uninf",end-start);
    }
    
    protected void doTestOwl2ApolloInf() throws Exception {
        doTestOwl2ApolloInf("TrivialOntology");
        doTestOwl2ApolloInf("wine1");
        doTestOwl2ApolloInf("wine");
        doTestOwl2ApolloInf("SUMO");
        if(doCyc) doTestOwl2ApolloInf("cyc-2004-06-04");
    }
    
    protected void doTestOwl2ApolloInf(String which) throws Exception {
        start = System.currentTimeMillis();
        parametersFS = new Properties();
        parametersFS.put("source","file:testdata/"+which+".owl");
        parametersFS.put("system-concepts","true");
        parametersFS.put("nameparsing","#"); // use either # (parts/of/namespace#concept) or lastpart (parts/of/namespace/concept)
        KnowledgeBase kb = new cz.cvut.km.io.OwlRdfsJena().loadKB(parametersFS);
        System.out.println("DEBUG: OWL2ApolloInf loaded, starting save -- "+new Date());
        cz.cvut.km.trans.Owl2Apollo trans = new cz.cvut.km.trans.Owl2Apollo();
        trans.transform(kb);
        
        kb = cz.cvut.km.oper.Subset.createNonSystemSubset(kb);
        cz.cvut.km.io.Apollo apollo = new cz.cvut.km.io.Apollo();
        parameters = new Properties();
        parameters.put("filename","disoutOwl2Apollo"+which+"Inf.xml");
        parameters.put("prefix","gof_");
        apollo.saveKB(kb, parameters);
        
        end = System.currentTimeMillis();
        benchmark("Owl2Apollo", "informed transformation of "+which,kb,"disoutOwl2Apollo"+which+"Inf",end-start);
    }
    
    protected void doTestOwl() throws Exception {
        // Starting warm-up
        System.out.println("Warming up OWL");
        for(int i=0;i<10;i++) {
            start = System.currentTimeMillis();
            parametersFS = new Properties();
            parametersFS.put("source","file:testdata/TrivialOntology.owl");
            parametersFS.put("system-concepts",i%2==1?"true":"false");
            KnowledgeBase kb = new cz.cvut.km.io.OwlRdfsJena().loadKB(parametersFS);
            end = System.currentTimeMillis();
            System.out.println("Temperature: "+(end-start));
        }
        
        doTestOwl("TrivialOntology",true);
        doTestOwl("wine1",true);
        doTestOwl("wine",true);
        doTestOwl("SUMO",true);
        if(doCyc) doTestOwl("cyc-2004-06-04",true);
        doTestOwl("TrivialOntology",false);
        doTestOwl("wine1",false);
        doTestOwl("wine",false);
        doTestOwl("SUMO",false);
        if(doCyc) doTestOwl("cyc-2004-06-04",false);
    }
    
    protected void doTestOwl(String which, boolean fos) throws Exception {
        start = System.currentTimeMillis();
        parametersFS = new Properties();
        parametersFS.put("source","file:testdata/"+which+".owl");
        parametersFS.put("system-concepts",fos?"true":"false");
        KnowledgeBase kb = new cz.cvut.km.io.OwlRdfsJena().loadKB(parametersFS);
        
        end = System.currentTimeMillis();
        benchmark("Owl", "load "+which+" "+(fos?"with":"without")+" system ontology",kb,"disoutOwl"+which+(fos?"Uninf":"Inf"),end-start);
        
        //allOutputs = new AllOutputs("disoutOwl"+which);
        //allOutputs.export(kb);
    }
    
    protected void doTestSql() {
        
    }
    
    protected void doTestOwl2ApolloDiff() throws Exception {
        doTestOwl2ApolloDiff("TrivialOntology");
        doTestOwl2ApolloDiff("wine1");
        doTestOwl2ApolloDiff("wine");
        doTestOwl2ApolloDiff("SUMO");
        if(doCyc) doTestOwl2ApolloDiff("cyc-2004-06-04");
    }
    
    
    protected void doTestOwl2ApolloDiff(String which) throws Exception {
        parametersFS = new Properties();
        parametersFS.put("source","file:disoutOwl2Apollo"+which+"Inf.xml");
        parametersFS.put("system-concepts","false");
        KnowledgeBase kbInf = new cz.cvut.km.io.Apollo().loadKB(parametersFS);
        parametersFS = new Properties();
        parametersFS.put("source","file:disoutOwl2Apollo"+which+"Uninf.xml");
        parametersFS.put("system-concepts","false");
        KnowledgeBase kbUninf = new cz.cvut.km.io.Apollo().loadKB(parametersFS);
        start = System.currentTimeMillis();
        KnowledgeBase kb = cz.cvut.km.oper.Diff.processDiff(kbUninf,kbInf);
        end = System.currentTimeMillis();
        benchmark("Owl2ApolloDiff", "differences between un/informed Owl2Apollo "+which,kb,"disoutOwl2Apollo"+which+"Diff",end-start);
        
        //kb.textFlush(System.out);
        allOutputs = new AllOutputs("disoutOwl2Apollo"+which+"Diff");
        allOutputs.export(kb);
    }
    
    protected void doTestApollo2OwlDiff() throws Exception {
        doTestOwl2ApolloDiff("historical_architecture");
    }
    
    
    protected void doTestApollo2OwlDiff(String which) throws Exception {
        parametersFS = new Properties();
        parametersFS.put("source","file:disoutApollo2Owl"+which+"Inf.xml");
        parametersFS.put("system-concepts","false");
        KnowledgeBase kbInf = new cz.cvut.km.io.OwlRdfsJena().loadKB(parametersFS);
        parametersFS = new Properties();
        parametersFS.put("source","file:disoutApollo2Owl"+which+"Uninf.xml");
        parametersFS.put("system-concepts","false");
        KnowledgeBase kbUninf = new cz.cvut.km.io.OwlRdfsJena().loadKB(parametersFS);
        start = System.currentTimeMillis();
        KnowledgeBase kb = cz.cvut.km.oper.Diff.processDiff(kbUninf,kbInf);
        end = System.currentTimeMillis();
        benchmark("Apollo2OwlDiff", "differences between un/informed Apollo2Owl "+which,kb,"disoutApollo2Owl"+which+"Diff",end-start);
        allOutputs = new AllOutputs("disoutApollo2Owl"+which+"Diff");
        allOutputs.export(kb);
    }
    
    protected void doTestMapping() throws Exception {
        doTestMappingO2A("TrivialOntology",1000);
        doTestMappingO2A("wine1",1000);
        doTestMappingO2A("wine",100);
        doTestMappingO2A("SUMO",1);
        if(doCyc) doTestMappingO2A("cyc-2004-06-04",1);
        doTestMappingA2O("historical_architecture",100);
        doTestMappingA2O("cipher-crm",100);
    }
    
    protected void doTestMappingA2O(String which, int count) throws Exception {
        KnowledgeBase kb = null;
        int sum = 0;
        for(int i=0;i<count;i++) {
            parametersFS = new Properties();
            parametersFS.put("source","file:testdata/"+which+".xml");
            parametersFS.put("system-concepts","false");
            kb = new cz.cvut.km.io.Apollo().loadKB(parametersFS);
            cz.cvut.km.io.OwlRdfsJena owl = new cz.cvut.km.io.OwlRdfsJena();
            
            parameters = new Properties();
            parameters.put("filename","disoutApollo2Owl"+which+"Uninf.owl");
            parameters.put("namespace","http://www.cvut.cz/apollo#");
            start = System.currentTimeMillis();
            owl.classifyKB(kb);
            end = System.currentTimeMillis();
            sum += end-start;
        }
        benchmark("Apollo2OwlMapping", "mapping engine with OWL rules "+which,kb,"--not-saved--",sum/count);
    }
    
    protected void doTestMappingO2A(String which, int count) throws Exception {
        KnowledgeBase kb = null;
        int sum = 0;
        for(int i=0;i<count;i++) {
            parametersFS = new Properties();
            parametersFS.put("source","file:testdata/"+which+".owl");
            parametersFS.put("system-concepts","false");
            parametersFS.put("nameparsing","#"); // use either # (parts/of/namespace#concept) or lastpart (parts/of/namespace/concept)
            kb = new cz.cvut.km.io.OwlRdfsJena().loadKB(parametersFS);
            
            cz.cvut.km.io.Apollo apollo = new cz.cvut.km.io.Apollo();
            parameters = new Properties();
            parameters.put("filename","disoutOwl2Apollo"+which+"Uninf.xml");
            parameters.put("prefix","gof_");
            start = System.currentTimeMillis();
            apollo.classifyKB(kb);
            //saveKB(kb, parameters);
            end = System.currentTimeMillis();
            sum += end-start;
        }
        benchmark("Owl2ApolloMapping", "mapping engine with Apollo rules "+which,kb,"--not-saved--",sum/count);
        
    }
    
}
