/*
 * Owl2Apollo.java
 *
 * Created on 27. ervenec 2004, 22:43
 */

package cz.cvut.km.trans;

import cz.cvut.km.model.*;
import cz.cvut.km.model.iterators.*;
import cz.cvut.km.io.*;
import cz.cvut.km.utils.*;
import java.util.*;


/**
 *
 * @author  jzak
 */
public class Owl2Apollo implements TransInterface {
    
    /** Creates a new instance of Owl2Apollo */
    public Owl2Apollo() {
    }
    
    public void transform(cz.cvut.km.model.KnowledgeBase kb){
        
        // transform union and intersection classes
        convertExpressions(kb, OwlRdfsJena.cIntersectionOf);
        convertExpressions(kb, OwlRdfsJena.cUnionOf);
        
        // transform types of concepts
        Vector restricts = new Vector();
        Vector listMembers = new Vector();
        ConceptIterator ci = kb.getConcepts();
        while(ci.hasNext()){
            Concept concept = ci.nextConcept();
            if(OwlJena.TYPE_OWL_CLASS.equals(concept.getProperty(OwlJena.OWLTYPE))){
                concept.setProperty(Apollo.APPTYPE, Apollo.TYPE_CLASS);
            } else if (OwlJena.TYPE_DATATYPE_PROPERTY.equals(concept.getProperty(OwlJena.OWLTYPE))){
                concept.setProperty(Apollo.APPTYPE, Apollo.TYPE_PROPERTY);
            } else if (OwlJena.TYPE_OBJECT_PROPERTY.equals(concept.getProperty(OwlJena.OWLTYPE))){
                concept.setProperty(Apollo.APPTYPE, Apollo.TYPE_PROPERTY);
            } else if (OwlJena.TYPE_FUNCTIONAL_PROPERTY.equals(concept.getProperty(OwlJena.OWLTYPE))){
                concept.setProperty(Apollo.APPTYPE, Apollo.TYPE_PROPERTY);
            } else if (OwlJena.TYPE_TRANSITIVE_PROPERTY.equals(concept.getProperty(OwlJena.OWLTYPE))){
                concept.setProperty(Apollo.APPTYPE, Apollo.TYPE_PROPERTY);
            } else if (OwlJena.TYPE_RESOURCE.equals(concept.getProperty(OwlJena.OWLTYPE))){
                concept.setProperty(Apollo.APPTYPE, Apollo.TYPE_CLASS);
                // TODO: not always true, handle properly
            } else if (OwlJena.TYPE_RDFS_CLASS.equals(concept.getProperty(OwlJena.OWLTYPE))){
                concept.setProperty(Apollo.APPTYPE, Apollo.TYPE_CLASS);
            } else if (OwlJena.TYPE_PROPERTY.equals(concept.getProperty(OwlJena.OWLTYPE))){
                concept.setProperty(Apollo.APPTYPE, Apollo.TYPE_PROPERTY);
            } else if (OwlJena.TYPE_ASSIGNMENT.equals(concept.getProperty(OwlJena.OWLTYPE))){
                concept.setProperty(Apollo.APPTYPE, Apollo.TYPE_ASSIGNMENT);
            } else if (OwlJena.TYPE_DATATYPE.equals(concept.getProperty(OwlJena.OWLTYPE))){
                concept.setProperty(Apollo.APPTYPE, Apollo.TYPE_CLASS);
                // TODO: not strictly true, handle properly acc. to Apollo
            } else if (OwlJena.TYPE_LITERAL.equals(concept.getProperty(OwlJena.OWLTYPE))){
                concept.setProperty(Apollo.APPTYPE, Apollo.TYPE_INSTANCE);
                // TODO: not strictly true, handle properly acc. to Apollo
            } else if (OwlJena.TYPE_OWL_INSTANCE.equals(concept.getProperty(OwlJena.OWLTYPE))){
                concept.setProperty(Apollo.APPTYPE, Apollo.TYPE_INSTANCE);
            } else if (OwlJena.TYPE_RDFS_INSTANCE.equals(concept.getProperty(OwlJena.OWLTYPE))){
                concept.setProperty(Apollo.APPTYPE, Apollo.TYPE_INSTANCE);
            } else if(OwlJena.TYPE_RESTRICTION.equals(concept.getProperty(OwlJena.OWLTYPE))){
                restricts.addElement(concept);
            } else if (OwlJena.TYPE_LIST_MEMBER.equals(concept.getProperty(OwlJena.OWLTYPE))){
                listMembers.addElement(concept);
            }
        }
        convertRestrictions(kb, restricts);
        convertOneOf(kb, listMembers);
    }
    
    private void convertExpressions(cz.cvut.km.model.KnowledgeBase kb, Concept type){
        Iterator it = kb.getRelationsTo(type).iterator();
        while(it.hasNext()){
            Relation rel = (Relation)it.next();
            if(rel instanceof InstanceOf){
                Concept assign = rel.getSource();
                Iterator itCls = kb.getRelationsFrom(assign).iterator();
                Concept node = null;
                Concept cls = null;
                Vector classes = new Vector();
                while(itCls.hasNext()){
                    Relation r = (Relation)itCls.next();
                    if(r instanceof PropertyOf){
                        cls = r.getTarget();
                    } else if (r instanceof HasValue){
                        Concept acls = r.getTarget();
                        classes.addElement(acls);
                    }
                }
                
                // add classes forming the expression
                itCls = classes.iterator();
                while( cls != null && itCls.hasNext()){
                    Concept acls = (Concept)itCls.next();
                    if(type.equals(OwlRdfsJena.cUnionOf))
                        kb.addRelation(new SubclassOf(acls, cls));
                    else if(type.equals(OwlRdfsJena.cIntersectionOf))
                        kb.addRelation(new SubclassOf(cls, acls));
                }
 
                kb.removeConceptWithRelations(assign);
            }
        }
    }
   
 
    
    private void convertRestrictions(cz.cvut.km.model.KnowledgeBase kb, Vector restrictions){
        Concept sub = null;
        Concept prop = null;
        Vector vals = new Vector();

        for(int j = 0; j < restrictions.size(); j++ ){
            Concept res = (Concept)restrictions.elementAt(j);
            Iterator it = kb.getRelationsTo(res).iterator();
            while(it.hasNext()){
                Relation rel = (Relation)it.next();
                if(rel instanceof SubclassOf){
                    sub = rel.getSource();
                } else if(rel instanceof PropertyOf){
                    Concept assign = rel.getSource();
                    if(kb.getInstanceOf(assign) != null 
                            && kb.getInstanceOf(assign).equals(OwlRdfsJena.cOnProperty)){
                        Iterator itVals = kb.getRelationsFrom(assign).iterator();
                        while(itVals.hasNext()){
                            Relation has = (Relation)itVals.next();
                            if(has instanceof HasValue){
                                prop = has.getTarget();
                            }
                        }
                    // add hasValue or allVAlues from    
                    } else if(kb.getInstanceOf(assign) != null
                            && (kb.getInstanceOf(assign).equals(OwlRdfsJena.cAllValuesFrom)
                                || kb.getInstanceOf(assign).equals(OwlRdfsJena.cHasValue))){
                        Iterator itVals = kb.getRelationsFrom(assign).iterator();
                        while(itVals.hasNext()){
                            Relation has = (Relation)itVals.next();
                            if(has instanceof HasValue){
                                vals.addElement(has.getTarget());
                            }
                        }
                    }
                    kb.removeConceptWithRelations(assign);
                }
            }
            kb.removeConceptWithRelations(res);
            
            // create representation of restriction in Apollo-acceptable form
            Concept assign = new Concept(prop.getNameSpace(), "ASSIGN"+"-"+sub.getName()+"-"+prop.getName());
            assign.setProperty(OwlJena.OWLTYPE, OwlJena.TYPE_ASSIGNMENT);
            assign.setProperty(Apollo.APPTYPE, Apollo.TYPE_ASSIGNMENT);
            assign.setCaption(prop.getName());
            kb.addConcept(assign);
            kb.addRelation(new InstanceOf(assign, prop));
            kb.addRelation(new PropertyOf(assign,sub));
            for(int i = 0; i < vals.size(); i++){
                Concept val = (Concept)vals.elementAt(i);
                kb.addRelation(new HasValue(assign, val));
            }
            vals.clear();
        }
    }
    
    private void convertOneOf(cz.cvut.km.model.KnowledgeBase kb, Vector list){
        for(int i = 0; i < list.size(); i++ ){
            Concept member = (Concept)list.elementAt(i);
            Iterator itRelsTo = kb.getRelationsTo(member).iterator();
            while( itRelsTo.hasNext() ){
                Relation relTo = (Relation)itRelsTo.next();
                if( relTo instanceof HasValue ){
                    Concept assign = (Concept)relTo.getSource();
                    Iterator itRelsFrom = kb.getRelationsFrom(assign).iterator();
                    while( itRelsFrom.hasNext() ){
                        Relation relFrom = (Relation)itRelsFrom.next();
                        if( relFrom instanceof PropertyOf ){
                            Concept itsClass = (Concept)relFrom.getTarget();
                            kb.addRelation(new InstanceOf(member, itsClass));
                        }
                    }
                }
            }
        }
    }
}
