/*
 * SumatraModulCreator.java
 *
 * Created on September 7, 2004, 5:43 PM
 */

package cz.cvut.km.utils;

import java.io.File;
import java.io.PrintWriter;
import java.io.FileWriter;
import java.io.IOException;
import java.io.FilenameFilter;
import java.util.Vector;
import java.util.Iterator;
import java.text.SimpleDateFormat;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import javax.xml.parsers.*;
import javax.xml.transform.*;
import javax.xml.transform.dom.*;
import javax.xml.transform.stream.*;

/**
 * SumatraModulCreator class creates SumatraTT modules from GOF I/O modules.
 * @author  Petr Aubrecht
 */
public class SumatraModulCreator {
    protected static final String OUTPUTPATH = "SumatraTTModules";
    // constants for kind of module - input, output or transformation
    /** input module
     */
    protected static final String KIND_FROM = "From";
    /** output module
     */
    protected static final String KIND_TO = "To";
    /** transformation module
     */
    protected static final String KIND_TRANS = "Proc";
    /** a list of modules made (for ant script for automatic build)
     */
    protected static Vector modules = new Vector();
    /** a number of modules created
     */
    protected static int moduleCount = 0;
    
    /** Creates a new instance of SumatraModulCreator */
    public SumatraModulCreator() {
    }
    
    /**
     * @param args the command line arguments
     */
    public static void main(String[] args) {
        try {
            System.out.println("--== SumatraModuleCreator ==--");
            moduleCount = 0;
            //System.out.println("Processing dir "+(new File(".").getAbsolutePath()));
            modules.clear();
            
            File iodir = new File("src/cz/cvut/km/io");
            File[] iolist = iodir.listFiles(new FilenameFilter() {
                public boolean accept(File dir, String name) {
                    return (!name.equals("IoInterface.java")) && name.endsWith(".java");
                }
            });
            if(iolist==null)
                iolist = new File[0];
            
            for(int i=0;i<iolist.length;i++) {
                File f = iolist[i];
                String fname = f.getName();
                String moduleName = fname.substring(0,fname.indexOf("."));
                System.out.println("===== "+moduleName);
                
                try {
                    Class cls = Class.forName("cz.cvut.km.io."+moduleName);
                    cz.cvut.km.io.IoInterface ioI = (cz.cvut.km.io.IoInterface)cls.newInstance();
                    if(ioI.supports(cz.cvut.km.io.IoInterface.SUPPORTS_LOAD)) {
                        createSumatraTTModule(moduleName,KIND_FROM);
                    }
                    if(ioI.supports(cz.cvut.km.io.IoInterface.SUPPORTS_SAVE)) {
                        createSumatraTTModule(moduleName,KIND_TO);
                    }
                } catch(Exception e) {
                    System.out.println("... failed");
                }
            }
            
            System.out.println("===== Union");
            createSumatraTTModule("Union",KIND_TRANS);
            System.out.println("===== Diff");
            createSumatraTTModule("Diff",KIND_TRANS);
            
            System.out.println("Creating Ant script...");
            createAntScript();
            System.out.println("All done -- "+moduleCount+" modules.");
        } catch(Exception e) {
            e.printStackTrace();
        }
    }
    
    /** create the Ant script buildGOF.xml for easy build of GOF modules
     */
    protected static void createAntScript() throws IOException {
        PrintWriter pw = new PrintWriter(new FileWriter(OUTPUTPATH+"/buildGOF.xml"));
        pw.println("<?xml version=\"1.0\" encoding=\"UTF-8\"?>");
        pw.println("<project basedir=\".\" default=\"all\" name=\"SumatraTT2\">");
        pw.println("    <description>Script for building all modules created from GOF gates</description>");
        pw.println("    <target name=\"init\">");
        pw.println("        <property name=\"appname\" value=\"SumatraTT2\"/>");
        pw.println("    </target>");
        pw.println();
        pw.println("    <target name=\"all\" description=\"Create jar files for all modules\"");
        pw.println("        depends=\"init,compile,Modules-KM\" />");
        pw.println();
        pw.println("    <target name=\"compile\" >");
        pw.println("        <ant target=\"compile\" antfile=\"build.xml\" />");
        pw.println("    </target>");
        pw.println();
        pw.println("    <target name=\"Modules-KM\">");
        for(Iterator it = modules.iterator();it.hasNext();) {
            String mName = it.next().toString(); // name of the processed module
            pw.println("        <ant target=\"Modules-KM-"+mName+"\" antfile=\"buildGOF.xml\" />");
        }
        //pw.println("        <copy file=\"modulessrc/cz/cvut/sumatra2/modules/KM/Core.png\" todir=\"modules/\" />");
        pw.println("    </target>");
        pw.println();
        for(Iterator it = modules.iterator();it.hasNext();) {
            String mName = it.next().toString(); // name of the processed module
            pw.println("    <!-- module KM/"+mName+" -->");
            pw.println("    <target depends=\"init,compile\" name=\"Modules-KM-"+mName+"\">");
            pw.println("        <property name=\"path\" value=\"KM\"/>");
            pw.println("        <property name=\"name\" value=\""+mName+"\" />");
            pw.println("        <property name=\"czpath\" value=\"cz/cvut/sumatra2/modules/${path}/${name}\"/>");
            pw.println("        <mkdir dir=\"modules/${path}\" />");
            pw.println("        <jar compress=\"true\" manifest=\"modulessrc/${czpath}/MANIFEST.MF\" jarfile=\"modules/${path}/${name}.jar\">");
            pw.println("            <!-- include compiled classes -->");
            pw.println("            <fileset dir=\"classes\" casesensitive=\"yes\">");
            pw.println("                <include name=\"${czpath}/**\"/>");
            pw.println("            </fileset>");
            pw.println("            <!-- include accompanying files (xsum, png, etc.) -->");
            pw.println("            <fileset dir=\"modulessrc\" casesensitive=\"yes\">");
            pw.println("                <include name=\"${czpath}/**\"/>");
            pw.println("                <exclude name=\"**/*.java\"/>");
            pw.println("                <exclude name=\"**/*.form\"/>");
            pw.println("                <exclude name=\"apidoc\"/>");
            pw.println("            </fileset>");
            pw.println("        </jar>");
            pw.println("    </target>");
            pw.println();
        }
        pw.println("</project>");
        pw.close();
    }
    
    /** create the whole SumatraTT module for GOF gate 'name'
     * @param name name of the gate (Apollo)
     * @param direction one of the constants KIND_FROM, KIND_TO, KIND_TRANS
     */
    protected static void createSumatraTTModule(String name, String direction) throws ParserConfigurationException, TransformerConfigurationException, TransformerException, IOException {
        String moduleName = direction+name;
        System.out.println("Processing module "+moduleName);
        File path = new File(OUTPUTPATH+"/modulessrc/cz/cvut/sumatra2/modules/KM/"+moduleName);
        path.mkdirs(); // ensure the directory exists
        createManifest(path, moduleName);
        createXmlFile(path, moduleName, direction);
        createProcessingJava(path, name, moduleName, direction);
        modules.add(moduleName);
        moduleCount++;
    }
    
    /** create the MANIFEST.MF file for jar creation
     * @param path path to save the module files
     * @param name name of the module (e.g. FromApollo)
     */
    protected static void createManifest(File path, String name) throws IOException {
        File mfFile = new File(path,"MANIFEST.MF");
        PrintWriter pw = new PrintWriter(new FileWriter(mfFile));
        pw.println("Manifest-Version: 1.0");
        pw.println("Created-By: SumatraModuleCreator");
        pw.println("xsum: cz/cvut/sumatra2/modules/KM/"+name+"/"+name+".xsum");
        pw.close();
    }
    
    /** create the xsum file describing the new SumatraTT module
     * @param path path to save the module files
     * @param name name of the module (e.g. FromApollo)
     * @param direction one of the constants KIND_FROM, KIND_TO, KIND_TRANS
     */
    protected static void createXmlFile(File path, String name, String direction) throws ParserConfigurationException, TransformerConfigurationException, TransformerException, IOException {
        File xmlFile = new File(path,name+".xsum");
        Document document = DocumentBuilderFactory.newInstance().newDocumentBuilder().newDocument();
        Element root = document.createElement("Template");
        addParam(root,"Name","A transformation name",name);
        addParam(root,"Icon","An icon of transformation","cz/cvut/sumatra2/modules/KM/"+name+"/"+name+".png");
        addParam(root,"ToolBarIcon","An icon of transformation in tool bar","cz/cvut/sumatra2/modules/KM/"+name+"/"+name+".png");
        addParam(root,"Inputs number","A number of inputs of datasource",direction==KIND_FROM?"0":(direction==KIND_TO?"1":"2"));
        addParam(root,"Outputs number","A number of outputs of datasource",direction==KIND_TO?"0":"1");
        addParam(root,"JavaProcessingClass", "A java class for processing","cz.cvut.sumatra2.modules.KM."+name+"."+name+"Processing");
        //addParam(root,"JavaGUIClass","A java class for GUI","cz.cvut.sumatra2.modules.KM."+name+"."+name+"GUI");
        if(direction==KIND_FROM) {
            addParam(root,"JavaGUIClass","A java class for GUI","cz.cvut.km.utils.SumatraFromGUI");
        } else if(direction==KIND_TO) {
            addParam(root,"JavaGUIClass","A java class for GUI","cz.cvut.km.utils.SumatraToGUI");
        } else if(direction==KIND_TRANS) {
            // no GUI
        }
        document.appendChild(root);
        
        Transformer transformer = TransformerFactory.newInstance().newTransformer();
        transformer.setOutputProperty(OutputKeys.INDENT,"yes");
        DOMSource source = new DOMSource(root);
        StreamResult result = new StreamResult(new FileWriter(xmlFile));
        transformer.transform(source, result);
    }
    
    /** add Param element to the Template element
     */
    protected static void addParam(Element root, String name, String comment, String text) {
        Document document = root.getOwnerDocument();
        Element param;
        param = document.createElement("Param");
        param.setAttribute("name",name);
        param.setAttribute("comment",comment);
        param.appendChild(document.createTextNode(text));
        root.appendChild(param);
    }
    
    /** create the processing class for the new module
     * @param direction one of the constants KIND_FROM, KIND_TO, KIND_TRANS
     */
    protected static void createProcessingJava(File path, String gofName, String sttName, String direction) throws IOException {
        String procName = sttName+"Processing";
        File procFile = new File(path,procName+".java");
        PrintWriter pw = new PrintWriter(new FileWriter(procFile));
        pw.println("/**");
        pw.println(" * "+procName);
        pw.println(" * Automatically created from SumatraModuleCreator, a part of GOF framework");
        SimpleDateFormat df = new SimpleDateFormat("dd. MM. yyyy HH:mm:ss");
        pw.println(" * Created on "+df.format(new java.util.Date()));
        pw.println(" */");
        pw.println();
        pw.println("package cz.cvut.sumatra2.modules.KM."+sttName+";");
        pw.println("");
        pw.println("import cz.cvut.sumatra2.simulation.*;");
        pw.println("import cz.cvut.sumatra2.ioUtils.*;");
        pw.println("import cz.cvut.sumatra2.ioUtils.log.Log;");
        pw.println("import cz.cvut.km.model.*;");
        pw.println("import org.w3c.dom.Element;");
        pw.println();
        pw.println("/**");
        pw.println(" * "+procName);
        pw.println(" * @author GOF framework");
        pw.println(" */");
        if(direction==KIND_FROM) {
            pw.println("public class "+procName+" extends ThreadedDataProcessing {");
        } else if(direction==KIND_TO || direction==KIND_TRANS) {
            pw.println("public class "+procName+" extends BasicDataProcessing {");
        }
        pw.println("    // local variables");
        pw.println("    String filename = null; // file name");
        pw.println("    String system_concepts = \"false\"; // create system concepts?");
        pw.println("    KnowledgeBase kb1 = null; // for Proc");
        //pw.println("    SumFieldXml field = null; // xml field used for communication");
        pw.println();
        pw.println("    /** Create a new "+procName+" */");
        pw.println("    public "+procName+"() {");
        pw.println("        super(); // always perform super() in constructor");
        pw.println("        defaultCopyMetaData = false; // stop sending 'eof' command");
        pw.println("    }");
        pw.println("");
        if(direction==KIND_FROM || direction==KIND_TO) {
            pw.println("    /** Provide initialization code");
            pw.println("     */");
            pw.println("    public void initialize() {");
            pw.println("        filename = paramVector.getParamValue(\"filename\");");
            pw.println("        if(filename==null) {");
            pw.println("            Log.logger.severe(\""+procName+".initialize(): Required parameter 'filename'\");");
            pw.println("            System.out.println();");
            pw.println("            return;");
            pw.println("        }");
            pw.println("        system_concepts = paramVector.getParamValue(\"system-concepts\");");
            pw.println("        System.out.println(\""+procName+": Working with file \"+filename);");
            pw.println();
            pw.println("        try {");
            pw.println("            super.initialize();");
            pw.println("        } catch(Exception e) {");
            pw.println("            e.printStackTrace();");
            pw.println("            Log.logger.severe(e.getMessage());");
            pw.println("        }");
            pw.println("    }");
            pw.println();
        }
        if(direction==KIND_FROM) {
            pw.println("    public void processData() {");
            pw.println("        // enter code for pocessing data");
            pw.println("        try {");
            pw.println("            java.util.Properties parametersFS = new java.util.Properties();");
            pw.println("            parametersFS.put(\"source\",filename);");
            pw.println("            parametersFS.put(\"system-concepts\",system_concepts);");
            pw.println("            KnowledgeBase kb = new cz.cvut.km.io."+gofName+"().loadKB(parametersFS);");
            pw.println("            Element cmdInfo = XMLDoc.createSimpleCommand(\"GOF-KB\");");
            pw.println("            String idOnto = sim.createSharedObject(kb);");
            pw.println("            Element cmdInfoId = cmdInfo.getOwnerDocument().createElement(\"KB\");");
            pw.println("            cmdInfoId.setAttribute(\"id\", idOnto);");
            pw.println("            cmdInfo.appendChild(cmdInfoId);");
            pw.println("            output.processMetaData(cmdInfo);");
            pw.println("        } catch(Exception e) {");
            pw.println("            e.printStackTrace();");
            pw.println("            Log.logger.severe(e.getMessage());");
            pw.println("        }");
            pw.println("        stop();");
            pw.println("    }");
        } else if(direction==KIND_TO) {
            pw.println("    public boolean processMetaData(DataTransfer from, Element metadata) {");
            pw.println("        if(!metadata.getNodeName().equals(\"GOF-KB\")) {");
            pw.println("            return super.processMetaData(from, metadata);");
            pw.println("        }");
            pw.println("        // enter code for pocessing data");
            pw.println("        try {");
            pw.println("            String id = metadata.getChildNodes().item(0).getAttributes().getNamedItem(\"id\").getNodeValue();");
            pw.println("            KnowledgeBase kb = (KnowledgeBase)sim.getSharedObject(id);");
            pw.println("            cz.cvut.km.io."+gofName+" ioModule = new cz.cvut.km.io."+gofName+"();");
            pw.println("            java.util.Properties parameters = new java.util.Properties();");
            pw.println("            parameters.put(\"filename\",filename);");
            pw.println("            System.out.println(\"Saving to...\");");
            pw.println("            ioModule.saveKB(kb, parameters);");
            pw.println("            System.out.println(\"...done\");");
            pw.println("            return true;");
            pw.println("        } catch(Exception e) {");
            pw.println("            Log.logger.severe(getClass().getName()+\":processMetaData throws exception: \"+e);");
            pw.println("            e.printStackTrace();");
            pw.println("            return false;");
            pw.println("        }");
            pw.println("    }");
        } else if(direction==KIND_TRANS) {
            pw.println("    public boolean processMetaData(DataTransfer from, Element metadata) {");
            pw.println("        if(metadata.getNodeName().equals(\"eof\")) {");
            pw.println("            return true;");
            pw.println("        }");
            pw.println("        if(!metadata.getNodeName().equals(\"GOF-KB\")) {");
            pw.println("            return super.processMetaData(from, metadata);");
            pw.println("        }");
            pw.println("        // enter code for pocessing data");
            pw.println("        try {");
            pw.println("            String id = metadata.getChildNodes().item(0).getAttributes().getNamedItem(\"id\").getNodeValue();");
            pw.println("            KnowledgeBase kb = (KnowledgeBase)sim.getSharedObject(id);");
            pw.println("            if(kb1==null) {");
            pw.println("                kb1 = kb;");
            pw.println("                System.out.println/*Log.logger.fine*/(\""+gofName+": first onlo loaded\");");
            pw.println("            } else {");
            pw.println("                System.out.println/*Log.logger.fine*/(\""+gofName+": second onlo loaded, processing "+gofName+"\");");
            pw.println("                KnowledgeBase ret = cz.cvut.km.oper."+gofName+".process"+gofName+"(kb1, kb);");
            pw.println("                Element cmdInfo = XMLDoc.createSimpleCommand(\"GOF-KB\");");
            pw.println("                String idOnto = sim.createSharedObject(ret);");
            pw.println("                Element cmdInfoId = cmdInfo.getOwnerDocument().createElement(\"KB\");");
            pw.println("                cmdInfoId.setAttribute(\"id\", idOnto);");
            pw.println("                cmdInfo.appendChild(cmdInfoId);");
            pw.println("                output.processMetaData(cmdInfo);");
            pw.println("                kb1 = null; // reset");
            pw.println("                output.processMetaData(XMLDoc.createSimpleCommand(\"eof\")); // send eof to the output");
            pw.println("                stop();");
            pw.println("            }");
            pw.println("            return true;");
            pw.println("        } catch(Exception e) {");
            pw.println("            Log.logger.severe(getClass().getName()+\":processMetaData throws exception: \"+e);");
            pw.println("            e.printStackTrace();");
            pw.println("            return false;");
            pw.println("        }");
            pw.println("    }");
        }
        pw.println("}");
        pw.println();
        
        
        pw.close();
    }
    
}
